%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for graphing 1 or 2 free beta         %
%          parameters along with the value of the                %
%          log-likelihood function                               %
%                                                                %
%          Function: betagraphdlg                                %
%                                                                %
%          Input:                                                %
%                selector: case string                           %
%                cm: structure with model info                   %
%                                                                %
%          Output:                                               %
%                handle: handle on figure                        %
%                                                                %
%          This version:        June 16, 2006                    %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function myhandle = betagraphdlg(selector,cm)

if strcmp(selector,'init')==1;
   bm = cm;
   %
   % reorder the columns of bm.R_orth such that the free parameters
   % of beta are ordered based on their index number, e.g.,
   % b(1,4) comes before b(2,2)
   %
   [Rort,myindex] = max(bm.R_orth);
   [thisindex,newindex] = sort(myindex);
   nR_orth = [];
   for j=1:size(newindex,2);
      nR_orth = [nR_orth bm.R_orth(:,newindex(j))];
   end;
   bm.R_orth = nR_orth;
   if size(bm.R_orth,2)>1;
      bm.showy = 1;
      mytypestr = '1 beta parameter|2 beta parameters';
   else;
      mytypestr = '1 beta parameter';
      bm.showy = 0;
   end;
   myexit = 0;
   bm.myrunning = 0;
   bm.mybetastr = '';
   for j=1:size(bm.R_orth,2);
      for i=1:size(bm.R_orth,1);
         thisvec = zeros((bm.n+bm.q1+bm.dcoint)*bm.r,1);
         if abs(bm.R_orth(i,j))>0;
            thisvec(i,1) = 1;
            thismat = vecinv(thisvec,bm.n+bm.q1+bm.dcoint);
            mycheck = 0;
            k = 1;
            while k<=bm.r;
               l = 1;
               while l<=(bm.n+bm.q1+bm.dcoint);
                  if thismat(l,k)==1;
                     elem = [l k];
                     l = bm.n+bm.q1+bm.dcoint+1;
                     mycheck = 1;
                  else;
                     l = l+1;
                  end;
               end;
               if mycheck==1;
                  k=bm.r+1;
               else;
                  k = k+1;
               end;
            end;
            bm.mybetastr = strvcat(bm.mybetastr,['beta(' num2str(elem(1,2)) ',' num2str(elem(1,1)) ') - parameter ' num2str(j)]);
         end;
      end;
   end;
   curr_ini = find_inisettings;
   if isnumeric(curr_ini.external)==0;
      curr_ini.external = 0;
   end;
   bm.realscrsz = get(0,'ScreenSize');
else;
   myexit = 1;
   betagraphgui = get(0,'CurrentFigure');
   graphbeta = get(betagraphgui,'UserData');
   curr_ini = get(graphbeta.type,'UserData');
   bm = curr_ini.models;
end;

switch selector;
   case 'init';
      betagraphgui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','betagraphdlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(bm.realscrsz(3)-420)/2 (bm.realscrsz(4)-300)/2 420 300], ...
         'Tag','FigParent', ...
         'Resize','off', ...
         'Name','Log-likelihood and Beta', ...
         'NumberTitle','off', ...
         'Visible','off', ...
         'CloseRequestFcn','betagraphdlg exit', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(betagraphgui,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 418 290],'Select Parameters',90,[0.5 1]);

      imageaxes = axes('Units','pixels', ...
         'Box','on', ...
         'TickLength',[0 0], ...
         'CameraUpVector',[0 1 0], ...
         'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 245 39 39], ...
         'XTickLabelMode','manual', ...
         'YTickLabelMode','manual', ...
         'Tag','AxisLogo', ...
         'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ZColor',get(0,'defaultuicontrolbackgroundcolor'));

      mycons = getstoredimages;
      if curr_ini.use_toolbar_bgcolor==1;
         bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
         bgcolor(1) = max(0,bgcolor(1));
         bgcolor(2) = max(0,bgcolor(2));
         bgcolor(3) = max(0,bgcolor(3));
      else;
         bgcolor = get(0,'defaultuicontrolbackgroundcolor');
      end;
      %
      image('CData',mirrorimage(maketrans(mycons.betamatlab_image,[160 160 164],bgcolor)),'Clipping','off','Parent',imageaxes);

      %
      % type of graph (2-D or 3-D)
      %

      graphbeta.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 215 130 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Select figure type:', ...
         'HorizontalAlignment','left', ...
         'Tag','FigureTypeText');

      graphbeta.type = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[212.5 220 191.5 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',mytypestr, ...
         'HorizontalAlignment','center', ...
         'Callback','betagraphdlg type', ...
         'Tag','FigureTypePopup',...
         'Value',1);

      %
      % Select beta parameter for x-axis
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 185 150 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Beta parameter on X axis:', ...
         'HorizontalAlignment','left', ...
         'Tag','XAxisText');

      graphbeta.xaxis = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[212.5 190 191.5 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',bm.mybetastr, ...
         'HorizontalAlignment','center', ...
         'Tag','XAxisPopup',...
         'Value',1);

      %
      % Min, max and gridlength for X axis
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 130 60.5 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','min X', ...
         'HorizontalAlignment','center', ...
         'Tag','XMinText');

      graphbeta.minx = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[16 110 60.5 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String','x-0.5|x-1.0|x-1.5|x-2.0|x-2.5|x-3.0|x-3.5|x-4.0|x-4.5|x-5.0|x-6.0|x-7.0|x-8.0|x-9.0|x-10.0', ...
         'HorizontalAlignment','center', ...
         'Value',2, ...
         'Tag','XMinPopup');

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[81.5 130 60.5 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','max X', ...
         'HorizontalAlignment','center', ...
         'Tag','XMinText');

      graphbeta.maxx = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[81.5 110 60.5 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String','x+0.5|x+1.0|x+1.5|x+2.0|x+2.5|x+3.0|x+3.5|x+4.0|x+4.5|x+5.0|x+6.0|x+7.0|x+8.0|x+9.0|x+10.0', ...
         'HorizontalAlignment','center', ...
         'Value',2, ...
         'Tag','XMaxPopup');

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[147 130 60.5 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','X grid size', ...
         'HorizontalAlignment','center', ...
         'Tag','XGridText');

      graphbeta.gridx = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[147 110 60.5 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String','0.001|0.005|0.01|0.02|0.025|0.04|0.05|0.10|0.125|0.20', ...
         'HorizontalAlignment','center', ...
         'Value',3, ...
         'Tag','XGridPopup');

      %
      % Select beta parameter for y-axis
      %

      graphbeta.yaxistext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 155 150 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'Enable','off', ...
         'String','Beta parameter on Y axis:', ...
         'HorizontalAlignment','left', ...
         'Tag','YAxisText');

      graphbeta.yaxis = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[212.5 160 191.5 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',bm.mybetastr, ...
         'Enable','off', ...
         'HorizontalAlignment','center', ...
         'Tag','YAxisPopup',...
         'Value',1);

      %
      % Min, max and gridlength for Y axis
      %

      graphbeta.minytext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[212.5 130 60.5 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','min Y', ...
         'Enable','off', ...
         'HorizontalAlignment','center', ...
         'Tag','YMinText');

      graphbeta.miny = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[212.5 110 60.5 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String','y-0.5|y-1.0|y-1.5|y-2.0|y-2.5|y-3.0|y-3.5|y-4.0|y-4.5|y-5.0|y-6.0|y-7.0|y-8.0|y-9.0|y-10.0', ...
         'HorizontalAlignment','center', ...
         'Value',2, ...
         'Enable','off', ...
         'Tag','YMinPopup');

      graphbeta.maxytext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[278 130 60.5 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','max Y', ...
         'Enable','off', ...
         'HorizontalAlignment','center', ...
         'Tag','YMaxText');

      graphbeta.maxy = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[278 110 60.5 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String','y+0.5|y+1.0|y+1.5|y+2.0|y+2.5|y+3.0|y+3.5|y+4.0|y+4.5|y+5.0|y+6.0|y+7.0|y+8.0|y+9.0|y+10.0', ...
         'HorizontalAlignment','center', ...
         'Value',2, ...
         'Enable','off', ...
         'Tag','YMaxPopup');

      graphbeta.gridytext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[343.5 130 60.5 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Y grid size', ...
         'Enable','off', ...
         'HorizontalAlignment','center', ...
         'Tag','YGridText');

      graphbeta.gridy = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[343.5 110 60.5 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String','0.001|0.005|0.01|0.02|0.025|0.04|0.05|0.10|0.125|0.20', ...
         'HorizontalAlignment','center', ...
         'Value',3, ...
         'Enable','off', ...
         'Tag','YGridPopup');

      %
      % Radio buttons for selecting whether to update all other parameters
      % or keep them fixed
      %

      graphbeta.update = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
       	 'Position',[16 75 180 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','radiobutton', ...
         'Value',1, ...
         'Enable','on', ...
         'Max',1, ...
         'UserData',1, ...
         'Callback','betagraphdlg method', ...
         'HorizontalAlignment','left', ...
         'String','Reestimate all other parameters', ...
         'Tag','UpdateRadio');

      graphbeta.fixed = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[212.5 75 180 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','radiobutton', ...
         'Value',0, ...
         'UserData',0, ...
         'Enable','on', ...
         'Max',1, ...
         'Callback','betagraphdlg method', ...
         'HorizontalAlignment','left', ...
         'String','Keep all other parameters fixed', ...
         'Tag','FixedRadio');

      %
      % Check box for shading
      %

      graphbeta.shading = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 50 300 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','checkbox', ...
         'Value',0, ...
         'Enable','off', ...
         'Visible','on', ...
         'HorizontalAlignment','left', ...
         'String','Apply interpolated shading', ...
         'Tag','CheckShade');

      %
      % Check box for colorbar
      %

      graphbeta.colorbar = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[212.5 50 300 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','checkbox', ...
         'Value',0, ...
         'Enable','off', ...
         'Visible','on', ...
         'HorizontalAlignment','left', ...
         'String','Add colorbar', ...
         'Tag','CheckColorbar');

      %
      % Select an action
      %

      graphbeta.graph = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[117.5 14 90 20], ...
         'String','Show figure', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','betagraphdlg graph', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','GraphButton');

      graphbeta.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[212.5 14 90 20], ...
         'String','Close', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','betagraphdlg done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','DoneButton');

      set(betagraphgui,'UserData',graphbeta,...
         'HandleVisibility','callback');

   case 'method'
      if bm.myrunning==0;
         whichobj = gco;
         set(whichobj,'Value',1,'UserData',1);
         if whichobj==graphbeta.update;
            set(graphbeta.fixed,'Value',0,'UserData',0);
         elseif whichobj==graphbeta.fixed;
            set(graphbeta.update,'Value',0,'UserData',0);
         end;
      else;
         newupdate = get(graphbeta.update,'UserData');
         set(graphbeta.update,'Value',newupdate);
         newfixed = get(graphbeta.fixed,'UserData');
         set(graphbeta.fixed,'Value',newfixed);
      end;
   case 'graph'
      if bm.myrunning==0;
         bm.myrunning = 1;
         if bm.showy==1;
            mytype = get(graphbeta.type,'Value');
            if mytype==1;
               make3d = 0;
            else;
               xval = get(graphbeta.xaxis,'Value');
               yval = get(graphbeta.yaxis,'Value');
               if xval==yval;
                  make3d = -1;
               else;
                  make3d = 1;
               end;
            end;
         else;
            make3d = 0;
         end;
         updatevalue = get(graphbeta.update,'Value');
         if (make3d==0)&(updatevalue==1);
            %
            % Make 2-D graph
            %
            minx = get(graphbeta.minx,'Value');
            maxx = get(graphbeta.maxx,'Value');
            gridx = get(graphbeta.gridx,'Value');
            betax = get(graphbeta.xaxis,'Value');
            %
            % relative min value for X grid
            %
            if minx==1;
               xmin = 0.5;
            elseif minx==2;
               xmin = 1;
            elseif minx==3;
               xmin = 1.5;
            elseif minx==4;
               xmin = 2;
            elseif minx==5;
               xmin = 2.5;
            elseif minx==6;
               xmin = 3;
            elseif minx==7;
               xmin = 3.5;
            elseif minx==8;
               xmin = 4;
            elseif minx==9;
               xmin = 4.5;
            elseif minx==10;
               xmin = 5;
            elseif minx==11;
               xmin = 6;
            elseif minx==12;
               xmin = 7;
            elseif minx==13;
               xmin = 8;
            elseif minx==14;
               xmin = 9;
            else;
               xmin = 10;
            end;
            %
            % relative max value for X grid
            %
            if maxx==1;
               xmax = 0.5;
            elseif maxx==2;
               xmax = 1;
            elseif maxx==3;
               xmax = 1.5;
            elseif maxx==4;
               xmax = 2;
            elseif maxx==5;
               xmax = 2.5;
            elseif maxx==6;
               xmax = 3;
            elseif maxx==7;
               xmax = 3.5;
            elseif maxx==8;
               xmax = 4;
            elseif maxx==9;
               xmax = 4.5;
            elseif maxx==10;
               xmax = 5;
            elseif maxx==11;
               xmax = 6;
            elseif maxx==12;
               xmax = 7;
            elseif maxx==13;
               xmax = 8;
            elseif maxx==14;
               xmax = 9;
            else;
               xmax = 10;
            end;
            %
            % grid length value
            %
            if gridx==1;
               myxgrid = 0.001;
            elseif gridx==2;
               myxgrid = 0.005;
            elseif gridx==3;
               myxgrid = 0.01;
            elseif gridx==4;
               myxgrid = 0.02;
            elseif gridx==5;
               myxgrid = 0.025;
            elseif gridx==6;
               myxgrid = 0.04;
            elseif gridx==7;
               myxgrid = 0.05;
            elseif gridx==8;
               myxgrid = 0.1;
            elseif gridx==9;
               myxgrid = 0.125;
            else;
               myxgrid = 0.2;
            end;
            %
            % Selected beta parameter index & free parameter
            % number (the latter should be compared with the
            % free parameter number for the y axis when y axis
            % beta parameter is requested)
            %
            xbeta = strtrim(bm.mybetastr(betax,:));
            xbeta = strrep(xbeta,'beta(','');
            xbeta = strrep(xbeta,') - parameter','');
            xbeta = strrep(xbeta,',',' ');
            xbeta = mystr2num(xbeta);
            %
            % Value of selected beta parameter
            %
            betaij = bm.beta(xbeta(2),xbeta(1));
            xmin = betaij-xmin;
            xmax = betaij+xmax;
            %
            % Setting up the loop to estimate beta subject to
            % the additional restriction
            %
            thisbij = xmin;
            bijvec = [];
            lnlvec = [];
            set(betagraphgui,'Visible','off');
            drawnow;
            currbeta = ['\beta_{' num2str(xbeta(1)) ',' num2str(xbeta(2)) '}'];
            input_var.title = ['Estimating \beta for given ' currbeta];
            input_var.facecolor = curr_ini.progress_facecolor;
            input_var.startfacecolor = curr_ini.progress_startfacecolor;
            input_var.edgecolor = curr_ini.progress_edgecolor;
            input_var.bgcolor = curr_ini.progress_bgcolor;
            input_var.label = [currbeta ' ='];
            input_var.stop = 0;
            input_var.clock = 1;
            input_var.name = ['Log-likelihood and beta(' num2str(xbeta(1)) ',' num2str(xbeta(2)) ')'];
            mywait = progressdlg(0,input_var);
%            set(mywait,'Name',['Log-likelihood and beta(' num2str(xbeta(1)) ',' num2str(xbeta(2)) ')']);
            set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
            drawnow;
            abort = '';
            while thisbij<=xmax;
               %
               % get info about waitbar
               %
               abort = get(mywait,'UserData');
               if (strcmp(abort,'cancel')==1);
                  break;
               else;
                  progressdlg([(thisbij-xmin)/(xmax-xmin) thisbij]);
               end;
               if (bm.restrict(1,1)==1)&(curr_ini.reestbeta==1);
                  %
                  % Estimate beta and alpha simultaneosly
                  %
                  thisbeta = beta_2d_restrict(bm.S_00,bm.S_01,bm.S_10,bm.S_11,bm.oldbeta,bm.T,bm.dcoint,bm.modelsdir,bm.modelfile,thisbij,xbeta(2),xbeta(1));
                  thisalpha = (bm.S_01*thisbeta)/(thisbeta'*bm.S_11*thisbeta);
                  if isempty(bm.Z2)==0;
                     thisPsi = (bm.M_02/bm.M_22)-(thisalpha*thisbeta'*(bm.M_12/bm.M_22));
                     Z_3 = [bm.Z2;thisbeta'*bm.Z1];
                  else;
                     thisPsi = [];
                     Z_3 = thisbeta'*bm.Z1;
                  end;
                  file = strcat(bm.modelsdir,'G-',int2str(bm.n),'x',int2str(bm.r),'-',bm.modelfile,'.rest');
                  thisG = loaddata(file);
                  [thisRmat,thisrvec] = rmatrix(thisG,bm.n,bm.r);
                  df_alpha = size(thisG,1);
                  if isempty(bm.Z2)==0;
                     thisRmat = [zeros(df_alpha,size(bm.Z2,1)*bm.n) thisRmat];
                  end;
                  thisTheta = [thisPsi thisalpha];
                  [thistheta,thisbeta] = alpha_beta_2d_restrict(bm.Z0,bm.Z1,bm.Z2,Z_3,thisbeta,thisTheta,thisRmat,thisrvec,bm.dcoint,bm.q1,bm.modelsdir,bm.modelfile,thisbij,xbeta(2),xbeta(1));
                  %
                  if isempty(bm.Z2)==0;
                     Z_3 = [bm.Z2;thisbeta'*bm.Z1];
                  else;
                     Z_3 = thisbeta'*bm.Z1;
                  end;
                  this_eps = bm.Z0-(thistheta*Z_3);
                  thislnl = loglike(this_eps);
               else;
                  %
                  % Estimate beta under restrictions
                  %
                  thisbeta = beta_2d_restrict(bm.S_00,bm.S_01,bm.S_10,bm.S_11,bm.oldbeta,bm.T,bm.dcoint,bm.modelsdir,bm.modelfile,thisbij,xbeta(2),xbeta(1));
                  thisalpha = (bm.S_01*thisbeta)/(thisbeta'*bm.S_11*thisbeta);
                  if isempty(bm.Z2)==0;
                     Z_3 = [bm.Z2;thisbeta'*bm.Z1];
                     thisPsi = (bm.M_02/bm.M_22)-(thisalpha*thisbeta'*(bm.M_12/bm.M_22));
                     this_eps = bm.Z0-(thisalpha*thisbeta'*bm.Z1)-(thisPsi*bm.Z2);
                  else;
                     Z_3 = thisbeta'*bm.Z1;
                     thisPsi = [];
                     this_eps = bm.Z0-(thisalpha*thisbeta'*bm.Z1);
                  end;
                  thislnl = loglike(this_eps);
                  thistheta = [thisPsi thisalpha];
                  %
                  % Estimate alpha under restrictions if decided
                  %
                  if bm.restrict(1,1)==1;
                     file = strcat(bm.modelsdir,'G-',int2str(bm.n),'x',int2str(bm.r),'-',bm.modelfile,'.rest');
                     thisG = loaddata(file);
                     [thisRmat,thisrvec] = rmatrix(thisG,bm.n,bm.r);
                     df_alpha = size(thisG,1);
                     if isempty(bm.Z2)==0;
                        thisRmat = [zeros(df_alpha,size(bm.Z2,1)*bm.n) thisRmat];
                     end;
                     thisTheta = [thisPsi thisalpha];
                     [thistheta,Sigma_thistheta,thisinfo,thisabort] = linrest(bm.Z0,Z_3,thisTheta,thisRmat,thisrvec,0);
                     this_eps = bm.Z0-(thistheta*Z_3);
                     thislnl = loglike(this_eps);
                  end;
               end;
               %
               % allow for restrictions on C
               %
               if (size(bm.restrict,2)==3)&(bm.restrict(1,3)==1);
                  file = strcat(bm.modelsdir,'C-',int2str(bm.n),'x',int2str(bm.r),'-',bm.modelfile,'.rest');
                  thisC = loaddata(file);
                  [Rmat_C,rvec_C] = rmatrix(thisC,bm.n,bm.n);
                  [thistheta,thisbeta] = c_2d_restrict(bm.Z0,bm.Z1,bm.Z2,bm.restrict,thisbeta,thistheta,Rmat_C,rvec_C,bm.p,bm.pe,bm.d,bm.dcoint,bm.q0,bm.q1,bm.modelsdir,bm.modelfile,thisbij,xbeta(2),xbeta(1));
                  %
                  % remake Z_3
                  %
                  if isempty(bm.Z2)==0;
                     Z_3 = [bm.Z2;thisbeta'*bm.Z1];
                  else;
                     Z_3 = thisbeta'*bm.Z1;
                  end;
                  this_eps = bm.Z0-(thistheta*Z_3);
                  thislnl = loglike(this_eps);
               end;
               %
               % Collect the results
               %
               bijvec = [bijvec;thisbij];
               lnlvec = [lnlvec;thislnl];
               thisbij = thisbij+myxgrid;
            end;
            %
            % remove waitbar
            %
            set(mywait,'UserData','done');
            close(mywait);
            drawnow;
            if strcmp(abort,'cancel')==0;
               lnl_cbs = max(lnlvec)-(chisq_inv(bm.cbs/100,1)/2);
               crossesdown = [];
               crossesup = [];
               if (min(lnlvec)<lnl_cbs)&(max(lnlvec)>lnl_cbs);
                  for i=2:size(lnlvec);
                     if (lnlvec(i,1)>lnl_cbs)&(lnlvec(i-1)<lnl_cbs);
                        crossesup = [crossesup;i];
                     elseif (lnlvec(i,1)<lnl_cbs)&(lnlvec(i-1,1)>lnl_cbs);
                        crossesdown = [crossesdown;i];
                     end;
                  end;
               end;
               if isempty(crossesdown)==0;
                  bijdown = [];
                  for i=1:size(crossesdown,1);
                     c_l = (lnl_cbs-lnlvec(crossesdown(i)))/(lnlvec(crossesdown(i)-1)-lnlvec(crossesdown(i)));
                     bijdown = [bijdown (c_l*bijvec(crossesdown(i)-1))+((1-c_l)*bijvec(crossesdown(i)))];
                  end;
               end;
               if isempty(crossesup)==0;
                  bijup = [];
                  for i=1:size(crossesup,1);
                     c_h = (lnl_cbs-lnlvec(crossesup(i)))/(lnlvec(crossesup(i)-1)-lnlvec(crossesup(i)));
                     bijup = [bijup (c_h*bijvec(crossesup(i)-1))+((1-c_h)*bijvec(crossesup(i)))];
                  end;
               end;
            end;
            %
            % Set up 2-D graph
            %
            set(betagraphgui,'Visible','on');
            drawnow;
            drawnow;
            if strcmp(abort,'cancel')==0;
               if curr_ini.colors==1;
                  colorstr = 'b';
                  colorstr2 = 'r';
                  typestr = '-';
               else;
                  colorstr = 'k';
                  colorstr2 = [0.5 0.5 0.5];
                  typestr = '--';
               end;
               thisplot = figure('Units','pixels', ...
                  'Position',[(bm.realscrsz(3)-550)/2 (bm.realscrsz(4)-360)/2 550 360], ...
                  'Menubar','none', ...
                  'Toolbar','none', ...
                  'NumberTitle','off', ...
                  'Visible','off', ...
                  'Name',['Plot of log-likelihood and beta(' num2str(xbeta(1)) ',' num2str(xbeta(2)) ')']);
               %
               % obey centering and filling defaults
               %
               setfigureproperties(thisplot);
               %
               %
               % setting up the plot
               %
               hp = plot(bijvec,lnlvec,'-','LineWidth',1,'Color',colorstr);
               set(hp,'Tag','lnL');
               hold('on');
               plot(bijvec,lnl_cbs*ones(size(bijvec)),typestr,'LineWidth',1,'Color',colorstr2);
               setaxesfonts('Title',['Values for ' currbeta ' and log-likelihood with ' num2str(bm.cbs) '% confidence region'],'Xlabel',currbeta,'YLabel',['lnL(' currbeta ',...)']);
               set(gca,'XLim',[min(bijvec) max(bijvec)]);
               if (isempty(crossesdown)==0)|(isempty(crossesup)==0);
                  ylim = get(gca,'YLim');
                  hold('on');
                  line_x = [];
                  line_y = [];
                  my_y = [lnl_cbs;ylim(1)];
                  if isempty(crossesdown)==0;
                     line_x = [line_x ones(2,1)*bijdown];
                     line_y = [line_y my_y*ones(1,size(bijdown,2))];
                  end;
                  if isempty(crossesup)==0;
                     line_x = [line_x ones(2,1)*bijup];
                     line_y = [line_y my_y*ones(1,size(bijup,2))];
                  end;
                  plot(line_x,line_y,':','LineWidth',1,'Color',colorstr2,'Tag','2dlines-lnl');
               end;
               my_pos = 1;
               if (lnlvec(size(lnlvec,1),1))>(lnlvec(1,1));
                  my_pos = 2;
               end;
%               old_ax_units = get(gca,'Units');
%               set(gca,'Units','pixels');
%               ax_pos = get(gca,'Position');
%               set(gca,'Position',ax_pos-[0 0 0 10]);
%               set(gca,'Units',old_ax_units);
               myleg = legend_fcn('lnL','confidence',my_pos);
               setaxesfonts(myleg);
               myleg = update_legend(myleg);
               set(gca,'UserData',myleg);
               toolbar_fcn(gca);
               hold('off');
               if curr_ini.external==0;
                  set(thisplot,'Visible','on');
               else;
                  file = [bm.outputdir 'lnl-beta-' num2str(xbeta(1)) '.' num2str(xbeta(2))];
                  mstat = autosavefigure(thisplot,file);
                  if mstat==0;
                     autoshowfigure(file);
                  end;
               end;
            end;
         elseif (make3d==0)&(updatevalue==0);
            %
            % Make 2-D graph
            %
            minx = get(graphbeta.minx,'Value');
            maxx = get(graphbeta.maxx,'Value');
            gridx = get(graphbeta.gridx,'Value');
            betax = get(graphbeta.xaxis,'Value');
            %
            % relative min value for X grid
            %
            if minx==1;
               xmin = 0.5;
            elseif minx==2;
               xmin = 1;
            elseif minx==3;
               xmin = 1.5;
            elseif minx==4;
               xmin = 2;
            elseif minx==5;
               xmin = 2.5;
            elseif minx==6;
               xmin = 3;
            elseif minx==7;
               xmin = 3.5;
            elseif minx==8;
               xmin = 4;
            elseif minx==9;
               xmin = 4.5;
            elseif minx==10;
               xmin = 5;
            elseif minx==11;
               xmin = 6;
            elseif minx==12;
               xmin = 7;
            elseif minx==13;
               xmin = 8;
            elseif minx==14;
               xmin = 9;
            else;
               xmin = 10;
            end;
            %
            % relative max value for X grid
            %
            if maxx==1;
               xmax = 0.5;
            elseif maxx==2;
               xmax = 1;
            elseif maxx==3;
               xmax = 1.5;
            elseif maxx==4;
               xmax = 2;
            elseif maxx==5;
               xmax = 2.5;
            elseif maxx==6;
               xmax = 3;
            elseif maxx==7;
               xmax = 3.5;
            elseif maxx==8;
               xmax = 4;
            elseif maxx==9;
               xmax = 4.5;
            elseif maxx==10;
               xmax = 5;
            elseif maxx==11;
               xmax = 6;
            elseif maxx==12;
               xmax = 7;
            elseif maxx==13;
               xmax = 8;
            elseif maxx==14;
               xmax = 9;
            else;
               xmax = 10;
            end;
            %
            % grid length value
            %
            if gridx==1;
               myxgrid = 0.001;
            elseif gridx==2;
               myxgrid = 0.005;
            elseif gridx==3;
               myxgrid = 0.01;
            elseif gridx==4;
               myxgrid = 0.02;
            elseif gridx==5;
               myxgrid = 0.025;
            elseif gridx==6;
               myxgrid = 0.04;
            elseif gridx==7;
               myxgrid = 0.05;
            elseif gridx==8;
               myxgrid = 0.1;
            elseif gridx==9;
               myxgrid = 0.125;
            else;
               myxgrid = 0.2;
            end;
            %
            % Selected beta parameter index & free parameter
            % number (the latter should be compared with the
            % free parameter number for the y axis when y axis
            % beta parameter is requested)
            %
            xbeta = strtrim(bm.mybetastr(betax,:));
            xbeta = strrep(xbeta,'beta(','');
            xbeta = strrep(xbeta,') - parameter','');
            xbeta = strrep(xbeta,',',' ');
            xbeta = mystr2num(xbeta);
            %
            % Value of selected beta parameter
            %
            betaij = bm.beta(xbeta(2),xbeta(1));
            change_beta = bm.beta;
            xmin = betaij-xmin;
            xmax = betaij+xmax;
            %
            % Setting up the loop to estimate beta subject to
            % the additional restriction
            %
            thisbij = xmin;
            bijvec = [];
            lnlvec = [];
            set(betagraphgui,'Visible','off');
            drawnow;
            currbeta = ['\beta_{' num2str(xbeta(1)) ',' num2str(xbeta(2)) '}'];
            input_var.title = 'Computing lnL for given parameters';
            input_var.facecolor = curr_ini.progress_facecolor;
            input_var.startfacecolor = curr_ini.progress_startfacecolor;
            input_var.edgecolor = curr_ini.progress_edgecolor;
            input_var.bgcolor = curr_ini.progress_bgcolor;
            input_var.label = [currbeta ' ='];
            input_var.stop = 0;
            input_var.clock = 1;
            input_var.name = ['Log-likelihood and beta(' num2str(xbeta(1)) ',' num2str(xbeta(2)) ')'];
            mywait = progressdlg(0,input_var);
%            set(mywait,'Name',['Log-likelihood and beta(' num2str(xbeta(1)) ',' num2str(xbeta(2)) ')']);
            set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
            drawnow;
            abort = '';
            while thisbij<=xmax;
               %
               % get info about waitbar
               %
               abort = get(mywait,'UserData');
               if (strcmp(abort,'cancel')==1);
                  break;
               else;
                  progressdlg([(thisbij-xmin)/(xmax-xmin) thisbij]);
               end;
               change_beta(xbeta(2),xbeta(1)) = thisbij;
               if isempty(bm.Z2)==0;
                  this_eps = bm.Z0-(bm.alpha*change_beta'*bm.Z1)-(bm.Psi*bm.Z2);
               else;
                  this_eps = bm.Z0-(bm.alpha*change_beta'*bm.Z1);
               end;
               thislnl = loglike(this_eps);
               %
               % Collect the results
               %
               bijvec = [bijvec;thisbij];
               lnlvec = [lnlvec;thislnl];
               thisbij = thisbij+myxgrid;
            end;
            %
            % remove waitbar
            %
            set(mywait,'UserData','done');
            close(mywait);
            drawnow;
            %
            % Set up 2-D graph
            %
            set(betagraphgui,'Visible','on');
            drawnow;
            drawnow;
            if strcmp(abort,'cancel')==0;
               if curr_ini.colors==1;
                  colorstr = 'b';
                  colorstr2 = 'r';
                  typestr = '-';
               else;
                  colorstr = 'k';
                  colorstr2 = [0.5 0.5 0.5];
                  typestr = '--';
               end;
               thisplot = figure('Units','pixels', ...
                  'Position',[(bm.realscrsz(3)-550)/2 (bm.realscrsz(4)-360)/2 550 360], ...
                  'NumberTitle','off', ...
                  'Menubar','none', ...
                  'Toolbar','none', ...
                  'Visible','off', ...
                  'Name',['Graph of log-likelihood and beta(' num2str(xbeta(1)) ',' num2str(xbeta(2)) ')']);
               %
               % obey centering and filling defaults
               %
               setfigureproperties(thisplot);
               %
               %
               % setting up the plot
               %
               hp = plot(bijvec,lnlvec,'-','LineWidth',1,'Color',colorstr);
               set(hp,'Tag','lnL');
               setaxesfonts('Title',['Values for ' currbeta ' and log-likelihood with all other parameters fixed'],'XLabel',currbeta,'YLabel',['lnL(' currbeta ',...)']);
               set(gca,'XLim',[min(bijvec) max(bijvec)]);
%               old_ax_units = get(gca,'Units');
%               set(gca,'Units','pixels');
%               ax_pos = get(gca,'Position');
%               set(gca,'Position',ax_pos-[0 0 0 10]);
%               set(gca,'Units',old_ax_units);
               my_pos = 1;
               if (lnlvec(size(lnlvec,1),1))>(lnlvec(1,1));
                  my_pos = 2;
               end;
               toolbar_fcn(gca);
               hold('off');
               if curr_ini.external==0;
                  set(thisplot,'Visible','on');
               else;
                  file = [bm.outputdir 'lnl-beta-fixed-' num2str(xbeta(1)) '.' num2str(xbeta(2))];
                  mstat = autosavefigure(thisplot,file);
                  if mstat==0;
                     autoshowfigure(file);
                  end;
               end;
            end;
         elseif (make3d==1)&(updatevalue==1);
            %
            % Make 3-D graph
            %
            minx = get(graphbeta.minx,'Value');
            maxx = get(graphbeta.maxx,'Value');
            gridx = get(graphbeta.gridx,'Value');
            betax = get(graphbeta.xaxis,'Value');
            %
            % relative min value for X grid
            %
            if minx==1;
               xmin = 0.5;
            elseif minx==2;
               xmin = 1;
            elseif minx==3;
               xmin = 1.5;
            elseif minx==4;
               xmin = 2;
            elseif minx==5;
               xmin = 2.5;
            elseif minx==6;
               xmin = 3;
            elseif minx==7;
               xmin = 3.5;
            elseif minx==8;
               xmin = 4;
            elseif minx==9;
               xmin = 4.5;
            elseif minx==10;
               xmin = 5;
            elseif minx==11;
               xmin = 6;
            elseif minx==12;
               xmin = 7;
            elseif minx==13;
               xmin = 8;
            elseif minx==14;
               xmin = 9;
            else;
               xmin = 10;
            end;
            %
            % relative max value for X grid
            %
            if maxx==1;
               xmax = 0.5;
            elseif maxx==2;
               xmax = 1;
            elseif maxx==3;
               xmax = 1.5;
            elseif maxx==4;
               xmax = 2;
            elseif maxx==5;
               xmax = 2.5;
            elseif maxx==6;
               xmax = 3;
            elseif maxx==7;
               xmax = 3.5;
            elseif maxx==8;
               xmax = 4;
            elseif maxx==9;
               xmax = 4.5;
            elseif maxx==10;
               xmax = 5;
            elseif maxx==11;
               xmax = 6;
            elseif maxx==12;
               xmax = 7;
            elseif maxx==13;
               xmax = 8;
            elseif maxx==14;
               xmax = 9;
            else;
               xmax = 10;
            end;
            %
            % grid length value
            %
            if gridx==1;
               myxgrid = 0.001;
               xtimes = 1000;
            elseif gridx==2;
               myxgrid = 0.005;
               xtimes = 200;
            elseif gridx==3;
               myxgrid = 0.01;
               xtimes = 100;
            elseif gridx==4;
               myxgrid = 0.02;
               xtimes = 50;
            elseif gridx==5;
               myxgrid = 0.025;
               xtimes = 40;
            elseif gridx==6;
               myxgrid = 0.04;
               xtimes = 25;
            elseif gridx==7;
               myxgrid = 0.05;
               xtimes = 20;
            elseif gridx==8;
               myxgrid = 0.1;
               xtimes = 10;
            elseif gridx==9;
               myxgrid = 0.125;
               xtimes = 8;
            else;
               myxgrid = 0.2;
               xtimes = 5;
            end;
            %
            % Selected beta parameter index & free parameter
            % number (the latter should be compared with the
            % free parameter number for the y axis when y axis
            % beta parameter is requested)
            %
            xbeta = strtrim(bm.mybetastr(betax,:));
            xbeta = strrep(xbeta,'beta(','');
            xbeta = strrep(xbeta,') - parameter','');
            xbeta = strrep(xbeta,',',' ');
            xbeta = mystr2num(xbeta);
            %
            % Value of selected beta parameter
            %
            betaijx = bm.beta(xbeta(2),xbeta(1));
            xmin = betaijx-xmin;
            xmax = betaijx+xmax;
            %
            % recovering info for Y axis
            %
            miny = get(graphbeta.miny,'Value');
            maxy = get(graphbeta.maxy,'Value');
            gridy = get(graphbeta.gridy,'Value');
            betay = get(graphbeta.yaxis,'Value');
            %
            % relative min value for Y grid
            %
            if miny==1;
               ymin = 0.5;
            elseif miny==2;
               ymin = 1;
            elseif miny==3;
               ymin = 1.5;
            elseif miny==4;
               ymin = 2;
            elseif miny==5;
               ymin = 2.5;
            elseif miny==6;
               ymin = 3;
            elseif miny==7;
               ymin = 3.5;
            elseif miny==8;
               ymin = 4;
            elseif miny==9;
               ymin = 4.5;
            elseif miny==10;
               ymin = 5;
            elseif miny==11;
               ymin = 6;
            elseif miny==12;
               ymin = 7;
            elseif miny==13;
               ymin = 8;
            elseif miny==14;
               ymin = 9;
            else;
               ymin = 10;
            end;
            %
            % relative max value for Y grid
            %
            if maxy==1;
               ymax = 0.5;
            elseif maxy==2;
               ymax = 1;
            elseif maxy==3;
               ymax = 1.5;
            elseif maxy==4;
               ymax = 2;
            elseif maxy==5;
               ymax = 2.5;
            elseif maxy==6;
               ymax = 3;
            elseif maxy==7;
               ymax = 3.5;
            elseif maxy==8;
               ymax = 4;
            elseif maxy==9;
               ymax = 4.5;
            elseif maxy==10;
               ymax = 5;
            elseif maxy==11;
               ymax = 6;
            elseif maxy==12;
               ymax = 7;
            elseif maxy==13;
               ymax = 8;
            elseif maxy==14;
               ymax = 9;
            else;
               ymax = 10;
            end;
            %
            % grid length value
            %
            if gridy==1;
               myygrid = 0.001;
               ytimes = 1000;
            elseif gridy==2;
               myygrid = 0.005;
               ytimes = 200;
            elseif gridy==3;
               myygrid = 0.01;
               ytimes = 100;
            elseif gridy==4;
               myygrid = 0.02;
               ytimes = 50;
            elseif gridy==5;
               myygrid = 0.025;
               ytimes = 40;
            elseif gridy==6;
               myygrid = 0.04;
               ytimes = 25;
            elseif gridy==7;
               myygrid = 0.05;
               ytimes = 20;
            elseif gridy==8;
               myygrid = 0.1;
               ytimes = 10;
            elseif gridy==9;
               myygrid = 0.125;
               ytimes = 8;
            else;
               myygrid = 0.2;
               ytimes = 5;
            end;
            %
            % Selected beta parameter index & free parameter
            % number (the latter should be compared with the
            % free parameter number for the y axis when y axis
            % beta parameter is requested)
            %
            ybeta = strtrim(bm.mybetastr(betay,:));
            ybeta = strrep(ybeta,'beta(','');
            ybeta = strrep(ybeta,') - parameter','');
            ybeta = strrep(ybeta,',',' ');
            ybeta = mystr2num(ybeta);
            %
            % Value of selected beta parameter
            %
            betaijy = bm.beta(ybeta(2),ybeta(1));
            ymin = betaijy-ymin;
            ymax = betaijy+ymax;
            %
            % Setting up the loop to estimate beta subject to
            % the additional restriction
            %
            set(betagraphgui,'Visible','off');
            drawnow;
            currbeta = ['\beta_{' num2str(xbeta(1)) ',' num2str(xbeta(2)) '},\beta_{' num2str(ybeta(1)) ',' num2str(ybeta(2)) '}'];
            input_var.title = ['Estimating \beta for given (' currbeta ') ' num2str((round(((xmax-xmin)*xtimes)+1)*round(((ymax-ymin)*ytimes)+1))) ' times'];
            input_var.facecolor = curr_ini.progress_facecolor;
            input_var.startfacecolor = curr_ini.progress_startfacecolor;
            input_var.edgecolor = curr_ini.progress_edgecolor;
            input_var.bgcolor = curr_ini.progress_bgcolor;
            input_var.label = ['(' currbeta ') ='];
            input_var.stop = 0;
            input_var.clock = 1;
            input_var.name = ['Log-likelihood, beta(' num2str(xbeta(1)) ',' num2str(xbeta(2)) ') and beta(' num2str(ybeta(1)) ',' num2str(ybeta(2)) ')'];
            mywait = progressdlg(0,input_var);
%            set(mywait,'Name',['Log-likelihood, beta(' num2str(xbeta(1)) ',' num2str(xbeta(2)) ') and beta(' num2str(ybeta(1)) ',' num2str(ybeta(2)) ')']);
            set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
            drawnow;
            abort = '';
            addxy = 0;
            numx = 0;
            thisbijx = xmin;
            bijmat = [];
            lnlvec = [];
            while thisbijx<=xmax+1.0e-05;
               thisbijy = ymin;
               while thisbijy<=ymax+1.0e-05;
                  addxy = addxy+1;
                  %
                  % get info about waitbar
                  %
                  abort = get(mywait,'UserData');
                  if strcmp(abort,'cancel')==1;
                     break;
                  else;
                     progressdlg([addxy/(round(((xmax-xmin)*xtimes)+1)*round(((ymax-ymin)*ytimes)+1)) thisbijx thisbijy]);
                  end;
                  if (bm.restrict(1,1)==1)&(curr_ini.reestbeta==1);
                     %
                     % Estimate beta and alpha simultaneosly
                     %
                     thisbeta = beta_3d_restrict(bm.S_00,bm.S_01,bm.S_10,bm.S_11,bm.oldbeta,bm.T,bm.dcoint,bm.modelsdir,bm.modelfile,thisbijx,xbeta(2),xbeta(1),thisbijy,ybeta(2),ybeta(1));
                     thisalpha = (bm.S_01*thisbeta)/(thisbeta'*bm.S_11*thisbeta);
                     if isempty(bm.Z2)==0;
                        thisPsi = (bm.M_02/bm.M_22)-(thisalpha*thisbeta'*(bm.M_12/bm.M_22));
                        Z_3 = [bm.Z2;thisbeta'*bm.Z1];
                     else;
                        thisPsi = [];
                        Z_3 = thisbeta'*bm.Z1;
                     end;
                     file = strcat(bm.modelsdir,'G-',int2str(bm.n),'x',int2str(bm.r),'-',bm.modelfile,'.rest');
                     thisG = loaddata(file);
                     [thisRmat,thisrvec] = rmatrix(thisG,bm.n,bm.r);
                     df_alpha = size(thisG,1);
                     if isempty(bm.Z2)==0;
                        thisRmat = [zeros(df_alpha,size(bm.Z2,1)*bm.n) thisRmat];
                     end;
                     thisTheta = [thisPsi thisalpha];
                     [thistheta,thisbeta] = alpha_beta_3d_restrict(bm.Z0,bm.Z1,bm.Z2,Z_3,thisbeta,thisTheta,thisRmat,thisrvec,bm.dcoint,bm.q1,bm.modelsdir,bm.modelfile,thisbijx,xbeta(2),xbeta(1),thisbijy,ybeta(2),ybeta(1));
                     %
                     if isempty(bm.Z2)==0;
                        Z_3 = [bm.Z2;thisbeta'*bm.Z1];
                     else;
                        Z_3 = thisbeta'*bm.Z1;
                     end;
                     this_eps = bm.Z0-(thistheta*Z_3);
                     thislnl = loglike(this_eps);
                  else;
                     %
                     % Estimate beta under restrictions
                     %
                     thisbeta = beta_3d_restrict(bm.S_00,bm.S_01,bm.S_10,bm.S_11,bm.oldbeta,bm.T,bm.dcoint,bm.modelsdir,bm.modelfile,thisbijx,xbeta(2),xbeta(1),thisbijy,ybeta(2),ybeta(1));
                     thisalpha = (bm.S_01*thisbeta)/(thisbeta'*bm.S_11*thisbeta);
                     if isempty(bm.Z2)==0;
                        Z_3 = [bm.Z2;thisbeta'*bm.Z1];
                        thisPsi = (bm.M_02/bm.M_22)-(thisalpha*thisbeta'*(bm.M_12/bm.M_22));
                        this_eps = bm.Z0-(thisalpha*thisbeta'*bm.Z1)-(thisPsi*bm.Z2);
                     else;
                        Z_3 = thisbeta'*bm.Z1;
                        thisPsi = [];
                        this_eps = bm.Z0-(thisalpha*thisbeta'*bm.Z1);
                     end;
                     thistheta = [thisPsi thisalpha];
                     thislnl = loglike(this_eps);
                     %
                     % Estimate alpha under restrictions if decided
                     %
                     if bm.restrict(1,1)==1;
                        file = strcat(bm.modelsdir,'G-',int2str(bm.n),'x',int2str(bm.r),'-',bm.modelfile,'.rest');
                        thisG = loaddata(file);
                        [thisRmat,thisrvec] = rmatrix(thisG,bm.n,bm.r);
                        df_alpha = size(thisG,1);
                        if isempty(bm.Z2)==0;
                           thisRmat = [zeros(df_alpha,size(bm.Z2,1)*bm.n) thisRmat];
                        end;
                        thisTheta = [thisPsi thisalpha];
                        [thistheta,Sigma_thistheta,thisinfo,thisabort] = linrest(bm.Z0,Z_3,thisTheta,thisRmat,thisrvec,0);
                        this_eps = bm.Z0-(thistheta*Z_3);
                        thislnl = loglike(this_eps);
                     end;
                  end;
                  %
                  % allow for restrictions on C
                  %
                  if (size(bm.restrict,2)==3)&(bm.restrict(1,3)==1);
                     file = strcat(bm.modelsdir,'C-',int2str(bm.n),'x',int2str(bm.r),'-',bm.modelfile,'.rest');
                     thisC = loaddata(file);
                     [Rmat_C,rvec_C] = rmatrix(thisC,bm.n,bm.n);
                     [thistheta,thisbeta] = c_3d_restrict(bm.Z0,bm.Z1,bm.Z2,bm.restrict,thisbeta,thistheta,Rmat_C,rvec_C,bm.p,bm.pe,bm.d,bm.dcoint,bm.q0,bm.q1,bm.modelsdir,bm.modelfile,thisbijx,xbeta(2),xbeta(1),thisbijy,ybeta(2),ybeta(1));
                     if isempty(bm.Z2)==0;
                        Z_3 = [bm.Z2;thisbeta'*bm.Z1];
                     else;
                        Z_3 = thisbeta'*bm.Z1;
                     end;
                     this_eps = bm.Z0-(thistheta*Z_3);
                     thislnl = loglike(this_eps);
                  end;
                  %
                  % Collect the results
                  %
                  bijmat = [bijmat;[thisbijx thisbijy]];
                  lnlvec = [lnlvec;thislnl];
                  thisbijy = thisbijy+myygrid;
               end;
               numx = numx+1;
               thisbijx = thisbijx+myxgrid;
            end;
            set(mywait,'UserData','done');
            close(mywait);
            drawnow;
            set(betagraphgui,'Visible','on');
            drawnow;
            drawnow;
            %
            % Set up 3-D graph
            %
            if strcmp(abort,'cancel')==0;
               numy = size(bijmat,1)/numx;
               bx = vecinv(bijmat(:,1),numy);
               by = vecinv(bijmat(:,2),numy);
               blnl = vecinv(lnlvec,numy);
               %
               % setting up the contour
               %
               nextplot = figure('Units','pixels', ...
                  'Position',[((bm.realscrsz(3)-550)/2)+22 ((bm.realscrsz(4)-360)/2)-22 550 360], ...
                  'NumberTitle','off', ...
                  'Menubar','none', ...
                  'Toolbar','none', ...
                  'Visible','off', ...
                  'Name',['Contour of log-likelihood, beta(' num2str(xbeta(1)) ',' num2str(xbeta(2)) ') and beta(' num2str(ybeta(1)) ',' num2str(ybeta(2)) ')']);
               %
               % obey centering and filling defaults
               %
               setfigureproperties(nextplot);
               %
               %
               % compute data for contour, i.e. 80, 90, 95, 98 and 99 percent values for lnL
               %
               this_vec = [80 90 95 98 99];
               contour_vec = [];
               for i=1:size(this_vec,2);
                  c_value = max(lnlvec)-(chisq_inv(this_vec(i)/100,2)/2);
                  contour_vec = [contour_vec c_value];
               end;
               [c_mat,l_handles] = contour(bx',by',blnl',contour_vec,'-');
               l_handles = sort(l_handles);
               num_h = size(l_handles,1);
               num_lines = ones(1,num_h);
               for i=1:num_h;
                  lines(i).xdata = get(l_handles(i,1),'XData');
                  lines(i).ydata = get(l_handles(i,1),'YData');
                  lines(i).zdata = get(l_handles(i,1),'ZData');
                  lines(i).color = get(l_handles(i,1),'Color');
                  lines(i).tag = ['lines-' num2str(i)];
               end;
               for i=1:num_h;
                  if i==1;
                     if num_h>1;
                        if lines(1).color==lines(2).color;
                           num_lines(1,1) = 2;
                        end;
                     end;
                  else;
                     if num_h>i;
                        if (lines(i).color==lines(i+1).color)|(lines(i).color==lines(i-1).color);
                           num_lines(1,i) = 2;
                        end;
                     else;
                        if lines(i-1).color==lines(i).color;
                           num_lines(1,i) = 2;
                        end;
                     end;
                  end;
               end;
               delete(l_handles);
               if curr_ini.colors==1;
                  c_vec = [1 0 0;
                           0 0 1;
                           1 0.5 0;
                           0 0.75 0;
                           0 0 0];
               else;
                  c_vec = [0.7 0.7 0.7;
                           0.6 0.6 0.6;
                           0.5 0.5 0.5;
                           0.4 0.4 0.4;
                           0.3 0.3 0.3];
               end;
               l_style = strvcat('--',':','-','-.',':');
               leg_style = strvcat('80%','90%','95%','98%','99%');
               leg_str = '';
               i = 1;
               count = 1;
               my_odd = 0;
               while i<=num_h;
                  lines(i).color = c_vec(count,:);
                  lines(i).linestyle = strtrim(l_style(count,:));
                  my_odd = my_odd+1;
                  if num_lines(1,i)==2;
                     if my_odd==2;
                        my_odd = 0;
                        lines(i).tag = ['2dlines-' num2str(i)];
                        count = count+1;
                     else;
                        new_str = strtrim(leg_style(count,:));
                        leg_str = strvcat(leg_str,new_str);
                     end;
                  else;
                     my_odd = 0;
                     new_str = strtrim(leg_style(count,:));
                     leg_str = strvcat(leg_str,new_str);
                     count = count+1;
                  end;
                  i = i+1;
               end;
               %
               % remaking lines from the contour function
               %
               my_leg_objects = [];
               for i=1:num_h;
                  this_line = line('Xdata',lines(i).xdata, ...
                     'YData',lines(i).ydata, ...
                     'LineStyle',lines(i).linestyle, ...
                     'Color',lines(i).color, ...
                     'LineWidth',2, ...
                     'Tag',lines(i).tag);
                  hold('on');
                  this_tag = lines(i).tag;
                  if strcmp(this_tag(1,1:2),'2d')==0;
                     my_leg_objects = [my_leg_objects this_line];
                  end;
                  if num_lines(i)==1;
                     %
                     % dealing with minimum points
                     %
                     if (min(lines(i).xdata)>min(min(bx)))&(min(lines(i).ydata)>min(min(by)));
                        [x_val,indx] = min(lines(i).xdata);
                        y_line = lines(i).ydata;
                        line('XData',[x_val x_val], ...
                           'YData',[min(min(by)) y_line(indx)], ...
                           'Tag',['nolegend-minx' num2str(i)], ...
                           'LineWidth',1, ...
                           'LineStyle',':', ...
                           'Color',lines(i).color);
                        [y_val,indy] = min(lines(i).ydata);
                        x_line = lines(i).xdata;
                        line('XData',[min(min(bx)) x_line(indy)], ...
                           'YData',[y_val y_val], ...
                           'Tag',['nolegend-miny' num2str(i)], ...
                           'LineWidth',1, ...
                           'LineStyle',':', ...
                           'Color',lines(i).color);
                     end;
                     %
                     % dealing with maximum points
                     %
                     if (max(lines(i).xdata)<max(max(bx)))&(max(lines(i).ydata)<max(max(by)));
                        [x_val,indx] = max(lines(i).xdata);
                        y_line = lines(i).ydata;
                        line('XData',[x_val x_val], ...
                           'YData',[min(min(by)) y_line(indx)], ...
                           'Tag',['nolegend-maxx' num2str(i)], ...
                           'LineWidth',1, ...
                           'LineStyle',':', ...
                           'Color',lines(i).color);
                        [y_val,indy] = max(lines(i).ydata);
                        x_line = lines(i).xdata;
                        line('XData',[min(min(bx)) x_line(indy)], ...
                           'YData',[y_val y_val], ...
                           'Tag',['nolegend-maxy' num2str(i)], ...
                           'LineWidth',1, ...
                           'LineStyle',':', ...
                           'Color',lines(i).color);
                     end;
                  end;
               end;
               set(gca,'XLim',[min(min(bx)) max(max(bx))]);
               set(gca,'YLim',[min(min(by)) max(max(by))]);
               %
               % drawing lines from the x and y axes to the max value
               %
               line('Xdata',[betaijx betaijx]', ...
                  'YData',[min(min(by)) betaijy]', ...
                  'Tag','nolegend-1', ...
                  'LineWidth',1, ...
                  'LineStyle',':', ...
                  'Color',[0.3 0.3 0.3]);
               hold('on');
               line('Xdata',[min(min(bx)) betaijx]', ...
                  'YData',[betaijy betaijy]', ...
                  'Tag','nolegend-2', ...
                  'LineWidth',1, ...
                  'LineStyle',':', ...
                  'Color',[0.3 0.3 0.3]);
               hold('on');
               %
               % including the max-value
               %
               line('XData',betaijx, ...
                  'YData',betaijy, ...
                  'LineStyle','-', ...
                  'Tag','nolegend-3', ...
                  'LineWidth',1, ...
                  'Marker','*', ...
                  'MarkerSize',8, ...
                  'Color',[0 0 0]);
               %
               % setting default values for figure
               %
               setaxesfonts('Title',['Confidence regions for (' currbeta ')'],'XLabel',['\beta_{' num2str(xbeta(1)) ',' num2str(xbeta(2)) '}'],'YLabel',['\beta_{' num2str(ybeta(1)) ',' num2str(ybeta(2)) '}']);
%               old_ax_units = get(gca,'Units');
%               set(gca,'Units','pixels');
%               ax_pos = get(gca,'Position');
%               set(gca,'Position',ax_pos-[0 0 0 10]);
%               set(gca,'Units',old_ax_units);
               if num_h>=1;
                  myleg = legend_fcn(my_leg_objects,leg_str,0);
                  setaxesfonts(myleg);
                  myleg = update_legend(myleg);
                  set(gca,'UserData',myleg);
               end;
               toolbar_fcn(gca);
               hold('off');
               %
               % Setting up the surface
               %
               thisplot = figure('Units','pixels', ...
                  'Position',[(bm.realscrsz(3)-550)/2 (bm.realscrsz(4)-360)/2 550 360], ...
                  'Menubar','none', ...
                  'Toolbar','none', ...
                  'NumberTitle','off', ...
                  'Visible','off', ...
                  'Name',['Plot of log-likelihood, beta(' num2str(xbeta(1)) ',' num2str(xbeta(2)) ') and beta(' num2str(ybeta(1)) ',' num2str(ybeta(2)) ')']);
               %
               % obey centering and filling defaults
               %
               setfigureproperties(thisplot);
               %
               %
               % setting up the surface
               %
               lnl_cbs = max(lnlvec)-(chisq_inv(bm.cbs/100,2)/2);
               this_surf = surf(bx',by',blnl','EdgeColor','none','LineWidth',1,'FaceLighting','gouraud','SpecularStrength',0.5);
               this_axes = gca;
               this_zlim = get(this_axes,'Zlim');
               if curr_ini.colors==1;
                  colormap(jet);
                  set(this_surf,'UserData','jet');
                  gray_check = 'off';
                  jet_check = 'on';
               else;
                  colormap(gray);
                  set(this_surf,'UserData','gray');
                  gray_check = 'on';
                  jet_check = 'off';
               end;
               if get(graphbeta.shading,'Value')==1;
                  shading('interp');
               end;
               hold('on');
               this_surf2 = surf(bx',by',lnl_cbs*ones(size(bx')),'EdgeColor','none','FaceColor',[0.5 0.5 0.5],'LineWidth',1);
               hold('on');
               set(this_surf2,'UserData',0.5);
               set(this_axes,'XLim',[min(min(bx)) max(max(bx))]);
               set(this_axes,'YLim',[min(min(by)) max(max(by))]);
               set(this_axes,'ZLim',this_zlim);
               setaxesfonts('Title',['Values for (' currbeta ') and log-likelihood with ' num2str(bm.cbs) '% confidence region'],'Xlabel',['\beta_{' num2str(xbeta(1)) ',' num2str(xbeta(2)) '}'],'YLabel',['\beta_{' num2str(ybeta(1)) ',' num2str(ybeta(2)) '}'],'ZLabel',['lnL(' currbeta ',...)']);
               myleg = legend_fcn('lnL','confidence',4);
               setaxesfonts(myleg);
               myleg = update_legend(myleg);
               leglines = findobj(myleg,'Type','patch')';
               set(leglines(1),'FaceColor',[0.5 0.5 0.5],'EdgeColor',[0.5 0.5 0.5]);
               set(leglines(2),'EdgeColor',get(leglines(2),'FaceColor'));
               set(this_axes,'UserData',myleg);
               this_light = light('Position',[min(min(bx)) min(min(by)) max(max(blnl))],'Style','infinite');
               hold('on');
               plot3(betaijx*ones(2,1),betaijy*ones(2,1),[max(max(blnl)) min(get(this_axes,'ZLim'))]','LineStyle','--','Color',[0 0 0],'LineWidth',2,'Marker','x','MarkerSize',12,'MarkerFaceColor',[0 0 0],'UserData',1,'Tag','3dlines-lnl');
               hold('on');
%               old_ax_units = get(this_axes,'Units');
%               set(this_axes,'Units','pixels');
%               ax_pos = get(this_axes,'Position');
%               set(this_axes,'Position',ax_pos-[0 0 0 10]);
%               set(this_axes,'Units',old_ax_units);
               if get(graphbeta.colorbar,'Value')==1;
                  this_colorbar = colorbar_fcn;
                  set(this_colorbar,'Tag','colorbar');
                  setaxesfonts(this_colorbar);
                  colorbar_pos = get(this_colorbar,'Position');
                  set(this_colorbar,'Position',[colorbar_pos(1)+(2*colorbar_pos(3)/3) colorbar_pos(2) colorbar_pos(3)/3 colorbar_pos(4)]);
                  new_leg = get(this_colorbar,'UserData');
                  set(this_axes,'UserData',new_leg);
                  set(this_colorbar,'UserData',this_axes);
               end;
               set(this_light,'UserData',this_surf);
               toolbar_fcn(this_axes);
               hold('off');
               if curr_ini.external==0;
                  set(thisplot,'Visible','on');
                  drawnow;
                  set(nextplot,'Visible','on');
                  drawnow;
               else;
                  file1 = [bm.outputdir 'lnl-beta-' num2str(xbeta(1)) '.' num2str(xbeta(2)) '-beta-' num2str(ybeta(1)) '.' num2str(ybeta(2))];
                  file2 = [bm.outputdir 'contour-beta-' num2str(xbeta(1)) '.' num2str(xbeta(2)) '-beta-' num2str(ybeta(1)) '.' num2str(ybeta(2))];
                  mstat1 = autosavefigure(thisplot,file1);
                  mstat2 = autosavefigure(nextplot,file2);
                  if mstat1==0;
                     autoshowfigure(file1);
                  end;
                  if mstat2==0;
                     autoshowfigure(file2);
                  end;
               end;
            end;
         elseif (make3d==1)&(updatevalue==0);
            %
            % Make 3-D graph
            %
            minx = get(graphbeta.minx,'Value');
            maxx = get(graphbeta.maxx,'Value');
            gridx = get(graphbeta.gridx,'Value');
            betax = get(graphbeta.xaxis,'Value');
            %
            % relative min value for X grid
            %
            if minx==1;
               xmin = 0.5;
            elseif minx==2;
               xmin = 1;
            elseif minx==3;
               xmin = 1.5;
            elseif minx==4;
               xmin = 2;
            elseif minx==5;
               xmin = 2.5;
            elseif minx==6;
               xmin = 3;
            elseif minx==7;
               xmin = 3.5;
            elseif minx==8;
               xmin = 4;
            elseif minx==9;
               xmin = 4.5;
            elseif minx==10;
               xmin = 5;
            elseif minx==11;
               xmin = 6;
            elseif minx==12;
               xmin = 7;
            elseif minx==13;
               xmin = 8;
            elseif minx==14;
               xmin = 9;
            else;
               xmin = 10;
            end;
            %
            % relative max value for X grid
            %
            if maxx==1;
               xmax = 0.5;
            elseif maxx==2;
               xmax = 1;
            elseif maxx==3;
               xmax = 1.5;
            elseif maxx==4;
               xmax = 2;
            elseif maxx==5;
               xmax = 2.5;
            elseif maxx==6;
               xmax = 3;
            elseif maxx==7;
               xmax = 3.5;
            elseif maxx==8;
               xmax = 4;
            elseif maxx==9;
               xmax = 4.5;
            elseif maxx==10;
               xmax = 5;
            elseif maxx==11;
               xmax = 6;
            elseif maxx==12;
               xmax = 7;
            elseif maxx==13;
               xmax = 8;
            elseif maxx==14;
               xmax = 9;
            else;
               xmax = 10;
            end;
            %
            % grid length value
            %
            if gridx==1;
               myxgrid = 0.001;
               xtimes = 1000;
            elseif gridx==2;
               myxgrid = 0.005;
               xtimes = 200;
            elseif gridx==3;
               myxgrid = 0.01;
               xtimes = 100;
            elseif gridx==4;
               myxgrid = 0.02;
               xtimes = 50;
            elseif gridx==5;
               myxgrid = 0.025;
               xtimes = 40;
            elseif gridx==6;
               myxgrid = 0.04;
               xtimes = 25;
            elseif gridx==7;
               myxgrid = 0.05;
               xtimes = 20;
            elseif gridx==8;
               myxgrid = 0.1;
               xtimes = 10;
            elseif gridx==9;
               myxgrid = 0.125;
               xtimes = 8;
            else;
               myxgrid = 0.2;
               xtimes = 5;
            end;
            %
            % Selected beta parameter index & free parameter
            % number (the latter should be compared with the
            % free parameter number for the y axis when y axis
            % beta parameter is requested)
            %
            xbeta = strtrim(bm.mybetastr(betax,:));
            xbeta = strrep(xbeta,'beta(','');
            xbeta = strrep(xbeta,') - parameter','');
            xbeta = strrep(xbeta,',',' ');
            xbeta = mystr2num(xbeta);
            %
            % Value of selected beta parameter
            %
            betaijx = bm.beta(xbeta(2),xbeta(1));
            xmin = betaijx-xmin;
            xmax = betaijx+xmax;
            %
            % recovering info for Y axis
            %
            miny = get(graphbeta.miny,'Value');
            maxy = get(graphbeta.maxy,'Value');
            gridy = get(graphbeta.gridy,'Value');
            betay = get(graphbeta.yaxis,'Value');
            %
            % relative min value for Y grid
            %
            if miny==1;
               ymin = 0.5;
            elseif miny==2;
               ymin = 1;
            elseif miny==3;
               ymin = 1.5;
            elseif miny==4;
               ymin = 2;
            elseif miny==5;
               ymin = 2.5;
            elseif miny==6;
               ymin = 3;
            elseif miny==7;
               ymin = 3.5;
            elseif miny==8;
               ymin = 4;
            elseif miny==9;
               ymin = 4.5;
            elseif miny==10;
               ymin = 5;
            elseif miny==11;
               ymin = 6;
            elseif miny==12;
               ymin = 7;
            elseif miny==13;
               ymin = 8;
            elseif miny==14;
               ymin = 9;
            else;
               ymin = 10;
            end;
            %
            % relative max value for Y grid
            %
            if maxy==1;
               ymax = 0.5;
            elseif maxy==2;
               ymax = 1;
            elseif maxy==3;
               ymax = 1.5;
            elseif maxy==4;
               ymax = 2;
            elseif maxy==5;
               ymax = 2.5;
            elseif maxy==6;
               ymax = 3;
            elseif maxy==7;
               ymax = 3.5;
            elseif maxy==8;
               ymax = 4;
            elseif maxy==9;
               ymax = 4.5;
            elseif maxy==10;
               ymax = 5;
            elseif maxy==11;
               ymax = 6;
            elseif maxy==12;
               ymax = 7;
            elseif maxy==13;
               ymax = 8;
            elseif maxy==14;
               ymax = 9;
            else;
               ymax = 10;
            end;
            %
            % grid length value
            %
            if gridy==1;
               myygrid = 0.001;
               ytimes = 1000;
            elseif gridy==2;
               myygrid = 0.005;
               ytimes = 200;
            elseif gridy==3;
               myygrid = 0.01;
               ytimes = 100;
            elseif gridy==4;
               myygrid = 0.02;
               ytimes = 50;
            elseif gridy==5;
               myygrid = 0.025;
               ytimes = 40;
            elseif gridy==6;
               myygrid = 0.04;
               ytimes = 25;
            elseif gridy==7;
               myygrid = 0.05;
               ytimes = 20;
            elseif gridy==8;
               myygrid = 0.1;
               ytimes = 10;
            elseif gridy==9;
               myygrid = 0.125;
               ytimes = 8;
            else;
               myygrid = 0.2;
               ytimes = 5;
            end;
            %
            % Selected beta parameter index & free parameter
            % number (the latter should be compared with the
            % free parameter number for the y axis when y axis
            % beta parameter is requested)
            %
            ybeta = strtrim(bm.mybetastr(betay,:));
            ybeta = strrep(ybeta,'beta(','');
            ybeta = strrep(ybeta,') - parameter','');
            ybeta = strrep(ybeta,',',' ');
            ybeta = mystr2num(ybeta);
            %
            % Value of selected beta parameter
            %
            betaijy = bm.beta(ybeta(2),ybeta(1));
            ymin = betaijy-ymin;
            ymax = betaijy+ymax;
            %
            % initializing the beta matrix
            %
            change_beta = bm.beta;
            %
            % Setting up the loop to estimate beta subject to
            % the additional restriction
            %
            set(betagraphgui,'Visible','off');
            drawnow;
            currbeta = ['\beta_{' num2str(xbeta(1)) ',' num2str(xbeta(2)) '},\beta_{' num2str(ybeta(1)) ',' num2str(ybeta(2)) '}'];
            input_var.title = ['Computing lnL for given parameters ' num2str((round(((xmax-xmin)*xtimes)+1)*round(((ymax-ymin)*ytimes)+1))) ' times'];
            input_var.edgecolor = curr_ini.progress_edgecolor;
            input_var.facecolor = curr_ini.progress_facecolor;
            input_var.startfacecolor = curr_ini.progress_startfacecolor;
            input_var.bgcolor = curr_ini.progress_bgcolor;
            input_var.label = ['(' currbeta ') ='];
            input_var.stop = 0;
            input_var.clock = 1;
            input_var.name = ['Log-likelihood, beta(' num2str(xbeta(1)) ',' num2str(xbeta(2)) ') and beta(' num2str(ybeta(1)) ',' num2str(ybeta(2)) ')'];
            mywait = progressdlg(0,input_var);
%            set(mywait,'Name',['Log-likelihood, beta(' num2str(xbeta(1)) ',' num2str(xbeta(2)) ') and beta(' num2str(ybeta(1)) ',' num2str(ybeta(2)) ')']);
            set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
            drawnow;
            abort = '';
            addxy = 0;
            numx = 0;
            thisbijx = xmin;
            bijmat = [];
            lnlvec = [];
            while thisbijx<=xmax+1.0e-05;
               thisbijy = ymin;
               change_beta(xbeta(2),xbeta(1)) = thisbijx;
               while thisbijy<=ymax+1.0e-05;
                  addxy = addxy+1;
                  %
                  % get info about waitbar
                  %
                  abort = get(mywait,'UserData');
                  if strcmp(abort,'cancel')==1;
                     break;
                  else;
                     progressdlg([addxy/(round(((xmax-xmin)*xtimes)+1)*round(((ymax-ymin)*ytimes)+1)) thisbijx thisbijy]);
                  end;
                  change_beta(ybeta(2),ybeta(1)) = thisbijy;
                  if isempty(bm.Z2)==0;
                     this_eps = bm.Z0-(bm.alpha*change_beta'*bm.Z1)-(bm.Psi*bm.Z2);
                  else;
                     this_eps = bm.Z0-(bm.alpha*change_beta'*bm.Z1);
                  end;
                  thislnl = loglike(this_eps);
                  %
                  % Collect the results
                  %
                  bijmat = [bijmat;[thisbijx thisbijy]];
                  lnlvec = [lnlvec;thislnl];
                  thisbijy = thisbijy+myygrid;
               end;
               numx = numx+1;
               thisbijx = thisbijx+myxgrid;
            end;
            set(mywait,'UserData','done');
            close(mywait);
            drawnow;
            set(betagraphgui,'Visible','on');
            drawnow;
            drawnow;
            %
            % Set up 3-D graph
            %
            if strcmp(abort,'cancel')==0;
               thisplot = figure('Units','pixels', ...
                  'Position',[(bm.realscrsz(3)-550)/2 (bm.realscrsz(4)-360)/2 550 360], ...
                  'Menubar','none', ...
                  'Toolbar','none', ...
                  'NumberTitle','off', ...
                  'Visible','off', ...
                  'Name',['Graph of log-likelihood, beta(' num2str(xbeta(1)) ',' num2str(xbeta(2)) ') and beta(' num2str(ybeta(1)) ',' num2str(ybeta(2)) ')']);
               %
               % obey centering and filling defaults
               %
               setfigureproperties(thisplot);
               %
               %
               % setting up the surface
               %
               numy = size(bijmat,1)/numx;
               bx = vecinv(bijmat(:,1),numy);
               by = vecinv(bijmat(:,2),numy);
               blnl = vecinv(lnlvec,numy);
               this_surf = surf(bx',by',blnl','EdgeColor','none','LineWidth',1,'FaceLighting','gouraud','SpecularStrength',0.5);
               this_zlim = get(gca,'Zlim');
               if curr_ini.colors==1;
                  colormap(jet);
                  set(this_surf,'UserData','jet');
                  gray_check = 'off';
                  jet_check = 'on';
               else;
                  colormap(gray);
                  set(this_surf,'UserData','gray');
                  gray_check = 'on';
                  jet_check = 'off';
               end;
               if get(graphbeta.shading,'Value')==1;
                  shading('interp');
               end;
               hold('on');
               set(gca,'XLim',[min(min(bx)) max(max(bx))]);
               set(gca,'YLim',[min(min(by)) max(max(by))]);
               set(gca,'ZLim',this_zlim);
               setaxesfonts('Title',['Values for (' currbeta ') and log-likelihood with all other parameters fixed'],'XLabel',['\beta_{' num2str(xbeta(1)) ',' num2str(xbeta(2)) '}'],'YLabel',['\beta_{' num2str(ybeta(1)) ',' num2str(ybeta(2)) '}'],'ZLabel',['lnL(' currbeta ',...)']);
               this_light = light('Position',[min(min(bx)) min(min(by)) max(max(blnl))],'Style','infinite');
               plot3(betaijx*ones(2,1),betaijy*ones(2,1),[max(max(blnl)) min(get(gca,'ZLim'))]','LineStyle','--','Color',[0 0 0],'LineWidth',2,'Marker','x','MarkerSize',12,'MarkerFaceColor',[0 0 0],'UserData',1);
%               old_ax_units = get(gca,'Units');
%               set(gca,'Units','pixels');
%               ax_pos = get(gca,'Position');
%               set(gca,'Position',ax_pos-[0 0 0 10]);
%               set(gca,'Units',old_ax_units);
               this_axes = gca;
               if get(graphbeta.colorbar,'Value')==1;
                  this_colorbar = colorbar_fcn;
                  set(this_colorbar,'Tag','colorbar');
                  setaxesfonts(this_colorbar);
                  colorbar_pos = get(this_colorbar,'Position');
                  set(this_colorbar,'Position',[colorbar_pos(1)+(2*colorbar_pos(3)/3) colorbar_pos(2) colorbar_pos(3)/3 colorbar_pos(4)]);
                  set(this_colorbar,'UserData',this_axes);
               end;
               set(this_light,'UserData',this_surf);
               toolbar_fcn(this_axes);
               hold('off');
               if curr_ini.external==0;
                  set(thisplot,'Visible','on');
               else;
                  file = [bm.outputdir 'lnl-beta-fixed-' num2str(xbeta(1)) '.' num2str(xbeta(2)) '-beta-' num2str(ybeta(1)) '.' num2str(ybeta(2))];
                  mstat = autosavefigure(thisplot,file);
                  if mstat==0;
                     autoshowfigure(file);
                  end;
               end;
            end;
         else;
            txt = 'You selected the same beta parameter for the X and the Y axis. Please select another value for one of the axes or select the 1 beta parameter figure type.';
            set(betagraphgui,'Visible','off');
            drawnow;
            about(txt,'information','Remake selections...',120,500);
            set(betagraphgui,'Visible','on');
            drawnow;
         end;
         bm.myrunning = 0;
      end;
   case 'type'
      if bm.myrunning==0;
         bm.myrunning = 1;
         if bm.showy==1;
            mytype = get(graphbeta.type,'Value');
            if mytype==2;
               set(graphbeta.yaxistext,'Enable','on');
               set(graphbeta.yaxis,'Enable','on');
               set(graphbeta.minytext,'Enable','on');
               set(graphbeta.maxytext,'Enable','on');
               set(graphbeta.gridytext,'Enable','on');
               set(graphbeta.miny,'Enable','on');
               set(graphbeta.maxy,'Enable','on');
               set(graphbeta.gridy,'Enable','on');
               set(graphbeta.shading,'Enable','on');
               set(graphbeta.colorbar,'Enable','on');
            else;
               set(graphbeta.yaxistext,'Enable','off');
               set(graphbeta.yaxis,'Enable','off');
               set(graphbeta.minytext,'Enable','off');
               set(graphbeta.maxytext,'Enable','off');
               set(graphbeta.gridytext,'Enable','off');
               set(graphbeta.miny,'Enable','off');
               set(graphbeta.maxy,'Enable','off');
               set(graphbeta.gridy,'Enable','off');
               set(graphbeta.shading,'Enable','off','Value',0);
               set(graphbeta.colorbar,'Enable','off','Value',0);
            end;
         end;
         bm.myrunning = 0;
      end;
   case 'done';
      if bm.myrunning==0;
         set(graphbeta.done,'UserData','done');
      end;
   case 'exit'
      if bm.myrunning==0;
         set(graphbeta.done,'UserData','done');
%         if strcmp(get(graphbeta.graph,'UserData'),'done')==1;
%            delete(betagraphgui);
%         end;
      end;
end;

if strcmp(selector,'init')==1;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(betagraphgui,'Name') ' ' fix_filepath([pwd '\icons\matfile.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(graphbeta.icon,'UserData',pid);
      end;
   end;
   set(betagraphgui,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(betagraphgui,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(graphbeta.done);
   end;
end;

if strcmp(selector,'exit')==0;
   curr_ini.models = bm;
   set(graphbeta.type,'UserData',curr_ini);
end;

if myexit==0;
   waitfor(graphbeta.done,'UserData','done');
end;

if nargout==1;
   myhandle = betagraphgui;
end;

%
% end of betagraphdlg.m
%
