function [nboot,nPsi,nalpha,nbeta,nepsilon,nSigma_theta,sample_status] = bootstrap_restrict(tm,curr_ini,boot,Epr,Omega_sig,my_r,my_T,Theta,beta,R,r_vec)

%
% creates bootstrap data which are consistent with general
% linear restrictions on Psi and alpha conditional on beta
% and estimates the original model on these new bootstrap data
%

%
% for speed reasons we don't reestimate beta along with the restrictions
% on Psi and alpha
%

[theta,S_theta,infomat,abort,inv_omega,my_status] = linrest(boot.Z0,boot.Z3,Theta,R,r_vec);
if isempty(theta)==1;
   theta = Theta;
end;
%
% handle restrictions on C
%
if (my_r==tm.r)&(size(tm.restrict,2)==3)&(tm.restrict(1,3)==1);
   file = strcat(tm.modelsdir,'C-',int2str(tm.n),'x',int2str(tm.r),'-',tm.modelfile,'.rest');
   G = loaddata(file);
   [Rmat_C,rvec_C] = rmatrix(G,tm.n,tm.n);
   [theta,Sigma_theta,dbeta,stdbeta,C,Sigma_C,num_c_rest,num_a_rest,infomat,abort] = c_restrict(boot.Z0,boot.Z1,boot.Z2,tm.restrict,beta,theta,Rmat_C,rvec_C,tm.p,tm.pe,tm.d,tm.dcoint,tm.q0,tm.q1,tm.modelsdir,tm.modelfile,0,0,R,r_vec);
end;
%
newbeta = beta;
g2 = size(theta,2);
if my_r>0;
   alphar = theta(:,g2-my_r+1:g2);
else;
   alphar = [];
end;
if g2>my_r;
   Psir = theta(:,1:g2-my_r);
else;
   Psir = [];
end;
%
% create data based on Psir, alphar, newbeta, and newOmega
%
neweps = boot.Z0-theta*boot.Z3;
newOmega = (1/my_T)*neweps*neweps';
if curr_ini.symmetric==0;
   nOmega_sig = chol(newOmega)';
else;
   nOmega_sig = symm(newOmega);
end;
nEpr = nOmega_sig*(Omega_sig\Epr);
mm = tm;
mm.alpha = alphar;
mm.Psi = Psir;
mm.T = my_T;
mm.Z0 = boot.Z0;
mm.Z1 = boot.Z1;
mm.Z2 = boot.Z2;
nboot = bootstrap_sample(mm,newbeta,nEpr);
%
% Estimate Psi, alpha, and beta based on nboot data
%
%
% computing parameters
%
[S_00,S_01,S_10,S_11,M_02,M_22,M_12,nbeta,sample_status] = boot_mlcointbasic(nboot.Z0,nboot.Z1,nboot.Z2,tm.n);
%
% check if nboot sample is proper
%
if sample_status==1;
   nPsi = [];
   nalpha = [];
   nbeta = [];
   nepsilon = [];
   nSigma_theta = [];
   return;
end;
if my_r>0;
   nbeta = nbeta(:,1:my_r);
   check_nan = sum(isnan(vec(nbeta)));
   check_ana = sum(isnan(vec(nbeta'*S_11*nbeta)));
   if (my_r==tm.r)&((isalmostequal(check_nan,0)==0)|(rank(nbeta)<size(nbeta,2)));
      nbeta = tm.beta;
      S_11 = tm.S_11;
      M_02 = tm.M_02;
      M_22 = tm.M_22;
      M_12 = tm.M_12;
      S_00 = tm.S_00;
      S_01 = tm.S_01;
   elseif (my_r==tm.r)&((isalmostequal(check_ana,0)==0)|(rank(nbeta'*S_11*nbeta)<my_r));
      nbeta = tm.beta;
      S_11 = tm.S_11;
      M_02 = tm.M_02;
      M_22 = tm.M_22;
      M_12 = tm.M_12;
      S_00 = tm.S_00;
      S_01 = tm.S_01;
   end;
   if isfield(tm,'restrict')==1;
      if tm.restrict(1,2)==1;
         nbeta = boot_betarestrict(S_00,S_11,S_01,S_10,nbeta,my_T,tm.modelsdir,tm.modelfile,tm.dcoint,tm);
         check_nan = sum(isnan(vec(nbeta)));
         if (isalmostequal(check_nan,0)==0)|(rank(nbeta)<size(nbeta,2));
            nbeta = tm.beta;
            S_11 = tm.S_11;
            M_02 = tm.M_02;
            M_22 = tm.M_22;
            M_12 = tm.M_12;
            S_00 = tm.S_00;
            S_01 = tm.S_01;
         elseif (isalmostequal(check_ana,0)==0)|(rank(nbeta'*S_11*nbeta)<my_r);
            nbeta = tm.beta;
            S_11 = tm.S_11;
            M_02 = tm.M_02;
            M_22 = tm.M_22;
            M_12 = tm.M_12;
            S_00 = tm.S_00;
            S_01 = tm.S_01;
         end;
         if (tm.restrict(1,1)==1)&(tm.r==my_r);
            %
            % need to cover the case when alpha is estimated under restrictions
            %
            nalpha = (S_01*nbeta)/(nbeta'*S_11*nbeta);
            if isempty(nboot.Z2)==0;
               nPsi = (M_02/M_22)-(nalpha*nbeta'*(M_12/M_22));
               Z3 = [nboot.Z2;nbeta'*nboot.Z1];
            else;
               nPsi = [];
               Z3 = nbeta'*nboot.Z1;
            end;
            Theta = [nPsi nalpha];
            file = strcat(tm.modelsdir,'G-',int2str(tm.n),'x',int2str(my_r),'-',tm.modelfile,'.rest');
            G = loaddata(file);
            [Rmat,rvec] = rmatrix(G,tm.n,my_r);
            tm.dfalpha = size(G,1);
            if isempty(nboot.Z2)==0;
               Rmat = [zeros(tm.dfalpha,size(nboot.Z2,1)*tm.n) Rmat];
            end;
            if curr_ini.reestbeta==0;
               [theta,nSigma_theta,infomat,abort] = linrest(nboot.Z0,Z3,Theta,Rmat,rvec);
            else;
               [theta,nSigma_theta,nbeta,newstdbeta,Z3,infomat,abort] = alpha_beta_restrict(nboot.Z0,nboot.Z1,nboot.Z2,Z3,nbeta,Theta,Rmat,rvec,tm.dcoint,tm.q1,tm.modelsdir,tm.modelfile);
               check_nan = sum(isnan(vec(nbeta)));
               if (isalmostequal(check_nan,0)==0)|(rank(nbeta)<size(nbeta,2));
                  nbeta = tm.beta;
                  S_11 = tm.S_11;
                  M_02 = tm.M_02;
                  M_22 = tm.M_22;
                  M_12 = tm.M_12;
                  S_00 = tm.S_00;
                  S_01 = tm.S_01;
               end;
            end;
            g2 = size(theta,2);
            nalpha = theta(:,g2-tm.r+1:g2);
            if g2>tm.r;
               nPsi = theta(:,1:g2-tm.r);
            else;
               nPsi = [];
            end;
            nepsilon = nboot.Z0-(theta*Z3);
         else;
            nalpha = (S_01*nbeta)/(nbeta'*S_11*nbeta);
            if isempty(nboot.Z2)==0;
               nPsi = (M_02/M_22)-(nalpha*nbeta'*(M_12/M_22));
               nepsilon = nboot.Z0-(nalpha*nbeta'*nboot.Z1)-(nPsi*nboot.Z2);
            else;
               nPsi = [];
               nepsilon = nboot.Z0-(nalpha*nbeta'*nboot.Z1);
            end;
            Omega = (1/my_T)*nepsilon*nepsilon';
            nSigma_theta = [];
         end;
      else;
         nalpha = (S_01*nbeta)/(nbeta'*S_11*nbeta);
         if isempty(nboot.Z2)==0;
            nPsi = (M_02/M_22)-(nalpha*nbeta'*(M_12/M_22));
            nepsilon = nboot.Z0-(nalpha*nbeta'*nboot.Z1)-(nPsi*nboot.Z2);
         else;
            nPsi = [];
            nepsilon = nboot.Z0-(nalpha*nbeta'*nboot.Z1);
         end;
         Omega = (1/my_T)*nepsilon*nepsilon';
         nSigma_theta = [];
      end;
   else;
      nalpha = (S_01*nbeta)/(nbeta'*S_11*nbeta);
      if isempty(nboot.Z2)==0;
         nPsi = (M_02/M_22)-(nalpha*nbeta'*(M_12/M_22));
         nepsilon = nboot.Z0-(nalpha*nbeta'*nboot.Z1)-(nPsi*nboot.Z2);
      else;
         nPsi = [];
         nepsilon = nboot.Z0-(nalpha*nbeta'*nboot.Z1);
      end;
      Omega = (1/my_T)*nepsilon*nepsilon';
      nSigma_theta = [];
   end;
else;
   nbeta = [];
   nalpha = [];
   if isempty(nboot.Z2)==0;
      nPsi = M_02/M_22;
      nepsilon = nboot.Z0-(nPsi*nboot.Z2);
   else;
      nPsi = [];
      nepsilon = nboot.Z0;
   end;
   Omega = (1/my_T)*nepsilon*nepsilon';
   nSigma_theta = [];
end;

%
% handle restrictions on C
%

if (my_r==tm.r)&(size(tm.restrict,2)==3)&(tm.restrict(1,3)==1);
   [theta,nSigma_theta,nbeta,stdbeta,C,Sigma_C,num_c_rest,num_a_rest,infomat,abort] = c_restrict(nboot.Z0,nboot.Z1,nboot.Z2,tm.restrict,nbeta,[nPsi nalpha],Rmat_C,rvec_C,tm.p,tm.pe,tm.d,tm.dcoint,tm.q0,tm.q1,tm.modelsdir,tm.modelfile,0);
   check_nan = sum(isnan(vec(nbeta)));
   if (isalmostequal(check_nan,0)==0)|(rank(nbeta)<size(nbeta,2));
      nbeta = tm.beta;
      v_theta = vec([tm.Psi tm.alpha]);
      Hmat = null(R);
      h_vec = (R'/(R*R'))*r_vec;
      v_psi = (Hmat'*Hmat)\(Hmat'*(v_theta-h_vec));
      v_theta = (Hmat*v_psi)+h_vec;
      theta = vecinv(v_theta,tm.n);
      nSigma_theta = tm.Sigma_theta;
   end;
   g2 = size(theta,2);
   if tm.r>0;
      nalpha = theta(:,g2-tm.r+1:g2);
   else;
      nalpha = [];
   end;
   if g2>tm.r;
      nPsi = theta(:,1:g2-tm.r);
      if tm.r>0;
         Z3 = [nboot.Z2;nbeta'*nboot.Z1];
      else;
         Z3 = nboot.Z2;
      end;
   else;
      nPsi = [];
      Z3 = nbeta'*nboot.Z1;
   end;
   nepsilon = nboot.Z0-(theta*Z3);
end;

%
% end of bootstrap_restrict.m
%
