%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for displaying data for editing     %
%                                                                %
%          Function: cond_forecast_data_fcn                      %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       tm: structure with model info            %
%                       deter_mat: matrix with values for        %
%                           deterministic variables              %
%                       sample_string: string vector with        %
%                           sample dates                         %
%                                                                %
%          Output:                                               %
%                       h: handle                                %
%                                                                %
%          This version:        May 18, 2006                     %
%          Written by:          Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = cond_forecast_data_fcn(selector,tm,deter_mat,sample_string,name,var_names)

if strcmp(selector,'init')==1;
   tm.deter_mat = deter_mat;
   tm.temp_deter_mat = deter_mat;
   tm.sample_string = sample_string;
   tm.var_names = var_names;
   curr_ini = find_inisettings;
   mycons = getstoredimages;
   if curr_ini.use_toolbar_bgcolor==1;
      bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
      bgcolor(1) = max(0,bgcolor(1));
      bgcolor(2) = max(0,bgcolor(2));
      bgcolor(3) = max(0,bgcolor(3));
   else;
      bgcolor = [NaN NaN NaN];
   end;
   btn_images.open = maketrans(mycons.open_image,[160 160 164],bgcolor);
   btn_images.save = maketrans(mycons.save_image,[160 160 164],bgcolor);
   btn_images.saveas = maketrans(mycons.saveas_image,[160 160 164],bgcolor);
   btn_images.close = maketrans(mycons.close_image,[160 160 164],bgcolor);
   f_cast.undo = maketrans(mycons.undo_image,[160 160 164],get(0,'defaultuicontrolbackgroundcolor'));
   f_cast.undodisable = maketrans(mycons.undodisable_image,[160 160 164],get(0,'defaultuicontrolbackgroundcolor'));
   %
   curr_ini.models = tm;
else;
   ed_gui = findobj('Type','figure','Tag','EditForecastDataGUI')';
   f_cast = get(ed_gui,'UserData');
   curr_ini = get(f_cast.open,'UserData');
   tm = curr_ini.models;
end;

switch selector;
   case 'init';
      realscrsz = get(0,'ScreenSize');
      ed_gui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','cond_forecast_data_fcn.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[0 32 realscrsz(3) realscrsz(4)-84], ...
         'Tag','EditForecastDataGUI', ...
         'Visible','off', ...
         'Resize','off', ...
         'NumberTitle','off', ...
         'Name',name, ...
         'CloseRequestFcn','cond_forecast_data_fcn close', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(ed_gui,'DockControl','off');
      end;

      %
      % setting up a toolbar
      %

      f_cast.toolbar = uitoolbar('Parent',ed_gui, ...
                           'HandleVisibility','off', ...
                           'Interruptible','on', ...
                           'Clipping','on');

      f_cast.open = uipushtool('Parent',f_cast.toolbar, ...
                         'TooltipString','Open', ...
                         'CData',btn_images.open, ...
                         'ClickedCallback','cond_forecast_data_fcn open', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Userdata',curr_ini, ...
                         'Enable','on', ...
                         'Tag','Open');

      f_cast.save = uipushtool('Parent',f_cast.toolbar, ...
                         'TooltipString','Save', ...
                         'CData',btn_images.save, ...
                         'ClickedCallback','cond_forecast_data_fcn save', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','Save');

      f_cast.saveas = uipushtool('Parent',f_cast.toolbar, ...
                         'TooltipString','Save As', ...
                         'CData',btn_images.saveas, ...
                         'ClickedCallback','cond_forecast_data_fcn saveas', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','SaveAs');

      f_cast.done = uipushtool('Parent',f_cast.toolbar, ...
                         'TooltipString','Close', ...
                         'CData',btn_images.close, ...
                         'ClickedCallback','cond_forecast_data_fcn close', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'UserData','waiting', ...
                         'Enable','on', ...
                         'Tag','Done');




      f_cast.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Style','pushbutton', ...
         'Position',[20 realscrsz(4)-135 45 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CData',f_cast.undodisable, ...
         'HorizontalAlignment','right', ...
         'Callback','cond_forecast_data_fcn cancel', ...
         'ToolTipString','Cancel', ...
         'Enable','off', ...
         'Tag','topleft');

      c1 = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Style','togglebutton', ...
         'Position',[65 realscrsz(4)-135 70 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'HorizontalAlignment','right', ...
         'Callback','cond_forecast_data_fcn vertical', ...
         'String','Date', ...
         'UserData',2, ...
         'Enable','on', ...
         'Tag','topmiddle');

      f_cast.header = [f_cast.icon c1];

      for i=1:size(var_names,1);
         if 135+(70*i)<realscrsz(3)-20;
            ci = uicontrol('Units','pixels', ...
               'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
               'Style','togglebutton', ...
               'Position',[135+(70*(i-1)) realscrsz(4)-135 70 20], ...
               'FontSize',curr_ini.gui_fontsize, ...
               'FontName',curr_ini.gui_fontname, ...
               'FontWeight',curr_ini.gui_fontweight, ...
               'FontAngle',curr_ini.gui_fontangle, ...
               'HorizontalAlignment','right', ...
               'String',strtrim(var_names(i,:)), ...
               'UserData',i+2, ...
               'Callback','cond_forecast_data_fcn vertical', ...
               'Enable','on', ...
               'Tag','topmiddle');

            f_cast.header = [f_cast.header ci];
         else;
            break;
         end;
      end;

      f_cast.controls = [];
      pos_b = realscrsz(4)-155;

      for i=1:size(tm.deter_mat,2);
         if pos_b>=20;
            %
            % number controls
            %
            con1 = uicontrol('Units','pixels', ...
               'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
               'Style','togglebutton', ...
               'Position',[20 pos_b 45 20], ...
               'FontSize',curr_ini.gui_fontsize, ...
               'FontName',curr_ini.gui_fontname, ...
               'FontWeight',curr_ini.gui_fontweight, ...
               'FontAngle',curr_ini.gui_fontangle, ...
               'HorizontalAlignment','right', ...
               'String',num2str(i,'%0.0f'), ...
               'Callback','cond_forecast_data_fcn horizontal', ...
               'UserData',i, ...
               'Enable','on', ...
               'Tag','topleft');
            %
            con2 = uicontrol('Units','pixels', ...
               'BackgroundColor',[1 1 1], ...
               'Style','edit', ...
               'Position',[65 pos_b 70 20], ...
               'FontSize',curr_ini.gui_fontsize, ...
               'FontName',curr_ini.gui_fontname, ...
               'FontWeight',curr_ini.gui_fontweight, ...
               'FontAngle',curr_ini.gui_fontangle, ...
               'HorizontalAlignment','right', ...
               'String',strtrim(tm.sample_string(i,:)), ...
               'Visible','on', ...
               'Enable','off', ...
               'Tag','topmiddle');
            %
            controls = [con1 con2];
            pos_l = 135;
            for j=1:size(tm.deter_mat,1);
               if pos_l<=realscrsz(3)-90;
                  coni = uicontrol('Units','pixels', ...
                     'BackgroundColor',[1 1 1], ...
                     'Style','edit', ...
                     'Position',[pos_l pos_b 70 20], ...
                     'FontSize',curr_ini.gui_fontsize, ...
                     'FontName',curr_ini.gui_fontname, ...
                     'FontWeight',curr_ini.gui_fontweight, ...
                     'FontAngle',curr_ini.gui_fontangle, ...
                     'HorizontalAlignment','right', ...
                     'Callback','cond_forecast_data_fcn edit', ...
                     'String',num2str(tm.deter_mat(j,i)), ...
                     'Userdata',[j i], ...
                     'Visible','on', ...
                     'Enable','on', ...
                     'Tag','topmiddle');
                  last_pos_l = pos_l;
                  if pos_l+70<=realscrsz(3)-90;
                     pos_l = pos_l+70;
                  else;
                     pos_l = realscrsz(3);
                  end;
                  controls = [controls coni];
               else;
                  break;
               end;
            end;
            
            f_cast.controls = [f_cast.controls;controls];

            last_pos_b = pos_b;
            if pos_b-20>=20;
               pos_b = pos_b-20;
            else;
               pos_b = 0;
            end;
         else;
            break;
         end;
      end;

      if (size(f_cast.controls)-[0 2])==size(tm.deter_mat');
         width = (70*(size(f_cast.controls,2)-1))+85;
         if width<300;
            diff_w = (300-width)/2;
            width = 300;
         else;
            diff_w = 0;
         end;
         height = ((size(f_cast.controls,1)+1)*20)+40;
         new_pos = [(realscrsz(3)-width)/2 (realscrsz(4)-height)/2 width height];
         set(ed_gui,'Position',new_pos);
         %
         % Adjust all controls vertically
         %
         l_c = size(f_cast.controls,1);
         b_c = size(f_cast.controls,2);
         f_pos = 20;
         for i=1:l_c;
            for j=1:b_c;
               new_pos = get(f_cast.controls(l_c-i+1,j),'Position');
               new_pos(1,1) = new_pos(1,1)+diff_w;
               new_pos(1,2) = f_pos;
               set(f_cast.controls(l_c-i+1,j),'Position',new_pos);
            end;
            f_pos = f_pos+20;
         end;
         for j=1:b_c;
            new_pos = get(f_cast.header(1,j),'Position');
            new_pos(1,1) = new_pos(1,1)+diff_w;
            new_pos(1,2) = f_pos;
            set(f_cast.header(1,j),'Position',new_pos);
         end;
         xline([1 height-1 width],[0.5 1]);
      else;
         %
         % we should add a horizontal and a vertical slider (or both)
         %
         % First determine the current area for the controls
         %
         base_pos = get(f_cast.controls(size(f_cast.controls,1),size(f_cast.controls,2)),'Position');
         l_pos_v = base_pos(1)+base_pos(3);
         b_pos_v = base_pos(2);
         h_pos_v = 20*(size(f_cast.controls,1)+1);
         base_pos = get(f_cast.controls(size(f_cast.controls,1),1),'Position');
         l_pos_h = base_pos(1);
         b_pos_h = base_pos(2)-14;
         w_pos_h = l_pos_v-l_pos_h;
         add_width = 0;
         add_height = 0;
         if size(f_cast.controls,1)<size(tm.deter_mat,2);
            f_cast.v_slider = uicontrol('Units','pixels', ...
               'BackgroundColor',(get(0,'defaultuicontrolbackgroundcolor')+[1 1 1])/2, ...
               'Position',[l_pos_v b_pos_v 14 h_pos_v], ...
               'Style','slider', ...
               'Min',size(f_cast.controls,1), ...
               'Max',size(tm.deter_mat,2), ...
               'Value',size(tm.deter_mat,2), ...
               'Callback','cond_forecast_data_fcn slider_vertical', ...
               'Enable','on', ...
               'Tag','Vertical');
            %
            % Set the slider step
            %
            if 10/(size(tm.deter_mat,2)-size(f_cast.controls,1))<=1;
               set(f_cast.v_slider,'SliderStep',[1/(size(tm.deter_mat,2)-size(f_cast.controls,1)) 10/(size(tm.deter_mat,2)-size(f_cast.controls,1))]);
            else;
               if 2/(size(tm.deter_mat,2)-size(f_cast.controls,1))<=1;
                  set(f_cast.v_slider,'SliderStep',[1/(size(tm.deter_mat,2)-size(f_cast.controls,1)) 2/(size(tm.deter_mat,2)-size(f_cast.controls,1))]);
               else;
                  set(f_cast.v_slider,'SliderStep',[0.9 1.0],'UserData',1);
               end;
            end;
            add_width = 14;
         end;
         if size(f_cast.controls,2)<size(tm.deter_mat,1);
            f_cast.h_slider = uicontrol('Units','pixels', ...
               'BackgroundColor',(get(0,'defaultuicontrolbackgroundcolor')+[1 1 1])/2, ...
               'Position',[l_pos_h b_pos_h w_pos_h 14], ...
               'Style','slider', ...
               'Min',size(f_cast.controls,2)-2, ...
               'Max',size(tm.deter_mat,1), ...
               'Value',size(f_cast.controls,2)-2, ...
               'Callback','cond_forecast_data_fcn slider_horizontal', ...
               'Enable','on', ...
               'Tag','Horizontal');
            %
            % set the slider step
            %
            if 10/(size(tm.deter_mat,1)-size(f_cast.controls,2)+2)<=1;
               set(f_cast.h_slider,'SliderStep',[1/(size(tm.deter_mat,1)-size(f_cast.controls,2)+2) 10/(size(tm.deter_mat,1)-size(f_cast.controls,2)+2)]);
            else;
               if 2/(size(tm.deter_mat,1)-size(f_cast.controls,2)+2)<=1;
                  set(f_cast.h_slider,'SliderStep',[1/(size(tm.deter_mat,1)-size(f_cast.controls,2)+2) 2/(size(tm.deter_mat,1)-size(f_cast.controls,2)+2)]);
               else;
                  set(f_cast.h_slider,'SliderStep',[0.9 1.0],'UserData',1);
               end;
            end;
            add_height = 14;
         end;
         %
         % now we need to adjust all controls
         %
         width = (70*(size(f_cast.controls,2)-1))+85+add_width;
         if width<300;
            diff_w = (300-width)/2;
            width = 300;
         else;
            diff_w = 0;
         end;
         height = ((size(f_cast.controls,1)+1)*20)+40+add_height;
         if height>realscrsz(4)-84;
            height = realscrsz(4)-84;
         end;
         new_pos = [(realscrsz(3)-width)/2 (realscrsz(4)-height)/2 width height];
         set(ed_gui,'Position',new_pos);
         %
         % first fix the horizontal slider
         %
         f_pos = 20;
         if isfield(f_cast,'h_slider')==1;
            s_pos = get(f_cast.h_slider,'Position');
            s_pos(2) = 20;
            s_pos(1) = s_pos(1)+diff_w;
            set(f_cast.h_slider,'Position',s_pos);
            f_pos = 34;
         end;
         %
         % next fix the vertical slider
         %
         if isfield(f_cast,'v_slider')==1;
            s_pos = get(f_cast.v_slider,'Position');
            s_pos(2) = f_pos;
            s_pos(1) = s_pos(1)+diff_w;
            set(f_cast.v_slider,'Position',s_pos);
         end;
         %
         % now fix all f_cast.controls controls
         %
         l_c = size(f_cast.controls,1);
         b_c = size(f_cast.controls,2);
         for i=1:l_c;
            for j=1:b_c;
               new_pos = get(f_cast.controls(l_c-i+1,j),'Position');
               new_pos(1,1) = new_pos(1,1)+diff_w;
               new_pos(1,2) = f_pos;
               set(f_cast.controls(l_c-i+1,j),'Position',new_pos);
            end;
            f_pos = f_pos+20;
         end;
         for j=1:b_c;
            new_pos = get(f_cast.header(1,j),'Position');
            new_pos(1,1) = new_pos(1,1)+diff_w;
            new_pos(1,2) = f_pos;
            set(f_cast.header(1,j),'Position',new_pos);
         end;

         xline([1 height-1 width],[0.5 1]);
      end;

      set(ed_gui,'UserData',f_cast,'HandleVisibility','callback');

   case 'slider_horizontal'
      slider_val = get(f_cast.h_slider,'Value');
      if isempty(get(f_cast.h_slider,'UserData'))==0;
         if isequal(slider_val,round(slider_val))==0;
            slider_val = round(slider_val);
            set(f_cast.h_slider,'Value',slider_val);
         end;
      end;
      if isint(slider_val)==0;
         slider_val = round(slider_val);
      end;
      left_con = slider_val-size(f_cast.controls,2)+3;
      right_con = size(f_cast.controls,2)+left_con-3;
      first_con = mystr2num(get(f_cast.controls(1,1),'String'));
      last_con = mystr2num(get(f_cast.controls(size(f_cast.controls,1),1),'String'));
      for j=left_con:right_con;
         set(f_cast.header(1,j-left_con+3),'String',strtrim(tm.var_names(j,:)),'UserData',j);
         for i=first_con:last_con;
            set(f_cast.controls(i-first_con+1,j-left_con+3),'String',num2str(tm.deter_mat(j,i)),'UserData',[j i]);
         end;
      end;
   case 'slider_vertical'
      slider_val = get(f_cast.v_slider,'Value');
      if isempty(get(f_cast.v_slider,'UserData'))==0;
         if isequal(slider_val,round(slider_val))==0;
            slider_val = round(slider_val);
            set(f_cast.v_slider,'Value',slider_val);
         end;
      end;
      if isint(slider_val)==0;
         slider_val = round(slider_val);
      end;
      first_con = size(tm.deter_mat,2)-slider_val+1;
      last_con = size(f_cast.controls,1)+first_con-1;
      left_con = get(f_cast.header(1,3),'UserData');
      right_con = size(f_cast.controls,2)+left_con-3;
      for i=first_con:last_con;
         set(f_cast.controls(i-first_con+1,1),'String',num2str(i,'%0.0f'));
         set(f_cast.controls(i-first_con+1,2),'String',strtrim(tm.sample_string(i,:)));
         %
         for j=left_con:right_con;
            set(f_cast.controls(i-first_con+1,j-left_con+3),'String',num2str(tm.deter_mat(j-2,i)),'UserData',[j-2 i]);
         end;
      end;
   case 'edit'
      curr_obj = gcbo;
      new_val = strtrim(get(curr_obj,'String'));
      %
      % allow for mathematical expressions +-/*
      %
      ud = get(curr_obj,'UserData');
      old_val = num2str(tm.deter_mat(ud(1),ud(2)),'%0.16f');
      if (isempty(str2math(new_val))==0)&(strcmp(new_val,old_val)==0);
         tm.temp_deter_mat(ud(1),ud(2)) = str2math(new_val);
         set(f_cast.icon,'CData',f_cast.undo,'Enable','on');
      elseif isempty(mystr2num(new_val))==1;
         set(curr_obj,'String',old_val);
      end;
      curr_ini.models = tm;
      set(f_cast.open,'UserData',curr_ini);
    case 'cancel'
      tm.temp_deter_mat = tm.deter_mat;
      %
      % reset all visible edit controls
      %
      num = get(f_cast.controls(1,1),'UserData');
      l_c = size(f_cast.controls,1);
      b_c = size(f_cast.controls,2);
      for i=1:l_c;
         set(f_cast.controls(i,2),'String',strtrim(tm.sample_string(i,:)));
         for j=3:b_c;
            set(f_cast.controls(i,j),'String',num2str(tm.deter_mat(j-2,i)));
         end;
      end;
      curr_ini.models = tm;
      set(f_cast.open,'UserData',curr_ini);
      set(f_cast.icon,'CData',f_cast.undodisable,'Enable','off');
   case 'horizontal'
      curr_obj = gco;
      curr_num = get(curr_obj,'UserData');
      curr_val = get(curr_obj,'Value');
      for i=1:size(f_cast.controls,1);
         set(f_cast.controls(i,1),'Value',0);
         set(f_cast.controls(i,:),'FontWeight','normal');
      end;
      if curr_val==1;
         set(f_cast.controls(curr_num,1),'Value',1);
         set(f_cast.controls(curr_num,:),'FontWeight','bold');
         set(f_cast.header,'Value',0,'FontWeight','normal');
      else;
         set(f_cast.controls(curr_num,:),'FontWeight','normal');
      end;
      curr_ini.models = tm;
      set(f_cast.open,'UserData',curr_ini);
   case 'vertical'
      curr_obj = gco;
      curr_num = get(curr_obj,'UserData');
      if isfield(f_cast,'h_slider')==1;
         %
         % need to adjust for the slider value
         %
         curr_num = sum((f_cast.header==curr_obj) .* (1:1:size(f_cast.header,2)));
      end;
      curr_val = get(curr_obj,'Value');
      for i=1:size(f_cast.controls,1);
         set(f_cast.controls(i,1),'Value',0);
         set(f_cast.controls(i,:),'FontWeight','normal');
      end;
      if curr_val==1;
         set(f_cast.header,'Value',0,'FontWeight','normal');
         set(curr_obj,'Value',1,'FontWeight','bold');
         set(f_cast.controls(:,curr_num),'FontWeight','bold');
      else;
         set(curr_obj,'FontWeight','normal');
      end;
      curr_ini.models = tm;
      set(f_cast.open,'UserData',curr_ini);
   case 'open'
      if matlabnumber>=6.0;
         inputfiletype = cell(1,2);
         inputfiletype{1,1} = '*.mat';
         inputfiletype{1,2} = 'MatLab MAT Files (*.mat)';
      else;
         if (iscompiled==1)&(iswinnt==1)&(curr_ini.opendialog==1);
            basedir = tm.modelsdir;
            file_fltr = 'MatLab MAT Files (*.mat)|*.mat|';
            best_pos = 1;
            title_txt = 'Open';
         else;
            inputfiletype = [tm.modelsdir '*.mat'];
         end;
      end;
      if (iscompiled==1)&(iswinnt==1)&(curr_ini.opendialog==1)&(matlabnumber<6.0);
         [openfile,openpath] = fileopen(basedir,file_fltr,best_pos,title_txt);
      else;
         if matlabnumber>=6;
            [openfile,openpath] = uigetfile(inputfiletype,'Open',getpath(tm.endofile));
         else;
            [openfile,openpath] = uigetfile(inputfiletype,'Open');
         end;
      end;
      drawnow;
      if isnumeric(openfile)==0;
         file = strcat(openpath,openfile);
         %
         % check if file can be used
         %
         if strcmp(lower(getext(file)),'.mat')==1;
            a = load(file);
            if isfield(a,'deter_mat')==1;
               new_deter_mat = a.deter_mat;
               if size(new_deter_mat)==size(tm.deter_mat);
                  %
                  % The data has passed all tests. Now we update the controls and variables
                  %
                  tm.deter_mat = new_deter_mat;
                  tm.temp_deter_mat = new_deter_mat;
                  set(f_cast.icon,'CData',f_cast.undodisable,'Enable','off');
                  if isfield(f_cast,'h_slider')==1;
                     %
                     % reset horizontal slider
                     %
                     set(f_cast.h_slider,'Value',size(tm.deter_mat,2));
                     for i=1:size(f_cast.header,2)-2;
                        set(f_cast.header(1,i+2),'String',strtrim(tm.var_names(i,:)),'UserData',i+2);
                     end;
                  end;
                  if isfield(f_cast,'v_slider')==1;
                     %
                     % reset vertical slider
                     %
                     set(f_cast.v_slider,'Value',size(f_cast.controls,2)-2);
                     for i=1:size(f_cast.controls,1);
                        set(f_cast.controls(i,1),'String',num2str(i,'%0.0f'));
                        set(f_cast.controls(i,2),'String',strtrim(tm.sample_string(i,:)));
                     end;
                  end;
                  for i=1:size(f_cast.controls,1);
                     for j=1:size(f_cast.controls,2)-2;
                        set(f_cast.controls(i,j+2),'String',num2str(tm.deter_mat(j,i)),'UserData',[j i]);
                     end;
                  end;
                  drawnow;
                  %
                  % save the new data so it can be used directly
                  %
                  deter_mat = tm.temp_deter_mat;
                  file = [tm.modelsdir tm.modelfile '-deter-forecast-' num2str(tm.datad,'%0.0f') '-' num2str(size(deter_mat,2),'%0.0f') '.mat'];
                  save(file,'deter_mat');
               else;
                  txt = ['The data in "' getname(file) '" does not match the dimensions you require. The data you requested has ' num2str(size(new_deter_mat,1),'%0.0f') ' variables and ' num2str(size(new_deter_mat,2),'%0.0f') ' observations, while you require ' num2str(size(tm.deter_mat,1),'%0.0f') ' variables and ' num2str(size(tm.deter_mat,2),'%0.0f') ' observations. Please try again!'];
                  about(txt,'information','Bad Data',120,500);
               end;
            else;
               txt = ['The file "' getname(file) '" does not contain valid data for the future observations for the deterministic variables. Please try again!'];
               about(txt,'information','Bad Data',120,500);
            end;
         else;
            txt = ['The file you selected "' getname(file) '" is not a MatLab MAT file. Structural VAR can only open created deterministic data from such files. Please try again!'];
            about(txt,'information','Bad File Type',120,500);
         end;
      end;
      curr_ini.models = tm;
      set(f_cast.open,'UserData',curr_ini);
   case 'save'
      if (curr_ini.sillyq==0)&(strcmp(computer,'PCWIN')==1);
         txt = 'Please note that Structural VAR can only save updated data. If you have edited any data and the cursor was still blinking in an edit control when you hit the "Save" button, that data has not been updated yet. Are you sure you want to save the data?';
         answer = query(txt,'question',120,get(ed_gui,'Name'),500,'no');
         drawnow;
      else;
         answer = 'Yes';
      end;
      if strcmp(answer,'Yes')==1;
         deter_mat = tm.temp_deter_mat;
         file = [tm.modelsdir tm.modelfile '-deter-forecast-' num2str(tm.datad,'%0.0f') '-' num2str(size(deter_mat,2),'%0.0f') '.mat'];
         save(file,'deter_mat');
      end;
   case 'saveas'
      if (curr_ini.sillyq==0)&(strcmp(computer,'PCWIN')==1);
         txt = 'Please note that Structural VAR can only save updated data. If you have edited any data and the cursor was still blinking in an edit control when you hit the "Save" button, that data has not been updated yet. Are you sure you want to save the data?';
         answer = query(txt,'question',120,get(ed_gui,'Name'),500,'no');
         drawnow;
      else;
         answer = 'Yes';
      end;
      if strcmp(answer,'Yes')==1;
         if matlabnumber>=6.0;
            inputfiletype = cell(1,2);
            inputfiletype{1,1} = '*.mat';
            inputfiletype{1,2} = 'MatLab MAT Files (*.mat)';
         else;
            inputfiletype = [tm.modelsdir '*.mat'];
         end;
         [filename,filepath] = uiputfile(inputfiletype,'Save As');
         drawnow;
         if isnumeric(filename)==0;
            file = [filepath filename];
            my_ext = getext(file);
            if strcmp(lower(my_ext),'.mat')==0;
               file = [file '.mat'];
            end;
            deter_mat = tm.temp_deter_mat;
            save(file,'deter_mat');
         end;
      end;
   case 'close'
      set(f_cast.done,'UserData','done');
end;

if nargout==1;
   h = ed_gui;
end;

if strcmp(selector,'init')==1;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(ed_gui,'Name') ' ' fix_filepath([pwd '\icons\spreadsheet.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(f_cast.icon,'UserData',pid);
      end;
   end;
   %
   set(ed_gui,'Visible','on');
   drawnow;
   drawnow;
   waitfor(f_cast.done,'UserData','done');
end;

%
% end of cond_forecast_data_fcn.m
%
