function [test,deg_free,pval,infomat,abort,Sigma_theta,Psi_n,alpha_n,Sigma_xi_orth,xi_orth,beta,stdbeta] = estimate_common_cycles(Z0,Z1,Z2,epsilon,alpha,Psi,beta,p,pe,q1,q0,dcoint,d,df,modelsdir,modelfile,restrict,convcrit,maxit,reestbeta,sscorrect,max_q,min_q,waitbar)

test = zeros(max_q-min_q+1,1);
pval = zeros(max_q-min_q+1,1);
deg_free = zeros(max_q-min_q+1,1+sscorrect);
infomat = [];
Sigma_theta = [];
Sigma_xi_orth = [];
xi_orth = [];
stdbeta = zeros(size(beta));

%
% Model: Z0 = gamma*Z2 + xi*eta'*Z1 + epsilon
%
% where Z2 contains the cointegration relations and the deterministics, while
% Z1 includes all lagged changes and exogenous I(d) variables.
%

[n T] = size(Z0);
r = size(alpha,2);

if d>0;
   Z2_new = Z2(1:d,:);
   Z1_new = Z2(d+1:size(Z2,1),:);
else;
   Z2_new = [];
   Z1_new = Z2;
end;

if r>0;
   Z1_new = [Z1_new;beta'*Z1];
end;

if convcrit<10;
   ch_convcrit = ['1.0e-00' num2str(convcrit)];
else;
   ch_convcrit = ['1.0e-0' num2str(convcrit)];
end;
ccrit = mystr2num(ch_convcrit);
      
%
% the rank of xi*eta' varies from cm.n to cm.r. This corresponds to 0 to cm.r-cm.n
% common cyles, where the common cycles parameters are given by xi_orth.
%
% The max_q and min_q parameters determines the highest and lowest rank that we can estimate under.
% If they're equal, then we only have one model to consider
%
num = 1;
k = size(Z1_new,1);

J_k = [];
if d>0;
   J_d = [eye(d) zeros(d,k)];
   J_k = zeros(k,d);
end;
J_k = [J_k eye(k)];


if (restrict(1,1)==1)|((size(restrict,2)>2)&(restrict(1,3)==1))|((reestbeta==1)&(r>0));
   %
   % set up a waitbar
   %
   if waitbar==1;
      show_waitbar = 1;
   else;
      show_waitbar = 0;
   end;
else;
   show_waitbar = 0;
end;

if restrict(1,1)==1;
   %
   % The restrictions on alpha
   %
   file = strcat(modelsdir,'G-',int2str(n),'x',int2str(r),'-',modelfile,'.rest');
   G = loaddata(file);
   [Rmat,rvec] = rmatrix(G,n,r);
   dfalpha = size(G,1);
   Rmat = [zeros(dfalpha,(k-r)*n) Rmat];
else;
   Rmat = [];
end;

if (size(restrict,2)>2)&(restrict(1,3)==1);
   file = strcat(modelsdir,'C-',int2str(n),'x',int2str(r),'-',modelfile,'.rest');
   G = loaddata(file);
   [Rmat_C,rvec_C] = rmatrix(G,n,n);
end;

if (restrict(1,2)==1)&(reestbeta==1)&(r>0);
   [h_b,H_b] = vecbetarel(beta,n,r,dcoint,q1,modelsdir,modelfile);
end;
%
% some product moment matrices
%
M01 = (1/T)*Z0*Z1';
M10 = M01';
M11 = (1/T)*Z1*Z1';
M21 = (1/T)*Z2*Z1';
M12 = M21';
M22 = (1/T)*Z2*Z2';

abort = '';
for i=min_q:max_q;
   grad = 1;
   it = 1;
   old_grad = 0;
   if show_waitbar==1;
      curr_ini = find_inisettings;
      input_var.title = ['Progress for maximum of ' int2str(maxit) ' iterations'];
      input_var.facecolor = curr_ini.progress_facecolor;
      input_var.startfacecolor = curr_ini.progress_startfacecolor;
      input_var.edgecolor = curr_ini.progress_edgecolor;
      input_var.bgcolor = curr_ini.progress_bgcolor;
      input_var.stop = 1;
      input_var.clock = 0;
      input_var.name = ['ML Estimation of ' num2str(i,'%0.0f') ' Common Cycles'];
      mywait = progressdlg(0,input_var);
%      set(mywait,'Name',['ML Estimation of ' num2str(i,'%0.0f') ' Common Cycles']);
      set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
      drawnow;
   end;
   while (it<=maxit)&(ccrit<grad);
      if show_waitbar==1;
         abort = get(mywait,'UserData');
         if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
            break;
         else;
            progressdlg([it/maxit grad]);
         end;
      end;
      %
      % run the inital reduced rank regression to obtain values for xi and eta conditional on beta.
      %
      [S_00,S_01,S_10,S_11,M_02,M_22,M_12,eta,lambda,m_trace] = mlcointbasic(Z0,Z1_new,Z2_new,i);
      %
      % find a basis for eta
      %
      rows = [];
      eta_h = eta;
      for j=1:i;
         [t,ind] = max(abs(eta_h(:,j)));
         rows = [rows;ind];
         eta_h(ind,:) = zeros(1,i);
      end;
      %
      H = eye(k);
      H = H(:,sort(rows));
      H_orth = null(H','r');
      eta = eta*inverse(H'*eta);
      psi = H_orth'*eta;
      eta = H+(H_orth*psi);
      %
      Mzz = (1/T)*[Z2_new;Z1_new]*[Z2_new;Z1_new]';
      Mzx = (1/T)*[Z2_new;Z1_new]*Z0';
      %
      % determine the unrestricted xi and gamma
      %
      xi = (S_01*eta)*inverse(eta'*S_11*eta);
      if isempty(Z2_new)==0;
         gamma = (M_02*inverse(M_22))-(xi*eta'*M_12*inverse(M_22));
         epsilon_n = Z0-(xi*eta'*Z1_new)-(gamma*Z2_new);
      else;
         gamma = [];
         epsilon_n = Z0-(xi*eta'*Z1_new);
      end;
      Omega = (1/T)*epsilon_n*epsilon_n';
      %
      % Determine Psi and alpha from these initial estimates
      %
      Phi = xi*eta';
      if r>0;
         alpha_n = Phi(:,k-r+1:k);
         Phi = Phi(:,1:k-r);
      else;
         alpha_n = [];
      end;
      Psi_n = [gamma Phi];
      Omega_n = Omega;
      eta_n = eta;
      xi_n = xi;
      psi_n = psi;
      gamma_n = gamma;
      %
      % estimate under restrictions on alpha if r>0 and there are no C restrictions
      %
      if (restrict(1,1)==1)&((size(restrict,2)==2)|(restrict(1,3)==0));
         Kki = commutation(k-i,i);
         %
         % The matrix of partial derivatives of vec([gamma xi*eta']) wrt [vec(gamma),vec(xi),vec(psi)]
         %
         G_mat = [];
         if d>0;
            G_mat = kron(J_d',eye(n));
         end;
         H_mat = [kron(eta,eye(n)) kron(H_orth,xi)*Kki];
         G_mat = [G_mat kron(J_k',eye(n))*H_mat];
         %
         % create adjustments of G_mat and H_mat
         %
         adj_G_mat = ones(size(G_mat,1),1);
         g_1 = G_mat*[vec(gamma);vec(xi);vec(psi)];
         g_2 = vec([gamma xi*eta']);
         for j=1:size(g_1,1);
            if abs(g_2(j,1))>eps;
               adj_G_mat(j,1) = g_1(j,1)/g_2(j,1);
            end;
         end;
         F_mat = inverse(diag(adj_G_mat))*G_mat;
         adj_H_mat = ones(size(H_mat),1);
         h_1 = H_mat*[vec(xi);vec(psi)];
         h_2 = vec(xi*eta');
         for j=1:size(h_1,1);
            if abs(h_2(j,1))>eps;
               adj_H_mat(j,1) = h_1(j,1)/h_2(j,1);
            end;
         end;
         %
         % The matrix of partials of the restrictions on (Psi and) alpha wrt [vec(gamma),vec(xi),vec(psi)]
         %
%         R_mat = Rmat*inverse(diag(adj_H_mat))*H_mat;
         R_mat = Rmat*H_mat;
         if d>0;
            R_mat = [zeros(dfalpha,n*d) R_mat];
         end;
         %
         % Determine the unrestricted estimate of theta = [vec(gamma),vec(xi),vec(psi)] through
         % the adjusted G_mat matrix denoted by F_mat above. Notice that the adjustments involve
         % a diagonal matrix with 1 or 1/2 along the diagonal
         %
         theta_p1 = G_mat'*kron(Mzz,inverse(Omega))*F_mat;
         theta_p11 = G_mat'*kron(Mzz,inverse(Omega))*G_mat;
         theta_p2 = G_mat'*vec(inverse(Omega)*Mzx');
         theta_hat = inverse(theta_p1)*theta_p2;
         %
         % this theta_hat is equal to vec(gamma);vec(xi);vec(psi)
         %
         %
         % begin the loop
         %
         f_vec = (Rmat*vec(xi*eta'))-rvec;
         part_1 = inverse(theta_p11)*R_mat';
         part_2 = R_mat*part_1;
         it_j = 1;
         theta_bar = theta_hat;
         conv_crit = max(abs(f_vec));
         while (it_j<=maxit)&(ccrit<conv_crit);
            theta_bar = theta_bar-(part_1*inverse(part_2)*f_vec);
            %
            % determine the xi, eta, and psi parameters
            %
            xi_n = vecinv(theta_bar((n*d)+1:n*(d+i),1),n);
            psi_n = vecinv(theta_bar((n*(d+i))+1:size(theta_bar,1),1),k-i);
            eta_n = H+(H_orth*psi_n);
            %
            % evaluate the restrictions on alpha
            %
            f_vec = (Rmat*vec(xi_n*eta_n'))-rvec;
            conv_crit = max(abs(f_vec));
            %
            % update matrices in theta_bar expression
            %
            if d>0;
               gamma_n = vecinv(theta_bar(1:n*d,1),n);
            end;
            if isempty(Z2_new)==0;
               epsilon_n = Z0-(xi_n*eta_n'*Z1_new)-(gamma_n*Z2_new);
            else;
               epsilon_n = Z0-(xi_n*eta_n'*Z1_new);
            end;
            Omega_n = (1/T)*epsilon_n*epsilon_n';
            %
            G_mat = [];
            if d>0;
               G_mat = kron(J_d',eye(n));
            end;
            H_mat = [kron(eta,eye(n)) kron(H_orth,xi)*Kki];
            G_mat = [G_mat kron(J_k',eye(n))*H_mat];
            F_mat = inverse(diag(adj_G_mat))*G_mat;
%            R_mat = Rmat*inverse(diag(adj_H_mat))*H_mat;
            R_mat = Rmat*H_mat;
            if d>0;
               R_mat = [zeros(dfalpha,n*d) R_mat];
            end;
            theta_p1 = G_mat'*kron(Mzz,inverse(Omega_n))*F_mat;
            theta_p11 = G_mat'*kron(Mzz,inverse(Omega_n))*G_mat;
            part_1 = inverse(theta_p11)*R_mat';
            part_2 = R_mat*part_1;
            it_j = it_j+1;
         end;
         %
         % collect values into gamma, Psi and alpha
         %
         if d>0;
            gamma_n = vecinv(theta_bar(1:n*d,1),n);
         end;
         Phi_n = xi_n*eta_n';
         if r>0;
            alpha_n = Phi_n(:,k-r+1:k);
            Phi_n = Phi_n(:,1:k-r);
         else;
            alpha_n = [];
         end;
         Psi_n = [gamma_n Phi_n];
         if isempty(Z2_new)==0;
            epsilon_n = Z0-(xi_n*eta_n'*Z1_new)-(gamma_n*Z2_new);
         else;
            epsilon_n = Z0-(xi_n*eta_n'*Z1_new);
         end;
         Omega_n = (1/T)*epsilon_n*epsilon_n';
      elseif (size(restrict,2)>2)&(restrict(1,3)==1);
         Kki = commutation(k-i,i);
         %
         % The matrix of partial derivatives of vec([gamma xi*eta']) wrt [vec(gamma),vec(xi),vec(psi)]
         %
         G_mat = [];
         if d>0;
            G_mat = kron(J_d',eye(n));
         end;
         H_mat = [kron(eta,eye(n)) kron(H_orth,xi)*Kki];
         G_mat = [G_mat kron(J_k',eye(n))*H_mat];
         %
         % create adjustments of G_mat and H_mat
         %
         adj_G_mat = ones(size(G_mat,1),1);
         g_1 = G_mat*[vec(gamma);vec(xi);vec(psi)];
         g_2 = vec([gamma xi*eta']);
         for j=1:size(g_1,1);
            if abs(g_2(j,1))>eps;
               adj_G_mat(j,1) = g_1(j,1)/g_2(j,1);
            end;
         end;
         F_mat = inverse(diag(adj_G_mat))*G_mat;
         adj_H_mat = ones(size(H_mat),1);
         h_1 = H_mat*[vec(xi);vec(psi)];
         h_2 = vec(xi*eta');
         for j=1:size(h_1,1);
            if abs(h_2(j,1))>eps;
               adj_H_mat(j,1) = h_1(j,1)/h_2(j,1);
            end;
         end;
         %
         % Determine the unrestricted estimate of theta = [vec(gamma),vec(xi),vec(psi)] through
         % the adjusted G_mat matrix denoted by F_mat above. Notice that the adjustment involves
         % a diagonal matrix with 1 or 1/2 along the diagonal
         %
         theta_p1 = G_mat'*kron(Mzz,inverse(Omega))*F_mat;
         theta_p11 = G_mat'*kron(Mzz,inverse(Omega))*G_mat;
         theta_p2 = G_mat'*vec(inverse(Omega)*Mzx');
         theta_hat = inverse(theta_p1)*theta_p2;
         %
         % taking care of the restrictions
         %
         Phi = xi*eta';
         if r>0;
            alpha = Phi(:,k-r+1:k);
            Phi = Phi(:,1:k-r);
         else;
            alpha = [];
         end;
         Psi = [gamma Phi];
         C_n = cmat(Psi,alpha,beta,epsilon_n,Z1,Z2,p,d,0);
         xi_c = xi_mat(C_n,Psi,alpha,p,pe,r,d,q1,q0);
         if restrict(1,1)==1;
            %
            % The matrix of partials of the restrictions on (Psi and) alpha wrt [vec(gamma),vec(xi),vec(psi)]
            %
%            R_mat = Rmat*inverse(diag(adj_H_mat))*H_mat;
            R_mat = Rmat*H_mat;
            if d>0;
               R_mat = [zeros(dfalpha,n*d) R_mat];
            end;
            f_vec = (Rmat*vec(xi*eta'))-rvec;
         else;
            R_mat = [];
            f_vec = [];
         end;
         %
         % The matrix of partials of restrictions on C wrt [vec(gamma),vec(xi),vec(psi)]
         %
%         R_mat = [R_mat;Rmat_C*kron(xi_c',C_n)*inverse(diag(adj_H_mat))*H_mat];
         if d>0;
%            He_mat = [eye(n*d) zeros(n*d,size(H_mat,2));zeros(size(H_mat,1),n*d) H_mat];
            He_mat = [zeros(n*d) zeros(n*d,size(H_mat,2));zeros(size(H_mat,1),n*d) H_mat];
         else;
            He_mat = H_mat;
         end;
         R_mat = [R_mat;Rmat_C*kron(xi_c',C_n)*He_mat];
         f_vec = [f_vec;(Rmat_C*vec(C_n))-rvec_C];
         %
         % make sure that there are no linearly dependent restrictions on alpha and C
         %
         if rank(R_mat)<size(R_mat);
            [red_rest,use_rest] = getdelvars(R_mat',[]);
            R_mat = R_mat(use_rest,:);
            f_vec = f_vec(use_rest,1);
         else;
            use_rest = (1:1:size(R_mat,1));
         end;
         part_1 = inverse(theta_p11)*R_mat';
         part_2 = R_mat*part_1;
         it_j = 1;
         theta_bar = theta_hat;
         conv_crit = max(abs(f_vec));
         while (it_j<=maxit)&(ccrit<conv_crit);
            theta_bar = theta_bar-(part_1*inverse(part_2)*f_vec);
            %
            % determine the xi, eta, and psi parameters
            %
            xi_n = vecinv(theta_bar((n*d)+1:n*(d+i),1),n);
            psi_n = vecinv(theta_bar((n*(d+i))+1:size(theta_bar,1),1),k-i);
            eta_n = H+(H_orth*psi_n);
            %
            % Transform parameters in Psi and alpha
            %
            if d>0;
               gamma_n = vecinv(theta_bar(1:n*d,1),n);
            end;
            Phi_n = xi_n*eta_n';
            if r>0;
               alpha_n = Phi_n(:,k-r+1:k);
               Phi_n = Phi_n(:,1:k-r);
            else;
               alpha_n = [];
            end;
            Psi_n = [gamma_n Phi_n];
            %
            % Compoute the C matrix
            %
            C_n = cmat(Psi_n,alpha_n,beta,epsilon_n,Z1,Z2,p,d,0);
            %
            % evaluate the restrictions on alpha (if any) and C
            %
            if restrict(1,1)==1;
               f_vec = (Rmat*vec(xi_n*eta_n'))-rvec;
            else;
               f_vec = [];
            end;
            f_vec = [f_vec;(Rmat_C*vec(C_n))-rvec_C];
            f_vec = f_vec(use_rest,1);
            conv_crit = max(abs(f_vec));
            %
            % update matrices in theta_bar expression
            %
            if isempty(Z2_new)==0;
               epsilon_n = Z0-(xi_n*eta_n'*Z1_new)-(gamma_n*Z2_new);
            else;
               epsilon_n = Z0-(xi_n*eta_n'*Z1_new);
            end;
            Omega_n = (1/T)*epsilon_n*epsilon_n';
            %
            G_mat = [];
            if d>0;
               G_mat = kron(J_d',eye(n));
            end;
            H_mat = [kron(eta,eye(n)) kron(H_orth,xi)*Kki];
            G_mat = [G_mat kron(J_k',eye(n))*H_mat];
            F_mat = inverse(diag(adj_G_mat))*G_mat;
            %
            xi_c = xi_mat(C_n,Psi_n,alpha_n,p,pe,r,d,q1,q0);
            if restrict(1,1)==1;
               %
               % The matrix of partials of the restrictions on (Psi and) alpha wrt [vec(gamma),vec(xi),vec(psi)]
               %
%               R_mat = Rmat*inverse(diag(adj_H_mat))*H_mat;
               R_mat = Rmat*H_mat;
               if d>0;
                  R_mat = [zeros(dfalpha,n*d) R_mat];
               end;
            else;
               R_mat = [];
            end;
            %
            % The matrix of partials of restrictions on C wrt [vec(gamma),vec(xi),vec(psi)]
            %
%            R_mat = [R_mat;Rmat_C*kron(xi_c',C_n)*inverse(diag(adj_H_mat))*H_mat];
            if d>0;
%               He_mat = [eye(n*d) zeros(n*d,size(H_mat,2));zeros(size(H_mat,1),n*d) H_mat];
               He_mat = [zeros(n*d) zeros(n*d,size(H_mat,2));zeros(size(H_mat,1),n*d) H_mat];
            else;
               He_mat = H_mat;
            end;
            R_mat = [R_mat;Rmat_C*kron(xi_c',C_n)*He_mat];
            R_mat = R_mat(use_rest,:);
            %
            theta_p1 = G_mat'*kron(Mzz,inverse(Omega_n))*F_mat;
            theta_p11 = G_mat'*kron(Mzz,inverse(Omega_n))*G_mat;
            part_1 = inverse(theta_p11)*R_mat';
            part_2 = R_mat*part_1;
            it_j = it_j+1;
         end;
         if isempty(Z2_new)==0;
            epsilon_n = Z0-(xi_n*eta_n'*Z1_new)-(gamma_n*Z2_new);
         else;
            epsilon_n = Z0-(xi_n*eta_n'*Z1_new);
         end;
         Omega_n = (1/T)*epsilon_n*epsilon_n';
      end;
      %
      % Update beta if desirec
      %
      if (restrict(1,2)==1)&(reestbeta==1)&(r>0);
         %
         % compute the gradient
         %
         grad = max(abs(T*(vec(alpha_n))'*kron(eye(r),inverse(Omega_n)*(M01-(alpha_n*beta'*M11)-(Psi_n*M21)))*H_b));
         if grad>ccrit;
            %
            % solve for beta from the first order conditions (use the same algorithm as in c_restrict
            %
            m_1 = H_b'*kron(alpha_n'*inverse(Omega_n)*alpha_n,M11)*H_b;
            if isempty(M12)==0;
               c_1 = vec((M10-(M12*Psi_n'))*inverse(Omega_n)*alpha_n);
            else;
               c_1 = vec(M10*inverse(Omega_n)*alpha_n);
            end;
            c_2 = kron(alpha_n'*inverse(Omega_n)*alpha_n,M11)*h_b;
            phi = inverse(m_1)*H_b'*(c_1-c_2);
            v_beta = h_b+(H_b*phi);
            beta = vecinv(v_beta,size(beta,1));
         end;
         %
         % compute the gradient wrt the free parameters of [vec(gamma),vec(xi),vec(psi)]
         %
         Kki = commutation(k-i,i);
         %
         % The matrix of partial derivatives of vec([gamma xi*eta']) wrt [vec(gamma),vec(xi),vec(psi)]
         %
         G_mat = [];
         if d>0;
            G_mat = kron(J_d',eye(n));
         end;
         H_mat = [kron(eta_n,eye(n)) kron(H_orth,xi_n)*Kki];
         G_mat = [G_mat kron(J_k',eye(n))*H_mat];
         %
         % create adjustments of G_mat and H_mat
         %
         adj_G_mat = ones(size(G_mat,1),1);
         g_1 = G_mat*[vec(gamma_n);vec(xi_n);vec(psi_n)];
         g_2 = vec([gamma_n xi_n*eta_n']);
         for j=1:size(g_1,1);
            if abs(g_2(j,1))>eps;
               adj_G_mat(j,1) = g_1(j,1)/g_2(j,1);
            end;
         end;
         F_mat = inverse(diag(adj_G_mat))*G_mat;
         adj_H_mat = ones(size(H_mat),1);
         h_1 = H_mat*[vec(xi_n);vec(psi_n)];
         h_2 = vec(xi_n*eta_n');
         for j=1:size(h_1,1);
            if abs(h_2(j,1))>eps;
               adj_H_mat(j,1) = h_1(j,1)/h_2(j,1);
            end;
         end;
         %
         if d>0;
            Z2_new = Z2(1:d,:);
            Z1_new = Z2(d+1:size(Z2,1),:);
         else;
            Z2_new = [];
            Z1_new = Z2;
         end;
         %
         Z1_new = [Z1_new;beta'*Z1];
         %
         Mzz = (1/T)*[Z2_new;Z1_new]*[Z2_new;Z1_new]';
         Mzx = (1/T)*[Z2_new;Z1_new]*Z0';
         %
         grad = T*(vec(Mzx'-([Psi_n alpha_n]*Mzz))')*kron(eye(k+d),inverse(Omega_n))*G_mat;
         %
         % adjust the gradient if there are restrictions on Psi and alpha
         %
         if (restrict(1,1)==1)&((size(restrict,2)==2)|(restrict(1,3)==0));
%            R_mat = Rmat*inverse(diag(adj_H_mat))*H_mat;
            R_mat = Rmat*H_mat;
            if d>0;
               R_mat = [zeros(dfalpha,n*d) R_mat];
            end;
            R_orth = null(R_mat);
            grad = grad*R_orth;
         elseif (size(restrict,2)>2)&(restrict(1,3)==1);
            C_n = cmat(Psi_n,alpha_n,beta,epsilon_n,Z1,Z2,p,d,0);
            xi_c = xi_mat(C_n,Psi_n,alpha_n,p,pe,r,d,q1,q0);
            if restrict(1,1)==1;
               %
               % The matrix of partials of the restrictions on (Psi and) alpha wrt [vec(gamma),vec(xi),vec(psi)]
               %
%               R_mat = Rmat*inverse(diag(adj_H_mat))*H_mat;
               R_mat = Rmat*H_mat;
               if d>0;
                  R_mat = [zeros(dfalpha,n*d) R_mat];
               end;
            else;
               R_mat = [];
            end;
            %
            % The matrix of partials of restrictions on C wrt [vec(gamma),vec(xi),vec(psi)]
            %
%            R_mat = [R_mat;Rmat_C*kron(xi_c',C_n)*inverse(diag(adj_H_mat))*H_mat];
            if d>0;
%               He_mat = [eye(n*d) zeros(n*d,size(H_mat,2));zeros(size(H_mat,1),n*d) H_mat];
               He_mat = [zeros(n*d) zeros(n*d,size(H_mat,2));zeros(size(H_mat,1),n*d) H_mat];
            else;
               He_mat = H_mat;
            end;
            R_mat = [R_mat;Rmat_C*kron(xi_c',C_n)*He_mat];
            R_mat = R_mat(use_rest,:);
            R_orth = null(R_mat);
            grad = grad*R_orth;
         end;
         %
         oldgrad = max(abs(grad));
         grad = max(abs(grad));
         grad = abs(grad-old_grad);
         old_grad = oldgrad;
      else;
         %
         % check if we should re-estimate beta
         %
         if (reestbeta==1)&(r>0);
            mygrad = alpha_n'*inverse(Omega_n)*(M01-(Psi_n*M21)-(alpha_n*beta'*M11));
            if (max(max(abs(mygrad)))>ccrit);
               %
               % re-compute beta from first order condition
               %
               beta = inverse(M11)*(M10-(M12*Psi_n'))*inverse(Omega_n)*alpha_n*inverse(alpha_n'*inverse(Omega_n)*alpha_n);
               %
               % re-compute data-series for xi, eta estimation
               %
               if d>0;
                  Z2_new = Z2(1:d,:);
                  Z1_new = Z2(d+1:size(Z2,1),:);
               else;
                  Z2_new = [];
                  Z1_new = Z2;
               end;
               Z1_new = [Z1_new;beta'*Z1];
               %
               Mzz = (1/T)*[Z2_new;Z1_new]*[Z2_new;Z1_new]';
               Mzx = (1/T)*[Z2_new;Z1_new]*Z0';
               %
               % compute the gradient wrt the free parameters of [vec(gamma),vec(xi),vec(psi)]
               %
               Kki = commutation(k-i,i);
               %
               % The matrix of partial derivatives of vec([gamma xi*eta']) wrt [vec(gamma),vec(xi),vec(psi)]
               %
               G_mat = [];
               if d>0;
                  G_mat = kron(J_d',eye(n));
               end;
               H_mat = [kron(eta_n,eye(n)) kron(H_orth,xi_n)*Kki];
               G_mat = [G_mat kron(J_k',eye(n))*H_mat];
               %
               % compute the gradient
               %
               grad = T*(vec(Mzx'-([Psi_n alpha_n]*Mzz))')*kron(eye(k+d),inverse(Omega_n))*G_mat;
               if (size(restrict,2)>2)&(restrict(1,3)==1);
                  C_n = cmat(Psi_n,alpha_n,beta,epsilon_n,Z1,Z2,p,d,0);
                  xi_c = xi_mat(C_n,Psi_n,alpha_n,p,pe,r,d,q1,q0);
                  %
                  % create adjustments of G_mat and H_mat
                  %
                  adj_G_mat = ones(size(G_mat,1),1);
                  g_1 = G_mat*[vec(gamma_n);vec(xi_n);vec(psi_n)];
                  g_2 = vec([gamma_n xi_n*eta_n']);
                  for j=1:size(g_1,1);
                     if abs(g_2(j,1))>eps;
                        adj_G_mat(j,1) = g_1(j,1)/g_2(j,1);
                     end;
                  end;
                  F_mat = inverse(diag(adj_G_mat))*G_mat;
                  adj_H_mat = ones(size(H_mat),1);
                  h_1 = H_mat*[vec(xi_n);vec(psi_n)];
                  h_2 = vec(xi_n*eta_n');
                  for j=1:size(h_1,1);
                     if abs(h_2(j,1))>eps;
                        adj_H_mat(j,1) = h_1(j,1)/h_2(j,1);
                     end;
                  end;
                  %
                  % The matrix of partials of restrictions on C wrt [vec(gamma),vec(xi),vec(psi)]
                  %
%                  R_mat = Rmat_C*kron(xi_c',C_n)*inverse(diag(adj_H_mat))*H_mat;
                  if d>0;
%                     He_mat = [eye(n*d) zeros(n*d,size(H_mat,2));zeros(size(H_mat,1),n*d) H_mat];
                     He_mat = [zeros(n*d) zeros(n*d,size(H_mat,2));zeros(size(H_mat,1),n*d) H_mat];
                  else;
                     He_mat = H_mat;
                  end;
                  R_mat = Rmat_C*kron(xi_c',C_n)*He_mat;
                  R_mat = R_mat(use_rest,:);
                  R_orth = null(R_mat);
                  grad = grad*R_orth;
               end;
               oldgrad = max(abs(grad));
               grad = max(abs(grad));
               grad = abs(grad-old_grad);
               old_grad = oldgrad;
            else;
               grad = max(max(abs(mygrad)));
            end;
         else;
            grad = 0;
         end;
      end;
      %
      % update the iteration counter
      %
      it = it+1;
   end;
   %
   %
   %
   if show_waitbar==1;
      set(mywait,'UserData','done');
      close(mywait);
      drawnow;
   end;
   %
   if strcmp(abort,'cancel')==0;
      %
      % Compute the asymptotic covariance matrix for vec[Psi alpha]
      %
      if min_q==max_q;
         %
         % compute the gradient wrt the free parameters of [vec(gamma),vec(xi),vec(psi)]
         %
         Kki = commutation(k-i,i);
         %
         % The matrix of partial derivatives of vec([gamma xi*eta']) wrt [vec(gamma),vec(xi),vec(psi)]
         %
         G_mat = [];
         if d>0;
            G_mat = kron(J_d',eye(n));
         end;
         H_mat = [kron(eta_n,eye(n)) kron(H_orth,xi_n)*Kki];
         G_mat = [G_mat kron(J_k',eye(n))*H_mat];
         %
         % create adjustments of G_mat and H_mat
         %
         adj_G_mat = ones(size(G_mat,1),1);
         g_1 = G_mat*[vec(gamma_n);vec(xi_n);vec(psi_n)];
         g_2 = vec([gamma_n xi_n*eta_n']);
         for j=1:size(g_1,1);
            if abs(g_2(j,1))>eps;
               adj_G_mat(j,1) = g_1(j,1)/g_2(j,1);
            end;
         end;
         F_mat = inverse(diag(adj_G_mat))*G_mat;
         adj_H_mat = ones(size(H_mat),1);
         h_1 = H_mat*[vec(xi_n);vec(psi_n)];
         h_2 = vec(xi_n*eta_n');
         for j=1:size(h_1,1);
            if abs(h_2(j,1))>eps;
               adj_H_mat(j,1) = h_1(j,1)/h_2(j,1);
            end;
         end;
         %
         if d>0;
            Z2_new = Z2(1:d,:);
            Z1_new = Z2(d+1:size(Z2,1),:);
         else;
            Z2_new = [];
            Z1_new = Z2;
         end;
         %
         Z1_new = [Z1_new;beta'*Z1];
         %
         Mzz = (1/T)*[Z2_new;Z1_new]*[Z2_new;Z1_new]';
         Mzx = (1/T)*[Z2_new;Z1_new]*Z0';
         %
         % begin setting up
         %
         Sigma_small = (1/T)*inverse(G_mat'*kron(Mzz,inverse(Omega_n))*G_mat);
         if (restrict(1,1)==1)&((size(restrict,2)==2)|(restrict(1,3)==0));
            %
            % make corrections when there are just restrictions on alpha
            %
%            R_mat = Rmat*inverse(diag(adj_H_mat))*H_mat;
            R_mat = Rmat*H_mat;
            if d>0;
               R_mat = [zeros(dfalpha,n*d) R_mat];
            end;
            adj_Sigma_small = Sigma_small*R_mat'*inverse(R_mat*Sigma_small*R_mat')*R_mat*Sigma_small;
            Sigma_small = Sigma_small-adj_Sigma_small;
         elseif (size(restrict,2)>2)&(restrict(1,3)==1);
            C_n = cmat(Psi_n,alpha_n,beta,epsilon_n,Z1,Z2,p,d,0);
            xi_c = xi_mat(C_n,Psi_n,alpha_n,p,pe,r,d,q1,q0);
            if restrict(1,1)==1;
               %
               % The matrix of partials of the restrictions on (Psi and) alpha wrt [vec(gamma),vec(xi),vec(psi)]
               %
%               R_mat = Rmat*inverse(diag(adj_H_mat))*H_mat;
               R_mat = Rmat*H_mat;
               if d>0;
                  R_mat = [zeros(dfalpha,n*d) R_mat];
               end;
            else;
               R_mat = [];
            end;
            %
            % The matrix of partials of restrictions on C wrt [vec(gamma),vec(xi),vec(psi)]
            %
%            R_mat = [R_mat;Rmat_C*kron(xi_c',C_n)*inverse(diag(adj_H_mat))*H_mat];
            if d>0;
%               He_mat = [eye(n*d) zeros(n*d,size(H_mat,2));zeros(size(H_mat,1),n*d) H_mat];
               He_mat = [zeros(n*d) zeros(n*d,size(H_mat,2));zeros(size(H_mat,1),n*d) H_mat];
            else;
               He_mat = H_mat;
            end;
            R_mat = [R_mat;Rmat_C*kron(xi_c',C_n)*He_mat];
            R_mat = R_mat(use_rest,:);
            %
            adj_Sigma_small = Sigma_small*R_mat'*inverse(R_mat*Sigma_small*R_mat')*R_mat*Sigma_small;
            Sigma_small = Sigma_small-adj_Sigma_small;
         end;
         Sigma_theta = G_mat*Sigma_small*G_mat';
         %
         % determine the asymptotic covariance matrix for xi_orth, the co-feature vectors
         %
         rows = [];
         xi_h = null(xi_n','r');
         for j=1:(n-max_q);
            [t,ind] = max(abs(xi_h(:,j)));
            rows = [rows;ind];
            xi_h(ind,:) = zeros(1,n-max_q);
         end;
         %
         K_xi_orth = eye(n);
         K_xi_orth = K_xi_orth(:,sort(rows));
         K_xi = null(K_xi_orth','r');
         xi_orth = K_xi_orth-(K_xi*inverse(xi_n'*K_xi)*xi_n'*K_xi_orth);
         %
         % determine the partials of xi_orth wrt [vec(gamma),vec(xi),vec(psi)]
         %
         p_1 = K_xi_orth'*(eye(n)-(xi_n*inverse(K_xi'*xi_n)*K_xi'));
         p_2 = K_xi*inverse(xi_n'*K_xi);
         D_xi_orth = -kron(p_1,p_2)*commutation(n,max_q);
         %
         D_theta = [];
         if d>0;
            D_theta = zeros(size(D_xi_orth,1),n*d);
         end;
         D_theta = [D_theta D_xi_orth zeros(size(D_xi_orth,1),size(vec(psi_n),1))]*inverse(G_mat'*G_mat)*G_mat';
         Sigma_xi_orth = D_theta*Sigma_theta*D_theta';
      end;
      %
      % information about the iterations
      %
      infomat = [infomat;it-1 grad];
      %
      % compute the test for n-i common cycles vs fewer than n-i
      %
      test(num,1) = 2*(loglike(epsilon)-loglike(epsilon_n));
      if sscorrect==0;
         deg_free(num,1) = (n-i)*(k-i);
         pval(num,1) = 1-chis_cdf(test(num,1),deg_free(num,1));
      else;
         deg_free(num,1) = (n-i)*(k-i);
         deg_free(num,2) = T-df+floor(deg_free(num,1)/n)-1;
         test(num,1) = (deg_free(num,2)/(T*deg_free(num,1)))*test(num,1);
         pval(num,1) = 1-fdis_cdf(test(num,1),deg_free(num,1),deg_free(num,2));
      end;
      num = num+1;
   else;
      alpha_n = [];
      Psi_n = [];
      break;
   end;
end;

if (strcmp(abort,'cancel')==0)&(min_q==max_q)&(restrict(1,2)==1)&(reestbeta==1);
   %
   % compute new conditional standard errors for beta
   %
   if isempty(Z2)==0;
      newR0 = Z0-((Psi+(alpha_n*beta'*(M12/M22)))*Z2);
      newR1 = Z1-((M12/M22)*Z2);
   else;
      newR0 = Z0;
      newR1 = Z1;
   end;
   %
   % Constructing unrestricted beta estimate
   %
   S_00_new = (1/T)*newR0*newR0';
   S_10_new = (1/T)*newR1*newR0';
   S_11_new = (1/T)*newR1*newR1';
   S_01_new = (1/T)*newR0*newR1';
   alpharort = null(alpha_n');
   S_11_alpha = S_11_new-(S_10_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'*S_01_new)));
   %
   sivec = zeros(r,1);
   %
   % bH is needed to check the gradient of the likelihood function
   % wrt the beta parameters
   %
   for i=1:r;
      file = strcat(modelsdir,'H_',num2str(i),'-',num2str(n),'x',num2str(r),'x',num2str(dcoint+q1),'-',modelfile,'.rest');
      if fileexist(file)==1;
         h = loaddata(file);
         if i==1;
            H = h;
            bH = h;
         else;
            H = [H h];
            bH = [[bH zeros(size(bH,1),size(h,2))];[zeros(size(h,1),size(bH,2)) h]];
         end;
         sivec(i,1) = size(h,2);
      end;
   end;
   %
   % Get the normalizing variables
   %
   file = strcat(modelsdir,'normalize-',num2str(n),'x',num2str(r),'x',num2str(dcoint+q1),'-',modelfile,'.norm');
   if fileexist(file)==1;
      normvec = loaddata(file);
   else;
      normvec = [];
   end;
   %
   % recompute ths conditional standard errors for beta
   %
   for i=1:r;
      %
      % Selecting the restrictions from H
      %
      si = sivec(i,1);
      if si>=2;
         sj = size(H,2);
         if i==1;
            Hsel = [eye(si);zeros(sj-si,si)];
         elseif i==r;
            Hsel = [zeros(sj-si,si);eye(si)];
         else;
            sk = sivec(1,1);
            for j=2:i-1;
               sk = sk+sivec(j,1);
            end;
            Hsel = [zeros(sk,si);eye(si);zeros(sj-si-sk,si)];
         end;
         Hi = H*Hsel;
         %
         % Determining the normalizations
         %
         normelem = normvec(i,1);
         myexit = 0;
         for j=1:si;
            %
            % Fix Hi to make sure that the normalizing row
            % contains one unit element and zeros
            %
            normmatrix = Hi(normelem,:);
            for k=1:(n+q1+dcoint);
               if k~=normelem;
                  checkHi = [normmatrix;Hi(k,:)];
                  if rank(checkHi*checkHi')==size(checkHi,1);
                     normmatrix = checkHi;
                     if size(normmatrix,1)==size(Hi,2);
                        break;
                     end;
                  end;
               end;
            end;
            Hi = Hi/normmatrix;
            check = Hi(normelem,j);
            if (abs(check)>0)&(check~=1);
               Hi(:,j) = (1/Hi(normelem,j))*Hi(:,j);
               check = Hi(normelem,j);
            end;          
            if check==1;
               %
               % Split Hi into hi and H_i
               %
               if j==1;
                  hi = Hi(:,1);
                  H_i = Hi(:,2:si);
               elseif j==si;
                  hi = Hi(:,si);
                  H_i = Hi(:,1:si-1);
               else;
                  hi = Hi(:,j);
                  H_i = [Hi(:,1:j-1) Hi(:,j+1:si)];
               end;
               %
               % Check H_i to make sure it is of full rank
               %
               check2 = H_i'*H_i;
               if abs(det(check2))>=.000000001;
                  myexit=1;
               end;
            end;
            if myexit==1;
               break;
            end;
         end;
         %
         % Computing the asymptotic covariance for the
         % si-1 free parameters of column i in betar
         %
         information = alpha_n(:,i)'*inverse(Omega_n)*alpha_n(:,i)*H_i'*S_11_alpha*H_i;
         covbeta = (1/T)*(H_i/information)*H_i';
         %
         % Computing the asyptotic standard errors and
         % including them in stdbeta
         %
         stdbeta(:,i) = sqrt(abs(diag(covbeta)));
      end;
   end;
end;

%
% end of estimate_common_cycles.m
%
