%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for displaying graphs of data       %
%                                                                %
%          Function: graphdlg                                    %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       nametype: string defining variable       %
%                           type (Endogenous, etc.)              %
%                       curr_model: structure with model info    %
%                                                                %
%          Output:                                               %
%          	            h: handle                                %
%                                                                %
%          This version:        May 3, 2006                      %
%          Written by:          Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = graphdlg(selector,nametype,curr_model)

if strcmp(selector,'init')==1;
   if strcmp(nametype,'Endogenous');
      datafile = curr_model.endofile;
      datasheet = curr_model.endosheet;
      origname = curr_model.evars;
      mytype = 'endo';
   elseif strcmp(nametype,'Exogenous I(1)')==1;
      datafile = curr_model.exo1file;
      datasheet = curr_model.exo1sheet;
      origname = curr_model.xvars1;
      mytype = 'exo1';
   elseif strcmp(nametype,'Exogenous I(0)')==1;
      datafile = curr_model.exo0file;
      datasheet = curr_model.exo0sheet;
      origname = curr_model.xvars0;
      mytype = 'exo0';
   elseif strcmp(nametype,'Deterministic')==1;
      datafile = curr_model.deterfile;
      datasheet = curr_model.detersheet;
      origname = curr_model.deter;
      mytype = 'deter';
   end;
   %
   f = get(0,'CurrentFigure');
   out = get(f,'UserData');
   if strcmp(get(out.scale_1,'Check'),'on')==1;
      curr_model.scale_endo = 0.0001;
   elseif strcmp(get(out.scale_2,'Check'),'on')==1;
      curr_model.scale_endo = 0.001;
   elseif strcmp(get(out.scale_3,'Check'),'on')==1;
      curr_model.scale_endo = 0.01;
   elseif strcmp(get(out.scale_4,'Check'),'on')==1;
      curr_model.scale_endo = 0.1;
   elseif strcmp(get(out.scale_5,'Check'),'on')==1;
      curr_model.scale_endo = 1;
   elseif strcmp(get(out.scale_6,'Check'),'on')==1;
      curr_model.scale_endo = 10;
   elseif strcmp(get(out.scale_7,'Check'),'on')==1;
      curr_model.scale_endo = 100;
   elseif strcmp(get(out.scale_8,'Check'),'on')==1;
      curr_model.scale_endo = 1000;
   elseif strcmp(get(out.scale_9,'Check'),'on')==1;
      curr_model.scale_endo = 10000;
   else;
      curr_model.scale_endo = 1;
   end;
   %
   curr_model.datafile = datafile;
   curr_model.datafile = loaddata(curr_model.datafile,datasheet);
   curr_model.n = size(curr_model.datafile,2);
   if size(curr_model.datafile,2)>size(curr_model.datafile,1);
      curr_model.n = size(curr_model.datafile,1);
      curr_model.datafile = curr_model.datafile';
   end;
   curr_model = getsample(curr_model,datafile,datasheet);
   %
   % setting up the actual sample period
   %
   curr_model.dataindex = make_dataindex(curr_model.freq,curr_model.samplebeg,curr_model.sampleend);
   %
   % check if origname is empty and if so generate
   % generic names
   %
   if strcmp(origname,'')==1;
      if strcmp(nametype,'Endogenous');
         i = 1;
         while i<=curr_model.n;
            if i==1;
               origname = 'y_1';
            else;
               origname = [origname ' y_' num2str(i)];
            end;
            i = i+1;
         end;
      elseif strcmp(nametype,'Exogenous I(1)')==1;
         i = 1;
         while i<=curr_model.n;
            if i==1;
               origname = 'x1_1';
            else;
               origname = [origname ' x1_' num2str(i)];
            end;
            i = i+1;
         end;
      elseif strcmp(nametype,'Exogenous I(0)')==1;
         i = 1;
         while i<=curr_model.n;
            if i==1;
               origname = 'x0_1';
            else;
               origname = [origname ' x0_' num2str(i)];
            end;
            i = i+1;
         end;
      elseif strcmp(nametype,'Deterministic')==1;
         for i=1:curr_model.n;
            if i==1;
               origname = 'd_1';
            else;
               origname = [origname ' d_' num2str(i)];
            end;
         end;
      end;
   end;
   curr_model.namevector = vecstr(origname);
   thisexit = 0;
   curr_ini = find_inisettings;
   if isnumeric(curr_ini.external)==0;
      curr_ini.external = 0;
   end;
   curr_model.ini = curr_ini;
else
   thisexit = 1;
   graphgui = get(0,'CurrentFigure');
   graph = get(graphgui,'UserData');
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';
      graphgui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','graphdlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-341)/2 (realscrsz(4)-140)/2 341 140], ...
         'Tag','FigParent', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Graph ' nametype ' Variables'], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','graphdlg wayout', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(graphgui,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 339 130],'Graphics',45,[0.5 1]);

      imageaxes = axes('Units','pixels', ...
         'Box','on', ...
         'TickLength',[0 0], ...
         'CameraUpVector',[0 1 0], ...
         'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 80 39 39], ...
         'XTickLabelMode','manual', ...
         'YTickLabelMode','manual', ...
         'Tag','AxisLogo', ...
         'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ZColor',get(0,'defaultuicontrolbackgroundcolor'));

      mycons = getstoredimages;
      if curr_ini.use_toolbar_bgcolor==1;
         bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
         bgcolor(1) = max(0,bgcolor(1));
         bgcolor(2) = max(0,bgcolor(2));
         bgcolor(3) = max(0,bgcolor(3));
      else;
         bgcolor = get(0,'defaultuicontrolbackgroundcolor');
      end;
      %
      image('CData',mirrorimage(maketrans(getfield(mycons,[mytype 'big_image']),[160 160 164],bgcolor)),'Clipping','off','Parent',imageaxes);


      graph.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50 200 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String',['Actual Sample:    ' curr_model.beginyear ':' curr_model.beginperiod ' - ' curr_model.endyear ':' curr_model.endperiod], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');

      %
      % Select variable
      %

      graph.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 16 80 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'UserData',curr_model, ...
         'String','Select Variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableText');


      graph.popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[105 20 80 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',curr_model.namevector, ...
         'HorizontalAlignment','center', ...
         'Tag','GraphListbox',...
         'Value',1);

      %
      % Create graph button
      %

      graph.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[236 55 90 20], ...
         'String','Display', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','graphdlg showgraph', ...
         'Enable','on', ...
         'UserData','working', ...
         'Tag','Done');

      %
      %  Done button
      %

      graph.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[236 20 90 20], ...
         'String','Done', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','graphdlg done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(graphgui,'UserData',graph,...
         'HandleVisibility','callback');
   case 'showgraph'
      curr_model = get(graph.variables,'UserData');
      curr_ini = curr_model.ini;
      nametype = get(graphgui,'Name');
      nametype = strrep(nametype,' Variables','');
      nametype = strrep(nametype,'Graph ','');
      series = get(graph.popup,'Value');
      newvalue = series+1;
      if newvalue>size(curr_model.namevector,1);
         set(graph.popup,'Value',1);
      else;
         set(graph.popup,'Value',newvalue);
      end;
      variable = curr_model.namevector(series,:);
      variable = strrep(variable,' ','');
      set(graphgui,'UserData',graph);
      header1 = ['The levels of ' variable];
      header2 = ['First differences of ' variable];
      if curr_ini.colors==1;
         colorstr = 'b';
      else;
         colorstr = 'k';
      end;
      endo_scale = 1;
      if strcmp(nametype,'Deterministic')==0;
         y = curr_model.scale_endo*curr_model.datafile(:,series);
      else;
         y = curr_model.datafile(:,series);
      end;
      T = size(y,1);
      dy = y(2:T,1)-y(1:T-1,:);
      thisplot = figure('Units','pixels', ...
         'Position',[(realscrsz(3)-512)/2 (realscrsz(4)-384)/2 512 384], ...
         'Menubar','none', ...
         'Toolbar','none', ...
         'NumberTitle','off', ...
         'Visible','off', ...
         'Name',['Time Series Plot of ' variable]);
      %
      % default figure properties
      %
      setfigureproperties(thisplot);
      %
      if strcmp(nametype,'Deterministic')==0;
         lh = plot(curr_model.dataindex,y,'-','LineWidth',1,'Color',colorstr);
         set(lh,'Tag',variable);
         if (min(y)<0)&(max(y)>0);
            hold('on');
            plot(curr_model.dataindex,zeros(T,1),'-','LineWidth',1,'Color','k');
         end;
      else;
         [y1,y2] = stairs(curr_model.dataindex,y);
         lh = plot(y1,y2,'-','LineWidth',1,'Color',colorstr);
         set(lh,'Tag',[variable '-stairs']);
         set(gca,'YLim',[min(y)-0.1 max(y)+0.1]);
      end;
      set(gca,'XLim',[curr_model.dataindex(1,1) curr_model.dataindex(size(curr_model.dataindex,1),1)]);
      setaxesfonts('Title',header1);
      toolbar_fcn(gca);
      if strcmp(nametype,'Deterministic')==0;
         nextplot = figure('Units','pixels', ...
            'Position',[(realscrsz(3)-512)/2+20 (realscrsz(4)-384)/2-20 512 384], ...
            'Menubar','none', ...
            'Toolbar','none', ...
            'NumberTitle','off', ...
            'Visible','off', ...
            'Name',['Time Series Plot of ' variable '(t)-' variable '(t-1)']);
         %
         % default figure properties
         %
         setfigureproperties(nextplot);
         %
         lh = plot(curr_model.dataindex(2:T,1),dy,'-','LineWidth',1,'Color',colorstr);
         set(lh,'Tag',['d' variable]);
         hold('on');
         plot(curr_model.dataindex,zeros(T,1),'-','LineWidth',1,'Color','k');
         set(gca,'XLim',[curr_model.dataindex(2,1) curr_model.dataindex(size(curr_model.dataindex,1),1)]);
         setaxesfonts('Title',header2);
         toolbar_fcn(gca);
      end;
      hold('off');
      if curr_ini.external==0;
         set(thisplot,'Visible','on');
         if strcmp(nametype,'Deterministic')==0;
            set(nextplot,'Visible','on');
         end;
      else;
         file = [curr_model.outputdir variable];
         status1 = autosavefigure(thisplot,file);
         if strcmp(nametype,'Deterministic')==0;
            filenext = [curr_model.outputdir 'd' variable];
            status2 = autosavefigure(nextplot,filenext);
         end;
         if status1==0;
            autoshowfigure(file);
         end;
         if strcmp(nametype,'Deterministic')==0;
            if status2==0;
               autoshowfigure(filenext);
            end;
         end;
      end;
   case 'done'
      set(graph.done,'UserData','done');
      set(graphgui,'UserData',graph);
   case 'wayout'
      set(graph.done,'UserData','done');
%      if strcmp(get(graph.show,'UserData'),'done')==1;
%         delete(graphgui);
%      end;
end;

if thisexit==0;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(graphgui,'Name') ' ' fix_filepath([pwd '\icons\' mytype 'data.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(graph.icon,'UserData',pid);
      end;
   end;
   %
   set(graphgui,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(graphgui,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(graph.done);
   end;
   waitfor(graph.done,'UserData','done');
end;

if nargout==1;
   h = graphgui;
end;

%
% end of graphdlg.m
%
