%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          Computes value of the maximized log likelihood        %
%          function for a normal distribution when Omega         %
%          is equal to (1/T)*epsilon*epsilon'                    %
%                                                                %
%          Function: loglike                                     %
%                                                                %
%          Input:                                                %
%                       epsilon: nxT matrix with residuals       %
%                                                                %
%          Output:                                               %
%          	            lnl: value of log likelihood function    %
%                       logdetOmega: the value of the natural    %
%                            logaritm of the determinant of      %
%                            Omega                               %
%                                                                %
%          This version:        May 31, 2004                     %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2004          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [lnl,logdetOmega] = loglike(epsilon)

[n T] = size(epsilon);
Omega = (1/T)*epsilon*epsilon';

lnl = -(T/2)*(n*log(2*pi)+log(det(Omega)))-((n*T)/2);

if nargout==2;
   logdetOmega = log(det(Omega));
end;

%
% end of loglike.m
%
