%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
%  Function: make_uimenu                                                       %
%                                                                              %
%  Purpose: Create a uimenu                                                    %
%                                                                              %
%  Input:                                                                      %
%        aStruct:     a structure (may be empty)                               %
%        par:         parent window (handle)                                   %
%        data:        structure with the following fields:                     %
%           fieldstr:       cell array with field names for aStruct            %
%           parentstr:      cell array with parent fields (must be part        %
%                           of aStruct or made part of aStruct by the          %
%                           function) - must not include parent window handle  %
%           labelstr:       cell array with labels                             %
%           callbackstr:    cell array with callback strings                   %
%           acceleratorstr: cell array with accelerators keys                  %
%           enablestr:      cell array with on/off entries                     %
%           separatorstr:   cell array with on/off entries                     %
%           tagstr:         cell array with tag strings                        %
%        ud:          vector structure with "userdata" field                   %
%                                                                              %
%           NOTE: All these fields must have the same number of entries        %
%                 which is equal to the number of buttons                      %
%                                                                              %
%  Output:                                                                     %
%        aStruct:    a structure containing all handles of the created         %
%                    objects+any handles passed to the function                %
%                                                                              %
%  This version:     May 30, 2003                                              %
%  Written by:       Anders Warne                                              %
%                    Copyright (c) 2001-2003                                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function aStruct = make_uimenu(aStruct,par,data,ud)

if (ishandle(par)==0)|(strcmp(get(par,'Type'),'figure')==0);
   par = get(0,'CurrentWindow');
end;

for i=1:size(data.fieldstr,2);
   h1 = uimenu('Label',data.labelstr{i}, ...
         'Enable',data.enablestr{i}, ...
         'UserData',ud(i).userdata, ...
         'Separator',data.separatorstr{i}, ...
         'Tag',data.tagstr{i});
   if i==1;
      set(h1,'Parent',par);
   else;
      set(h1,'Parent',getfield(aStruct,data.parentstr{i-1}));
   end;
   if isempty(strtrim(data.callbackstr{i}))==0;
      set(h1,'Callback',strtrim(data.callbackstr{i}));
   end;
   if isempty(strtrim(data.acceleratorstr{i}))==0;
      set(h1,'Accelerator',strtrim(data.acceleratorstr{i}));
   end;
   aStruct = setfield(aStruct,data.fieldstr{i},h1);
end;

%
% end of make_uimenu.m
%
