function [res,stat] = margin_beta(tm,res)

mywait = waitdlg(strvcat(' ','Computing the marginal likelihood for beta restrictions...','Please wait!'),'bootstrap','Marginalizing',350,124);
waitf = get(mywait,'UserData');
setlogtext('Computing marginal likelihood for beta restrictions...');

stat = 0;
if tm.amat_type==1;
   if tm.amat_lag==1;
      A = tm.rank(tm.n+1).Omega;
   else;
      A = tm.amat_val;
   end;
   q = tm.n+2;
else;
   A = tm.lambda_a*eye(tm.n);
   q = tm.q_parm;
end;
if (tm.prior_inf_shortrun==2)&(tm.prior_inf_deter==2);
   %
   % improper priors on delta and Gamma
   %
   if isempty(tm.Z2)==0;
      Z = tm.Z2;
      z = det(Z*Z');
      N = eye(tm.T)-Z'*inverse(Z*Z')*Z;
      num_z = size(tm.Z2,1);
   else;
      z = 1;
      N = eye(tm.T);
      num_z = 0;
   end;
elseif (tm.prior_inf_shortrun==1)&(tm.prior_inf_deter==2);
   %
   % improper prior on delta
   %
   if isempty(tm.Z2)==0;
      if tm.d>0;
         D = tm.Z2(1:tm.d,:);
         M_D = eye(tm.T)-(D'*inverse(D*D')*D);
      else;
         D = 1;
         M_D = eye(tm.T);
      end;
      if size(tm.Z2,1)>tm.d;
         Z2 = tm.Z2(tm.d+1:size(tm.Z2,1),:);
         G_sigma = tm.prior.shortrun.sigma;
         N = M_D-(M_D*Z2'*inverse(((Z2*M_D*Z2')+inverse(G_sigma)))*Z2*M_D);
         z = det(G_sigma)*det(D*D')*det(((Z2*M_D*Z2')+inverse(G_sigma)));
      else;
         N = M_D;
         z = det(D*D');
      end;
   else;
      N = eye(tm.T);
      z = 1;
   end;
   num_z = tm.d;
elseif (tm.prior_inf_shortrun==2)&(tm.prior_inf_deter==1);
   %
   % improper prior on Gamma
   %
   if isempty(tm.Z2)==0;
      if tm.d>0;
         D_sigma = tm.prior.deter.sigma;
         D = tm.Z2(1:tm.d,:);
         if size(tm.Z2)>tm.d;
            Z2 = tm.Z2(tm.d+1:size(tm.Z2,1),:);
            M_Z2 = eye(tm.T)-(Z2'*inverse(Z2*Z2')*Z2);
            N = M_Z2-(M_Z2*D'*inverse(((D*M_Z2*D')+inverse(D_sigma)))*D*M_Z2);
            z = det(D_sigma)*det(Z2*Z2')*det(((D*M_Z2*D')+inverse(D_sigma)));
            num_z = size(Z2,1);
         else;
            N = eye(tm.T)-D'*inverse(((D*D')+inverse(D_sigma)))*D;
            z = det(D_sigma)*det(((D*D')+inverse(D_sigma)));
            num_z = 0;
         end;
      else;
         Z2 = tm.Z2;
         M_Z2 = eye(tm.T)-(Z2'*inverse(Z2*Z2')*Z2);
         N = M_Z2;
         z = det(Z2*Z2');
         num_z = size(Z2,1);
      end;
   else;
      N = eye(tm.T);
      z = 1;
      num_z = 0;
   end;
else;
   %
   % proper priors on delta and Gamma
   %
   if isempty(tm.Z2)==0;
      D_sigma = tm.prior.deter.sigma;
      G_sigma = tm.prior.shortrun.sigma;
      Z = tm.Z2;
      if tm.d>0;
         if size(tm.Z2)>tm.d;
            F_sigma = [[D_sigma zeros(tm.d,size(G_sigma,1))];[zeros(size(G_sigma,1),tm.d) G_sigma]];
         else;
            F_sigma = D_sigma;
         end;
      else;
         F_sigma = G_sigma;
      end;
      N = eye(tm.T)-(Z'*inverse(((Z*Z')+inverse(F_sigma)))*Z);
      z = det(F_sigma)*det(((Z*Z')+inverse(F_sigma)));
   else;
      N = eye(tm.T);
      z = 1;
   end;
   num_z = 0;
end;

C_1 = (tm.Z1*N*tm.Z1')+((1/(tm.lambda_alpha^2))*eye(tm.n));

log_k1 = ((q/2)*log(det(A)))-( ((tm.T-num_z)*(tm.n/2)*log(pi)) + ((tm.n/2)*log(z)) + gamma_product(tm.n,q) );

alpha = res.posterior_mode.alpha;
beta = res.posterior_mode.beta;

if tm.beta_rest.tot_free>0;
   %
   % The restriction matrices for vec(beta) = h + H*psi
   %
   h_vec = tm.beta_rest.h_vec;
   H_mat = tm.beta_rest.H_mat;
   %
   % and the free parameters of vec(beta)
   %
   psi = inverse(H_mat'*H_mat)*H_mat'*(vec(beta)-h_vec);
end;
%
% Step 1: The value of the log posterior for alpha, psi conditional on data
%
W = tm.Z0-(alpha*beta'*tm.Z1);
Q = A+((1/(tm.lambda_alpha^2))*alpha*beta'*beta*alpha')+(W*N*W');
log_numerator = log_k1 + gamma_product(tm.n,tm.T+q+tm.r-num_z) + gamma_product(tm.r,tm.n,tm.r) - ((( ((2*tm.n)-tm.r)*tm.r)/2)*log(pi)) - ((tm.n*tm.r)*log(tm.lambda_alpha)) - (((tm.T+q+tm.r-num_z)/2)*log(det(Q)));
%
% Step 2: The value of the log marginal posterior of alpha conditional on beta
%
df = tm.T+q-tm.n-num_z;
alpha_hat = tm.Z0*N*tm.Z1'*beta*inverse(beta'*C_1*beta);
W = tm.Z0-(alpha_hat*beta'*tm.Z1);
P_mat = inverse(A+(tm.Z0*N*W'));
Q_mat = inverse(beta'*C_1*beta);
log_alpha_marg_like = matrix_t_density(alpha_hat,P_mat,Q_mat,df,alpha);
%
% Step 3: The value of the log marginal posterior of psi
%
log_beta_marg_like = 0;
if tm.beta_rest.tot_free>0;
   file = strtrim(tm.betafilename);
   %
   % load gibbs draws
   %
   gibbs_output = load(file);
   s = size(psi,1);
   norm_data = zeros(tm.save_draws,2);
   %
   check_alpha_n = isfield(gibbs_output.gibbs_results.posterior(1).alpha_n);
   for i=1:tm.save_draws;
      if check_alpha_n==1;
         alpha = gibbs_output.gibbs_results.posterior(i).alpha_n;
         Omega = gibbs_output.gibbs_results.posterior(i).Omega_n;
      else;
         alpha = gibbs_output.gibbs_results.posterior(i).alpha;
         Omega = gibbs_output.gibbs_results.posterior(i).Omega;
      end;
      %
      cov_mat = kron(alpha'*inverse(Omega)*alpha,tm.Z1*N*tm.Z1');
      mean_vec = vec(tm.Z1*N*tm.Z0'*inverse(Omega)*alpha);
      %
      B_sigma = kron((1/(tm.lambda_alpha^2))*alpha'*inverse(Omega)*alpha,eye(tm.n));
      cov_mat = cov_mat+B_sigma;
      %
      cov_mat_b_inv = cov_mat;
      cov_mat = inverse(cov_mat);
      mean_vec = cov_mat*mean_vec;
      cov_mat_psi = inverse((H_mat'*cov_mat_b_inv*H_mat));
      mean_vec_psi = cov_mat_psi*H_mat'*cov_mat_b_inv*(mean_vec-h_vec);
      %
      psi_dev = psi-mean_vec_psi;
      norm_data(i,1) = -(1/2)*psi_dev'*inverse(cov_mat_psi)*psi_dev;
      norm_data(i,2) = det(cov_mat_psi)^(-1/2);
      drawnow;
   end;
   test_vec = log(norm_data(:,2))+norm_data(:,1)-((s/2)*log(2*pi));
   mean_test_vec = mean(test_vec);
   test_vec = test_vec-mean_test_vec;
   exp_value = exp(test_vec);
   stat = sum(isinf(exp_value));
   if stat==0;
      log_beta_marg_like = log((1/tm.save_draws)*sum(exp_value))+mean_test_vec;
   end;
end;

res.posterior_mode.log_post = log_numerator-log_alpha_marg_like-log_beta_marg_like;

%
% shut down the wait dialog
%

set(waitf.text,'UserData','done');
delete(mywait);
drawnow;

%
% end of margin_beta.m
%
