%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for selecting the value             %
%          for the hyperparameter determining the mean           %
%          for the prior distribution of Omega                   %
%                                                                %
%          Function: omegaprior_dlg                              %
%                                                                %
%          Input:                                                %
%                       selector: case value string              %
%                       curr_ini: var.ini settings               %
%                                                                %
%          Output:                                               %
%          	            h: handle to figure                      %
%                                                                %
%          This version:        March 16, 2006                   %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = omegaprior_dlg(selector,curr_ini)

if strcmp(selector,'init')==1;
   tm.ini = curr_ini;
   tm.lambda_a = curr_ini.lambda_a;
   tm.old.lambda_a = curr_ini.lambda_a;
   tm.amat_type = curr_ini.amat_type;
   tm.old.amat_type = curr_ini.amat_type;
   tm.q_df = curr_ini.q_df;
   tm.old.q_df = curr_ini.q_df;
   tm.amat_lag = curr_ini.amat_lag;
   tm.old.amat_lag = curr_ini.amat_lag;
   tm.realscrsz = get(0,'ScreenSize');
   width = 500;
   height = 380;
   lag_str = strvcat('Model lag order','1 lag');
   for i=2:curr_ini.maxlag;
      lag_str = strvcat(lag_str,[num2str(i,'%0.0f') ' lags']);
   end;
else;
   omegaprior_gui = findobj('Type','figure','Tag','OmegaPriorDlg');
   omegaprior_controls = get(omegaprior_gui,'UserData');
   tm = get(omegaprior_controls.apply,'UserData');
   curr_ini = tm.ini;
end;

switch selector;
   case 'init';
      omegaprior_gui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','omegaprior_dlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(tm.realscrsz(3)-width)/2 (tm.realscrsz(4)-height)/2 width height], ...
         'Tag','OmegaPriorDlg', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name','Hyperparameters - Prior for Omega Parameters', ...
         'NumberTitle','off', ...
         'WindowStyle','modal', ...
         'CloseRequestFcn','omegaprior_dlg done', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(omegaprior_gui,'DockControl','off');
      end;

      axesbox([2 2 width-2 height-10],'Omega Parameters',92,[0.5 1]);

      %
      % GUI Header
      %

      mycons = getstoredimages;
      header_txt = 'Hyperparameters for Prior Distribution';
      desc_txt = 'The prior distribution for the parameters of the residual covariance matrix is inverted Wishart with parameters A and q as described in the help file. Press the "Apply" button to save your changes, the "Cancel" button to reset the values, and the "OK" button to finish.';
      omegaprior_controls.icon = dialog_header_fcn([16 height-109 width-30 85],mirrorimage(maketrans(mycons.lambdabig_image,[160 160 164],[1 1 1])),header_txt,desc_txt);
      omegaprior_controls.icon = omegaprior_controls.icon(1,1);
      
      axesbox([16 55 width-30 height-180],'Hyperparameters',80,[0.5 1]);
      
      %
      % create the header text and popup controls
      %

      omegaprior_controls.amat_type0 = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[40 height-170 400 20], ...
         'Style','radiobutton', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Visible','on', ...
         'Value',curr_ini.amat_type, ...
         'Enable','on', ...
         'Callback','omegaprior_dlg amat_type', ...
         'String','Use ML estimate of Omega from full rank model as A matrix', ...
         'Tag','AmatRadio0');

      omegaprior_controls.lag_text = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[30 height-200 165 20], ...
         'Style','text', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'HorizontalAlignment','left', ...
         'String','Full rank system with:', ...
         'Tag','LagText');

      omegaprior_controls.lag = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[width-230 height-195 200 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Visible','on', ...
         'Value',curr_ini.amat_lag, ...
         'Callback','omegaprior_dlg amat_lag', ...
         'String',lag_str, ...
         'Tag','LagPopup');

      omegaprior_controls.amat_type1 = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[40 height-230 400 20], ...
         'Style','radiobutton', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Visible','on', ...
         'Value',1-curr_ini.amat_type, ...
         'Enable','on', ...
         'Callback','omegaprior_dlg amat_type', ...
         'String','Use A matrix shrinkage parameter', ...
         'Tag','AmatRadio1');

      omegaprior_controls.lambda_atext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[30 height-270 165 20], ...
         'Style','text', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'HorizontalAlignment','left', ...
         'String','A matrix shrinkage:', ...
         'Tag','LambdaOmegaText');

      omegaprior_controls.lambda_a = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[width-230 height-265 200 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Visible','on', ...
         'Value',1, ...
         'Enable','off', ...
         'Callback','omegaprior_dlg omega', ...
         'String','0.01|0.05|0.10|0.15|0.20|0.25|0.30|0.35|0.40|0.45|0.50|0.55|0.60|0.65|0.70|0.75|0.80|0.85|0.90|0.95|1.00|1.05|1.10|1.15|1.20|1.25|1.30|1.35|1.40|1.45|1.50|1.55|1.60|1.65|1.70|1.75|1.80|1.85|1.90|1.95|2.00|2.10|2.20|2.30|2.40|2.50|2.60|2.70|2.80|2.90|3.00|3.25|3.50|3.75|4.00|4.25|4.50|4.75|5.00|5.50|6.00|6.50|7.00|7.50|8.00|8.50|9.00|9.50|10.00|15.00|20.00|25.00|30.00|35.00|40.00|45.00|50.00|55.00|60.00|65.00|70.00|75.00|80.00|85.00|90.00|95.00|100.00|105.00|110.00|115.00|120.00|125.00|130.00|135.00|140.00|145.00|150.00|155.00|160.00|165.00|170.00|175.00|180.00|185.00|190.00|195.00|200.00|210.00|220.00|230.00|240.00|250.00|260.00|270.00|280.00|290.00|300.00|325.00|350.00|375.00|400.00|425.00|450.00|475.00|500.00|550.00|600.00|650.00|700.00|750.00|800.00|850.00|900.00|950.00|1000.00|1500.00|2000.00|2500.00|3000.00|3500.00|4000.00|4500.00|5000.00|5500.00|6000.00|6500.00|7000.00|7500.00|8000.00|8500.00|9000.00|9500.00|10000.00', ...
         'Tag','LambdaOmegaPopup');

      omegaprior_controls.q_dftext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[30 height-310 165 20], ...
         'Style','text', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'HorizontalAlignment','left', ...
         'String','q degrees of freedom parameter:', ...
         'Tag','qDFText');

      omegaprior_controls.q_df = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[width-290 height-305 260 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Visible','on', ...
         'Enable','off', ...
         'Callback','omegaprior_dlg q_df', ...
         'String','Number of endogenous variables + 2 (minimum)|10 percent of sample size|20 percent of sample size|30 percent of sample size|40 percent of sample size|50 percent of sample size|60 percent of sample size|70 percent of sample size|80 percent of sample size|90 percent of sample size|Sample size (maximum)', ...
         'Value',curr_ini.q_df, ...
         'Tag','qDFPopup');


      d_str = get(omegaprior_controls.lambda_a,'String');
      for i=1:size(d_str,1);
         if tm.lambda_a==mystr2num(strtrim(d_str(i,:)));
            set(omegaprior_controls.lambda_a,'Value',i);
            break;
         end;
      end;

      if curr_ini.amat_type==1;
         set(omegaprior_controls.lambda_atext,'Enable','off');
         set(omegaprior_controls.lambda_a,'Enable','off');
         set(omegaprior_controls.q_df,'Enable','off');
         set(omegaprior_controls.q_dftext,'Enable','off');
      else;
         set(omegaprior_controls.lambda_atext,'Enable','on');
         set(omegaprior_controls.lambda_a,'Enable','on');
         set(omegaprior_controls.q_df,'Enable','on');
         set(omegaprior_controls.q_dftext,'Enable','on');
         set(omegaprior_controls.lag_text,'Enable','off');
         set(omegaprior_controls.lag,'Enable','off');
      end;

      %
      % Buttons
      %

      omegaprior_controls.helpbutton = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 20 20 20], ...
         'String','?', ...
         'TooltipString','Open the help file', ...
         'FontWeight','bold', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'ForegroundColor',[0.5 0 0], ...
         'Enable','on', ...
         'Callback','help_fcn(''PriorDistributions'');', ...
         'Tag','HelpButton');

      omegaprior_controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-295 20 90 20], ...
         'String','OK', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','omegaprior_dlg done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      omegaprior_controls.cancel = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-200 20 90 20], ...
         'String','Cancel', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','omegaprior_dlg cancel', ...
         'Enable','on', ...
         'Tag','Cancel');

      omegaprior_controls.apply = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-105 20 90 20], ...
         'String','Apply', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','omegaprior_dlg apply', ...
         'UserData',tm, ...
         'Enable','off', ...
         'Tag','Apply');

      set(omegaprior_gui,'UserData',omegaprior_controls,...
         'HandleVisibility','callback');

   case 'omega'
      d_val = get(omegaprior_controls.lambda_a,'Value');
      d_str = get(omegaprior_controls.lambda_a,'String');
      newlambda_a = mystr2num(strtrim(d_str(d_val,:)));
      if abs(newlambda_a-tm.lambda_a)>0;
         tm.lambda_a = newlambda_a;
         set(omegaprior_controls.apply,'Enable','on','UserData',tm);
      end;
   case 'amat_lag'
      lag_val = get(omegaprior_controls.lag,'Value');
      if abs(lag_val-tm.amat_lag)>0;
         tm.amat_lag = lag_val;
         set(omegaprior_controls.apply,'Enable','on','UserData',tm);
      end;
   case 'amat_type'
      if gcbo==omegaprior_controls.amat_type0;
         tm.amat_type = 1;
      else;
         tm.amat_type = 0;
      end;
      set(omegaprior_controls.amat_type0,'Value',tm.amat_type);
      set(omegaprior_controls.amat_type1,'Value',1-tm.amat_type);
      if tm.amat_type==1;
         set(omegaprior_controls.lambda_atext,'Enable','off');
         set(omegaprior_controls.lambda_a,'Enable','off');
         set(omegaprior_controls.q_df,'Enable','off');
         set(omegaprior_controls.q_dftext,'Enable','off');
         set(omegaprior_controls.lag_text,'Enable','on');
         set(omegaprior_controls.lag,'Enable','on');
      else;
         set(omegaprior_controls.lambda_atext,'Enable','on');
         set(omegaprior_controls.lambda_a,'Enable','on');
         set(omegaprior_controls.q_df,'Enable','on');
         set(omegaprior_controls.q_dftext,'Enable','on');
         set(omegaprior_controls.lag_text,'Enable','off');
         set(omegaprior_controls.lag,'Enable','off');
      end;
      if tm.amat_type~=curr_ini.amat_type;
         set(omegaprior_controls.apply,'Enable','on','UserData',tm);
      end;
   case 'q_df'
      q_val = get(omegaprior_controls.q_df,'Value');
      if q_val~=tm.q_df;
         tm.q_df = q_val;
         set(omegaprior_controls.apply,'Enable','on','UserData',tm);
      end;
   case 'cancel'
      tm.amat_type = tm.old.amat_type;
      tm.lambda_a = tm.old.lambda_a;
      tm.q_df = tm.old.q_df;
      tm.amat_lag = tm.old.amat_lag;
      curr_ini.lambda_a = tm.lambda_a;
      curr_ini.amat_type = tm.amat_type;
      curr_ini.q_df = tm.q_df;
      curr_ini.amat_lag = tm.amat_lag;
      tm.ini = curr_ini;
      set(omegaprior_controls.apply,'Enable','off','UserData',tm);
      %
      % Now we fix the controls
      %
      d_str = get(omegaprior_controls.lambda_a,'String');
      for i=1:size(d_str,1);
         if tm.lambda_a==mystr2num(strtrim(d_str(i,:)));
            set(omegaprior_controls.lambda_a,'Value',i);
            break;
         end;
      end;
      set(omegaprior_controls.amat_type0,'Value',tm.amat_type);
      set(omegaprior_controls.amat_type1,'Value',1-tm.amat_type);
      set(omegaprior_controls.q_df,'Value',tm.q_df);
      set(omegaprior_controls.lag,'Value',tm.amat_lag);
      if tm.amat_type==1;
         set(omegaprior_controls.lambda_atext,'Enable','off');
         set(omegaprior_controls.lambda_a,'Enable','off');
         set(omegaprior_controls.q_df,'Enable','off');
         set(omegaprior_controls.q_dftext,'Enable','off');
         set(omegaprior_controls.lag_text,'Enable','on');
         set(omegaprior_controls.lag,'Enable','on');
      else;
         set(omegaprior_controls.lambda_atext,'Enable','on');
         set(omegaprior_controls.lambda_a,'Enable','on');
         set(omegaprior_controls.q_df,'Enable','on');
         set(omegaprior_controls.q_dftext,'Enable','on');
         set(omegaprior_controls.lag_text,'Enable','off');
         set(omegaprior_controls.lag,'Enable','off');
      end;
   case 'apply'
      curr_ini.lambda_a = tm.lambda_a;
      curr_ini.amat_type = tm.amat_type;
      curr_ini.q_df = tm.q_df;
      curr_ini.amat_lag = tm.amat_lag;
      tm.ini = curr_ini;
      set(omegaprior_controls.apply,'Enable','off','UserData',tm);
   case 'done'
      set(omegaprior_controls.done,'UserData','done');
end;

if strcmp(selector,'init')==1;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(omegaprior_gui,'Name') ' ' fix_filepath([pwd '\icons\lambda_params.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(omegaprior_controls.icon,'UserData',pid);
      end;
   end;
   %
   set(omegaprior_gui,'Visible','on');
   drawnow;
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(omegaprior_gui,'Name') ' 3'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(omegaprior_controls.done);
   end;
   waitfor(omegaprior_controls.done,'UserData','done');
end;

if nargout==1;
   h = omegaprior_gui;
end;

%
% end of omegaprior_dlg.m
%
