function [parm_matrix,param_mode,param_median,param_lower,param_upper,param_mean] = parameter_stats(parm_matrix,compute_mean,cumdist,cbs)

%
% compute mode, median, and mean of parameter and organize the distribution
%
% parm_matrix has parameter values in first column and density value in second column.
% The density values do not necessarily include a common integration constant.
% Computation of the mean depends on the boolean variable "compute_mean".
% Computation of cumulative distribution depends on boolean variable "cumdist"
%

[val,ind] = max(parm_matrix(:,2));

param_mode = parm_matrix(ind,1);

dx = parm_matrix(2,1)-parm_matrix(1,1);
param_mat = parm_matrix;
param_mat(:,2) = dx*param_mat(:,2);
param_mat(:,2) = cumsum(param_mat(:,2));
max_val = max(param_mat(:,2));
param_mat(:,2) = (1/max_val)*param_mat(:,2);

%
% compute the median
%

n = size(param_mat,1);
dist_mat = param_mat(:,2);
median_mat = dist_mat(dist_mat<=0.5);
m_val = size(median_mat,1);
if m_val<n;
   param_median = (parm_matrix(m_val,1)+parm_matrix(m_val+1,1))/2;
else;
   param_median = parm_matrix(n,1);
end;

%
% compute lower and upper bands.
%
% choose the quantile values with the smallest difference between parameter values
% that are consistent with a cbs percent band
%

ul_mat = zeros(size(dist_mat),3);
for i=1:size(dist_mat,1);
   low_val = dist_mat(i,1);
   if low_val<=(1-(cbs/100));
      up_val = low_val+(cbs/100);
      j = dist_mat(dist_mat<=up_val);
      j = size(j,1);
      ul_mat(i,:) = [i j parm_matrix(j,1)-parm_matrix(i,1)];
   else;
      if i>1;
         ul_mat = ul_mat(1:i-1,:);
      else;
         ul_mat = [];
      end;
      break;
   end;
end;
if isempty(ul_mat)==0;
   [diff_val,ind] = min(ul_mat(:,3));
   i = ul_mat(ind,1);
   j = ul_mat(ind,2);
   param_lower = parm_matrix(i,1);
   param_upper = parm_matrix(j,1);
else;
   param_lower = [];
   param_upper = [];
end;

%l_b = (1-(cbs/100))/2;
%u_b = 1-l_b;

%l_mat = dist_mat(dist_mat<=l_b);
%l_val = size(l_mat,1);
%if l_val<n;
%   param_lower = (parm_matrix(l_val,1)+parm_matrix(l_val+1,1))/2;
%else;
%   param_lower = parm_matrix(n,1);
%end;

%u_mat = dist_mat(dist_mat<=u_b);
%u_val = size(u_mat,1);
%if u_val<n;
%   param_upper = (parm_matrix(u_val,1)+parm_matrix(u_val+1,1))/2;
%else;
%   param_upper = parm_matrix(n,1);
%end;

%
% compute the mean if asked to
%

if compute_mean==1;
   param_mean = sum(parm_matrix(:,1) .* ((dx/max_val)*parm_matrix(:,2)));
else;
   param_mean = [];
end;

if cumdist==1;
   parm_matrix = param_mat;
else;
   parm_matrix(:,2) = (1/max_val)*parm_matrix(:,2);
end;

%
% end of parameter_stats.m
%
