function res = posterior_mode(tm,curr_ini,r,s,A,q,D_sigma,G_sigma,prior_inf_alpha,prior_inf_omega,beta_rest_bool)

res = [];

if (tm.prior_inf_shortrun==2)&(tm.prior_inf_deter==2);
   %
   % improper priors on delta and Gamma
   %
   if isempty(tm.Z2)==0;
      Z = tm.Z2;
      N = eye(tm.T)-Z'*inverse(Z*Z')*Z;
   else;
      N = eye(tm.T);
   end;
   num_z = 0;
   %
   % matrices need for Gamma and delta posterior mode values
   %
   if tm.d>0;
      D = tm.Z2(1:tm.d,:);
      M_D = eye(tm.T)-(D'*inverse(D*D')*D);
   else;
      M_D = eye(tm.T);
   end;
   if size(tm.Z2,1)>tm.d;
      Z2 = tm.Z2(tm.d+1:size(tm.Z2,1),:);
      G_N = M_D*Z2'*inverse(Z2*M_D*Z2');
      if tm.d>0;
         P_N = (eye(tm.T)-(G_N*Z2))*D'*inverse(D*D');
      else;
         P_N = [];
      end;
   else;
      G_N = [];
      if tm.d>0;
         P_N = D'*inverse(D*D');
      else;
         P_N = [];
      end;
   end;
elseif (tm.prior_inf_shortrun==1)&(tm.prior_inf_deter==2);
   %
   % improper prior on delta
   %
   if isempty(tm.Z2)==0;
      if tm.d>0;
         D = tm.Z2(1:tm.d,:);
         M_D = eye(tm.T)-(D'*inverse(D*D')*D);
      else;
         D = 1;
         M_D = eye(tm.T);
      end;
      if size(tm.Z2,1)>tm.d;
         Z2 = tm.Z2(tm.d+1:size(tm.Z2,1),:);
         G_sigma = tm.prior.shortrun.sigma;
         N = M_D-(M_D*Z2'*inverse(((Z2*M_D*Z2')+inverse(G_sigma)))*Z2*M_D);
      else;
         N = M_D;
      end;
   else;
      N = eye(tm.T);
   end;
   num_z = size(tm.Z2,1)-tm.d;
   %
   % matrices need for Gamma and delta posterior mode values
   %
   if tm.d>0;
      D = tm.Z2(1:tm.d,:);
      M_D = eye(tm.T)-(D'*inverse(D*D')*D);
   else;
      M_D = eye(tm.T);
   end;
   if size(tm.Z2,1)>tm.d;
      Z2 = tm.Z2(tm.d+1:size(tm.Z2,1),:);
      G_N = M_D*Z2'*inverse((Z2*M_D*Z2')+inverse(G_sigma));
      if tm.d>0;
         P_N = (eye(tm.T)-(G_N*Z2))*D'*inverse(D*D');
      else;
         P_N = [];
      end;
   else;
      G_N = [];
      if tm.d>0;
         P_N = D'*inverse(D*D');
      else;
         P_N = [];
      end;
   end;
elseif (tm.prior_inf_shortrun==2)&(tm.prior_inf_deter==1);
   %
   % improper prior on Gamma
   %
   if isempty(tm.Z2)==0;
      if tm.d>0;
         D_sigma = tm.prior.deter.sigma;
         D = tm.Z2(1:tm.d,:);
         if size(tm.Z2)>tm.d;
            Z2 = tm.Z2(tm.d+1:size(tm.Z2,1),:);
            M_Z2 = eye(tm.T)-(Z2'*inverse(Z2*Z2')*Z2);
            N = M_Z2-(M_Z2*D'*inverse(((D*M_Z2*D')+inverse(D_sigma)))*D*M_Z2);
         else;
            N = eye(tm.T)-D'*inverse(((D*D')+inverse(D_sigma)))*D;
         end;
      else;
         Z2 = tm.Z2;
         M_Z2 = eye(tm.T)-(Z2'*inverse(Z2*Z2')*Z2);
         N = M_Z2;
      end;
   else;
      N = eye(tm.T);
   end;
   num_z = tm.d;
   %
   % matrices need for Gamma and delta posterior mode values
   %
   if tm.d>0;
      D = tm.Z2(1:tm.d,:);
      M_D = eye(tm.T)-(D'*inverse((D*D')+inverse(D_sigma))*D);
   else;
      M_D = eye(tm.T);
   end;
   if size(tm.Z2,1)>tm.d;
      Z2 = tm.Z2(tm.d+1:size(tm.Z2,1),:);
      G_N = M_D*Z2'*inverse(Z2*M_D*Z2');
      if tm.d>0;
         P_N = (eye(tm.T)-(G_N*Z2))*D'*inverse((D*D')+inverse(D_sigma));
      else;
         P_N = [];
      end;
   else;
      G_N = [];
      if tm.d>0;
         P_N = D'*inverse((D*D')+inverse(D_sigma));
      else;
         P_N = [];
      end;
   end;
else;
   %
   % proper priors on delta and Gamma
   %
   if isempty(tm.Z2)==0;
      D_sigma = tm.prior.deter.sigma;
      G_sigma = tm.prior.shortrun.sigma;
      Z = tm.Z2;
      if tm.d>0;
         if size(tm.Z2)>tm.d;
            F_sigma = [[D_sigma zeros(tm.d,size(G_sigma,1))];[zeros(size(G_sigma,1),tm.d) G_sigma]];
         else;
            F_sigma = D_sigma;
         end;
      else;
         F_sigma = G_sigma;
      end;
      N = eye(tm.T)-(Z'*inverse(((Z*Z')+inverse(F_sigma)))*Z);
   else;
      N = eye(tm.T);
   end;
   num_z = size(tm.Z2,1);
   if tm.d>0;
      D = tm.Z2(1:tm.d,:);
      M_D = eye(tm.T)-(D'*inverse((D*D')+inverse(D_sigma))*D);
   else;
      M_D = eye(tm.T);
   end;
   if size(tm.Z2,1)>tm.d;
      Z2 = tm.Z2(tm.d+1:size(tm.Z2,1),:);
      G_N = M_D*Z2'*inverse((Z2*M_D*Z2')+inverse(G_sigma));
      if tm.d>0;
         P_N = (eye(tm.T)-(G_N*Z2))*D'*inverse((D*D')+inverse(D_sigma));
      else;
         P_N = [];
      end;
   else;
      G_N = [];
      if tm.d>0;
         P_N = D'*inverse((D*D')+inverse(D_sigma));
      else;
         P_N = [];
      end;
   end;
end;

if (prior_inf_alpha==1)&(prior_inf_omega==1);
   %
   % informative prior on alpha and Omega
   %
   %
   % Setting up the Sij matrices
   %
   S_00 = (1/(tm.T+tm.n+q+r+1+num_z))*((tm.Z0*N*tm.Z0')+A);
   S_01 = (1/(tm.T+tm.n+q+r+1+num_z))*tm.Z0*N*tm.Z1';
   S_10 = S_01';
   S_11 = (1/(tm.T+tm.n+q+r+1+num_z))*((tm.Z1*N*tm.Z1')+((1/tm.lambda_alpha^2)*eye(tm.n)));
elseif (prior_inf_alpha==1)&(prior_inf_omega==2);
   %
   % informative prior on alpha, but not on Omega
   %
   %
   % Setting up the Sij matrices
   %
   S_00 = (1/(tm.T+tm.n+r+1+num_z))*((tm.Z0*N*tm.Z0'));
   S_01 = (1/(tm.T+tm.n+r+1+num_z))*tm.Z0*N*tm.Z1';
   S_10 = S_01';
   S_11 = (1/(tm.T+tm.n+r+1+num_z))*((tm.Z1*N*tm.Z1')+((1/tm.lambda_alpha^2)*eye(tm.n)));
elseif (prior_inf_alpha==2)&(prior_inf_omega==1);
   %
   % informative prior on Omega but not on alpha
   %
   %
   % Setting up the Sij matrices
   %
   S_00 = (1/(tm.T+tm.n+q+1+num_z))*((tm.Z0*N*tm.Z0')+A);
   S_01 = (1/(tm.T+tm.n+q+1+num_z))*tm.Z0*N*tm.Z1';
   S_10 = S_01';
   S_11 = (1/(tm.T+tm.n+q+1+num_z))*((tm.Z1*N*tm.Z1'));
elseif (prior_inf_alpha==2)&(prior_inf_omega==2);
   %
   % non-informative priors on alpha and Omega
   %
   %
   % Setting up the Sij matrices
   %
   S_00 = (1/(tm.T+tm.n+1+num_z))*((tm.Z0*N*tm.Z0'));
   S_01 = (1/(tm.T+tm.n+1+num_z))*tm.Z0*N*tm.Z1';
   S_10 = S_01';
   S_11 = (1/(tm.T+tm.n+1+num_z))*((tm.Z1*N*tm.Z1'));
end;

if r>0;
   %
   % Now solve Johansen's eigenvalue problem
   %
   %
   % Step 1: Diagonalize S_11
   %
   [W,rho] = eig(S_11);
   rho = diag(rho,0);
   [rho2,AscInd] = sort(abs(rho));
   nr = size(AscInd,1);
   DescInd = zeros(nr,1);
   i = 1;
   while i<=nr;
      DescInd(i)=AscInd(nr+1-i);
      i =i+1;
   end;
   %
   % Resort eigenvalues and eigenvectors
   % in descending order
   %
   rho = rho(DescInd);
   sqrho = sqrt(rho);
   rho = diag(rho,0);
   sqrho = diag(sqrho,0);
   W = W(:,DescInd);
   %
   % Step 2: Reformulating eigenvalue problem to
   %         calculating the eigenvalues for
   %
   %         adjS_11 = sqS_11*S_10*inverse(S_00)*S_01*sqS_11
   %
   sqS_11 = (W/sqrho)*W';
   adjS_11 = sqS_11*(S_10/S_00)*S_01*sqS_11;
   [U,lambda] = eig(adjS_11);
   lambda = diag(lambda,0);
   [lambda2,AscInd] = sort(abs(lambda));
   nr = size(AscInd,1);
   DescInd = zeros(nr,1);
   i = 1;
   while i<=nr;
      DescInd(i)=AscInd(nr+1-i);
      i =i+1;
   end;
   %
   % Resort eigenvalues and eigenvectors
   % in descending order
   %
   lambda = abs(lambda(DescInd));
   U = U(:,DescInd);
   %
   % Renormalize U such that V'S_11V=I
   %
   V = sqS_11*U;
   beta = V(:,1:r);
else;
   beta = [];
end;

beta_cs = beta;

if beta_rest_bool==0;
   %
   % there are no overidentifying restrictions on beta
   %
   if r>0;
      c_mat = tm.rank(r+1).c_mat;
      c_mat_orth = tm.rank(r+1).c_mat_orth;
      c_bar = c_mat*inverse(c_mat'*c_mat);
      c_bar_orth = c_mat_orth*inverse(c_mat_orth'*c_mat_orth);
      psi = c_bar_orth'*beta*inverse(c_bar'*beta);
      alpha = S_01*beta*beta'*c_bar;
      beta = beta*inverse(c_bar'*beta);
      Omega = S_00-(S_01*beta*inverse(beta'*S_11*beta)*beta'*S_10);
   else;
      alpha = [];
      Omega = S_00;
   end;
else;
   %
   % there are over-identifying restrictions on beta
   %
   if s>0;
      [beta,newlambda,dfbeta,stdbeta,outputtxt,abort,infomat,conv_txt] = betarestrict(S_00,S_11,S_01,S_10,beta,tm.T,tm.modelsdir,tm.modelfile,0,tm.dcoint);
   else;
      beta = vecinv(tm.beta_rest.h_vec,tm.n);
   end;
   alpha = S_01*beta*inverse(beta'*S_11*beta);
   Omega = S_00-(S_01*beta*inverse(beta'*S_11*beta)*beta'*S_10);
end;
if isempty(P_N)==0;
   if r>0;
      delta = (tm.Z0-(alpha*beta'*tm.Z1))*P_N;
   else;
      delta = tm.Z0*P_N;
   end;
else;
   delta = [];
end;
if isempty(G_N)==0;
   if r>0;
      Gamma = (tm.Z0-(alpha*beta'*tm.Z1))*G_N;
   else;
      Gamma = tm.Z0*G_N;
   end;
else;
   Gamma = [];
end;
Psi = [delta Gamma];
%
% computing the maximum non-unit eigenvalue
%
e_values = companionrest(alpha,Psi,beta,tm.p,tm.n,tm.d);
e_values = sort(abs(e_values));
for i1=1:size(e_values,1);
   max_evalue = e_values(size(e_values,1)-i1+1,1);
   if strcmp(num2str(max_evalue,'%1.10f'),'1.0000000000')==0;
      break;
   end;
end;
%
% computing the value of the log posterior
%
if beta_rest_bool==0;
   lpr = logprior_rank(delta,Gamma,Omega,alpha,beta,tm.n,r,tm.d,tm.prior_inf_deter,A,q,tm.lambda_alpha,tm.prior.deter.mu,tm.prior.deter.sigma,tm.prior.shortrun.mu,tm.prior.shortrun.sigma);
else;
   lpr = logprior_beta(delta,Gamma,Omega,alpha,beta,tm.n,r,tm.d,tm.prior_inf_deter,prior_inf_alpha,prior_inf_omega,A,q,tm.lambda_alpha,tm.prior.deter.mu,tm.prior.deter.sigma,tm.prior.shortrun.mu,tm.prior.shortrun.sigma);
end;
epsilon = tm.Z0;
if isempty(tm.Z2)==0;
   epsilon = epsilon-(Psi*tm.Z2);
end;
if r>0;
   epsilon = epsilon-(alpha*beta'*tm.Z1);
end;
log_like = loglikefcn(Omega,epsilon);
log_posterior = log_like+lpr;
%
% preparing results for output
%
res.alpha = alpha;
res.beta = beta;
res.delta = delta;
res.Gamma = Gamma;
res.Psi = Psi;
res.Omega = Omega;
res.beta_cs = beta_cs;
res.max_evalue = max_evalue;
res.log_posterior = log_posterior;
res.log_like = log_like;
res.log_prior = lpr;

%
% end of posterior_mode.m
%
