function edit_legend(gco)
%Edit a legend

if ~strcmp(get(gco,'type'),'text'), return, end

% Determine which string was clicked on
units = get(gco,'units');
set(gco,'units','data')
cp = get(gca,'currentpoint');
ext = get(gco,'extent');
nstrings = size(get(gco,'string'),1);

% The k-th string (from the top) was clicked on
k = floor((ext(4) - cp(1,2))/ext(4)*nstrings) + 1;
legh = get(gco,'parent');
ud = get(legh,'userdata');

newcontext = [];
if isfield(ud,'UIContextMenu')==1;
   if ishandle(ud.UIContextMenu)==1;
      kids = get(ud.UIContextMenu,'Children')';
      if isempty(kids)==0;
         kids = sort(kids);
         newcontext = uicontextmenu('Callback','set(gcf,''Pointer'',''arrow'')');
         for i=1:size(kids,2);
            uimenu('Parent',newcontext,'Label',get(kids(1,i),'Label'),'Separator',get(kids(1,i),'Separator'),'Callback',get(kids(1,i),'Callback'));
         end;
      end;
   end;
end;

nrows = cellfun('size',ud.lstrings,1);
crows = cumsum(nrows);

% Determine which string in the cell array was clicked on
active_string = floor( ...
            interp1([0 cumsum(nrows)+1],[1:length(nrows) length(nrows)],k));
if isnan(active_string), return, end

% Disable legend buttondownfcn's
savehandle = findobj('buttondownfcn','moveaxis_var');
set(savehandle,'buttondownfcn','')

% Make a editable string on top of the legend string
pos = get(gco,'position');
y = ext(4) - (crows(active_string)-nrows(active_string)/2)*ext(4)/nstrings;
pos(2) = ext(2) + y;
TextHandle = copyobj(gco,gca);
set(TextHandle,'string',char(ud.lstrings{active_string}),'position',pos, ...
               'Editing','on')
waitfor(TextHandle,'Editing');

% Protect against the handles being destroyed during the waitfor
if ishandle(TextHandle) & ishandle(legh) & ishandle(savehandle)
  ud.lstrings{active_string} = get(TextHandle,'String');
  delete(TextHandle);
  set(legh,'UserData',ud);
  %
  % find old legend font and color properties
  %
  old_color = get(legh,'Color');
  old_border_color = get(legh,'XColor');
  text_strings = findobj(legh,'Type','text')';
  if isempty(text_strings)==0;
     text_obj.color = get(text_strings(1,1),'Color');
     text_obj.fontangle = get(text_strings(1,1),'FontAngle');
     text_obj.fontname = get(text_strings(1,1),'FontName');
     text_obj.fontsize = get(text_strings(1,1),'FontSize');
     text_obj.fontweight = get(text_strings(1,1),'FontWeight');
     text_obj.fontunits = get(text_strings(1,1),'FontUnits');
  end;
  set(gco,'units',units);
  % Enable legend buttondfcn's
  set(savehandle,'buttondownfcn','moveaxis_var');
  new_leg = update_legend(legh);
  if isempty(newcontext)==0;
     if ishandle(new_leg)==1;
        new_ud = get(new_leg,'UserData');
        new_ud.UIContextMenu = newcontext;
        set(new_leg,'UIContextMenu',newcontext,'UserData',new_ud,'Color',old_color,'XColor',old_border_color,'YColor',old_border_color);
        set(get(new_leg,'Children'),'UIContextMenu',newcontext);
        new_text_strings = findobj(new_leg,'Type','text')';
        if isempty(new_text_strings)==0;
           for i=1:size(new_text_strings,2);
              set(new_text_strings(1,i),'Color',text_obj.color);
              set(new_text_strings(1,i),'FontAngle',text_obj.fontangle);
              set(new_text_strings(1,i),'FontName',text_obj.fontname);
              set(new_text_strings(1,i),'FontSize',text_obj.fontsize);
              set(new_text_strings(1,i),'FontWeight',text_obj.fontweight);
              set(new_text_strings(1,i),'FontUnits',text_obj.fontunits);
           end;
        end;
        %
        % adding legend handles as user data to lines, patches, etc
        %
        leg_ud = get(new_leg,'UserData');
        num = 1;
        for i=1:size(leg_ud.LabelHandles,1);
           if strcmp(get(leg_ud.LabelHandles(i,1),'Type'),'text')==0;
              if strcmp(get(leg_ud.handles(num),'Type'),'surface')==0;
                 set(leg_ud.handles(num),'UserData',leg_ud.LabelHandles(i));
              end;
              num = num+1;
           end;
        end;
     end;
  end;
  %
  % update legend position information
  %
  if iscompiled==1;
     toolbar = get(gcf,'UserData');
     myaxes = get(toolbar.print,'UserData');
     if isempty(get(toolbar.edit,'UserData'))==0;
        all_leg_handles = [];
        for i=1:size(myaxes,2);
           new_leg_handle = get(myaxes(1,i),'UserData');
           if isempty(new_leg_handle)==0;
              if ishandle(new_leg_handle)==1;
                 all_leg_handles = [all_leg_handles new_leg_handle];
              end;
           end;
        end;
        thislegpos = [];
        for i=1:size(all_leg_handles,2);
           old_units = get(all_leg_handles(1,i),'Units');
           set(all_leg_handles(1,i),'Units','pixels');
           newlegpos = get(all_leg_handles(1,i),'Position');
           thislegpos = [thislegpos;newlegpos];
           set(all_leg_handles(1,i),'Units',old_units);
           set(toolbar.edit,'UserData',thislegpos);
        end;
     end;
  end;
end;

%
% end of edit_legend.m
%
