function [w,stat] = wish_rnd(sigma,v,cu_symmetric)
% PURPOSE: generate random wishart matrix
%------------------------------------------------
% USAGE:     w = wish_rnd(sigma,n)
% Where: sigma = symmetric pds input matrix
%            v = degrees of freedom parameter
%------------------------------------------------
% RETURNS:
%        w = random wishart_n(sigma)
%             distributed matrix
%------------------------------------------------
% REFERENCES: Gelman, Carlin, Stern, Rubin, Bayesian Data 
%             Analysis, (1995,96) pages 474, 480-481.

% written by:
% Aki Vehtari
% Helsinki University of Technology 
% Lab. of Computational Engineering 
% P.O.Box 9400 
% FIN-02015 HUT 
% FINLAND
% Aki.Vehtari@hut.fi
%
% Revised by: Anders Warne May 19, 2005
%

n = size(sigma,1);

[t,stat] = chol(sigma);
if stat>0;
   w = [];
   return;
end;
%
if cu_symmetric==1;
   t = symm(sigma);
end;
randn('state',sum(100*clock));
y = t'*randn(n,v);
w = y*y';

%
% end of wish_rnd.m
%
