function signshock = sign_shock_fcn(tm,B)

file = strcat(tm.modelsdir,'signs-',int2str(tm.n),'x',int2str(tm.r),'-',tm.modelfile,'.rest');
if fileexist(file)==1;
   signshock = loaddata(file);
   for i=1:tm.n;
      if isalmostequal(signshock(i,2),0)==1;
         signshock(i,2) = 1;
      end;
   end;
   %
   % need to test the signshock data to make sure the signs are OK for the variables and shocks in question,
   % i.e. that a certain shock does not have a zero affect on the variable based on the selected
   % identification scheme
   %
   status = test_sign(signshock,B);
   if status==0;
      return;
   end;
end;

%
% need to create a default tm.nx2 matrix based on B (which is a random
% matrix satisfying the identifying assumptions).
%

%signshock = zeros(tm.n,2);
signshock = [zeros(tm.n,1) ones(tm.n,1)];

%
% use the diagonal elements if possible
%

if min(abs(diag(B)))>1.0e-008;
   signshock(:,1) = (1:1:tm.n)';
   return;
end;

%
% here comes the tricky part: i is variable j is shock. we
% first pick all the non-zero values of B
%

non_zero_values = [];
for i=1:tm.n;
   for j=1:tm.n;
      if abs(B(i,j))<1.0e-008;
         B(i,j) = 0;
      else;
         non_zero_values = [non_zero_values;[i j]];
      end;
   end;
end;

%
% the non_zero_values matrix tells us which pairs of i and j
% are admissible. now we pick the unique elements based on
% sorted values of j. p gives us the rows of non_zero_values
% where j=1,2,...,n are located
%

loop_this = 1;
while loop_this==1;
   [b,p,k] = unique(non_zero_values(:,2));
   vars = non_zero_values(p,1);
   [u_vars,p1,k1] = unique(vars);
   if size(u_vars,1)==size(vars,1);
      loop_this = 0;
   else;
      del_rows = [];
      for i=1:size(p,1);
         del_i = 1;
         for j=1:size(p1,1);
            if i==p1(j,1);
               del_i = 0;
            end;
         end;
         if del_i==1;
            del_rows = [del_rows;p(i,1)];
         end;
      end;
      if isempty(del_rows)==0;
         del_rows = sort(del_rows);
         for i=1:size(del_rows,1);
            del_row_num = size(del_rows,1)-i+1;
            del_row = del_rows(del_row_num,1);
            num_val = size(non_zero_values,1);
            if del_row==1;
               non_zero_values = non_zero_values(2:num_val,:);
            elseif del_row==num_val;
               non_zero_values = non_zero_values(1:num_val-1,:);
            else;
               non_zero_values = [non_zero_values(1:del_row-1,:);non_zero_values(del_row+1:num_val,:)];
            end;
         end;
      end;
   end;
end;

if size(non_zero_values(p,1),1)<size(signshock,1);
   %
   % need to check which value is missing
   %
   miss_val = [];
   check_vec = non_zero_values(p,1);
   for i=1:tm.n;
      test_value = min(abs(check_vec-i));
      if isalmostequal(test_value,0)==0;
         miss_val = [miss_val;i];
      end;
   end;
   %
   % check to see where we can
   %
   signshock(:,1) = [check_vec;miss_val];
else;
   signshock(:,1) = non_zero_values(p,1);
end;

%
% end of sign_shock_fcn.m
%
