function outvec = tracepval(n,trace_test,type_var)

%
% n = dimension
% trace_test = trace-tests
% type_var = distribution
%
% Critical values taken from: MacKinnon, J.G., Haug, A.A. & Michelis, L. (1996)
%   "Numerical Distribution Functions of Likelihood Ratio Tests for
%    Cointegration".
%

if type_var==[1 0 0 0];
   %
   % unrestricted constant
   %
   table = trace_1_model;
elseif type_var==[0 1 0 0];
   %
   % restricted constant
   %
   table = trace_2_model;
elseif type_var==[0 0 1 0];
   %
   % constant+linear trend
   %
   table = trace_3_model;
elseif type_var==[0 0 0 1];
   %
   % constant+restricted linear trend
   %
   table = trace_4_model;
end;

quantiles = quantiles_fcn;
table = table(:,1:n);

%
% find first value greater than trace-tests
%

pos = [];
for i=1:n;
   for j=1:221;
      if table(j,n-i+1)>=trace_test(i,1);
         pos = [pos;j];
         break;
      end;
      if j==221;
         pos = [pos;222];
      end;
   end;
end;

outvec = [];
for i=1:n;
   y = trace_test(i,1);
   posi = pos(i,1);
   if (posi>1)&(posi<222);
      x1 = table(posi-1,n-i+1);
      x2 = table(posi,n-i+1);
      beta = (y-x1)/(x2-x1);
      z1 = quantiles(posi-1,1);
      z2 = quantiles(posi,1);
      pval = ((1-beta)*z1)+(beta*z2);
      outvec = [outvec;(1-pval)];
   elseif posi==1;
      outvec = [outvec;1.0000];
   elseif posi==222;
      outvec = [outvec;0.0000];
   end;
end;

%
% end of tracepval.m
%
