function [beta_1,beta_2,alpha1,alpha2,Psi_n,Omega_n,epsilon_n] = varyrank(Z0,Z1,Z3,data_dummy,r1,r2)

%
% estimating the model:
%
%   Z0(t) = alpha(1)*beta(1)'*Z1(t) + alpha(2)*beta(2)'Z2(t) + Gamma*Z3(t) + epsilon(t)
%
% where alpha(1), beta(1) have rank r1
%       alpha(2), beta(2) have rank r2
%

beta_1 = [];
beta_2 = [];
alpha_1 = [];
alpha_2 = [];
[n T] = size(Z0);

%
% define Z2
%

Z2 = Z1 .* (ones(size(Z1,1),1)*data_dummy');

%
% step 1: estimate alpha(1), beta(1) by reduced rank regression if r1>0
%

if r1>0;

   if isempty(Z3)==0;
      M_03 = (1/T)*Z0*Z3';
      M_13 = (1/T)*Z1*Z3';
      M_23 = (1/T)*Z2*Z3';
      M_33 = (1/T)*Z3*Z3';
      R0 = Z0-(M_03*inverse(M_33)*Z3);
      R1 = Z1-(M_13*inverse(M_33)*Z3);
      R2 = Z2-(M_23*inverse(M_33)*Z3);
   else;
      R0 = Z0;
      R1 = Z1;
      R2 = Z2;
   end;
   
   m_02 = (1/T)*R0*R2';
   m_12 = (1/T)*R1*R2';
   m_22 = (1/T)*R2*R2';
   r_0 = R0-(m_02*inverse(m_22)*R2);
   r_1 = R1-(m_12*inverse(m_22)*R2);
   S_01 = (1/T)*r_0*r_1';
   S_00 = (1/T)*r_0*r_0';
   S_11 = (1/T)*r_1*r_1';
   S_10 = S_01';
   
   %
   % solve the eigenvalue problem
   %
   
   [W,rho] = eig(S_11);
   rho = diag(rho,0);
   [rho2,AscInd] = sort(abs(rho));
   
   nr = size(AscInd,1);
   DescInd = zeros(nr,1);
   i = 1;
   while i<=nr;
      DescInd(i)=AscInd(nr+1-i);
      i =i+1;
   end;
   
   %
   % Resort eigenvalues and eigenvectors
   % in descending order
   %
   
   rho = rho(DescInd);
   sqrho = sqrt(rho);
   rho = diag(rho,0);
   sqrho = diag(sqrho,0);
   W = W(:,DescInd);
   
   %
   % Step 2: Reformulating eigenvalue problem to
   %         calculating the eigenvalues for
   %
   %         adjS_11 = sqS_11*S_10*inverse(S_00)*S_01*sqS_11
   %
   
   sqS_11 = (W/sqrho)*W';
   
   adjS_11 = sqS_11*(S_10/S_00)*S_01*sqS_11;
   
   [U,lambda] = eig(adjS_11);
   
   lambda = diag(lambda,0);
   [lambda2,AscInd] = sort(abs(lambda));
   
   nr = size(AscInd,1);
   DescInd = zeros(nr,1);
   i = 1;
   while i<=nr;
      DescInd(i)=AscInd(nr+1-i);
      i =i+1;
   end;
   
   %
   % Resort eigenvalues and eigenvectors
   % in descending order
   %
   
   lambda = abs(lambda(DescInd));
   U = U(:,DescInd);
   
   %
   % Renormalize U such that V'S_11V=I
   %
   
   V = sqS_11*U;
   
   %
   % Remove zero eigenvalues for the case when
   % the number of variables i Z1 is greater than
   % the number of variables in Z0 and adjust
   % the eigenvectors accordingly
   %
   
   if n<size(Z1,1);;
      lambda = lambda(1:n,:);
      V = V(:,1:n);
   end;
   
   beta_1 = V(:,1:r1);
   alpha_1 = S_01*beta_1;
   Zb0 = Z0;
   if isempty(Z3)==0;
      nr_z3 = size(Z3,1);
   else;
      nr_z3 = 0;
   end;

   Z3 = [Z3;beta_1'*Z1];
%   Zb0 = Z0-(alpha_1*beta_1'*Z1);
else;
   beta_1 = [];
   alpha_1 = [];
   if isempty(Z3)==0;
      nr_z3 = size(Z3,1);
   else;
      nr_z3 = 0;
   end;
   Zb0 = Z0;
end;

%
% step 2: estimate alpha(2), beta(2) by reduced rank regression
%

if isempty(Z3)==0;
   N_03 = (1/T)*Zb0*Z3';
   N_23 = (1/T)*Z2*Z3';
   N_33 = (1/T)*Z3*Z3';
   z0 = Zb0-(N_03*inverse(N_33)*Z3);
   z2 = Z2-(N_23*inverse(N_33)*Z3);
else;
   z0 = Zb0;
   z2 = Z2;
end;

Sb_00 = (1/T)*z0*z0';
Sb_02 = (1/T)*z0*z2';
Sb_22 = (1/T)*z2*z2';
Sb_20 = Sb_02';

%
% solve the eigenvalue problem
%

[W,rho] = eig(Sb_22);
rho = diag(rho,0);
[rho2,AscInd] = sort(abs(rho));

nr = size(AscInd,1);
DescInd = zeros(nr,1);
i = 1;
while i<=nr;
   DescInd(i)=AscInd(nr+1-i);
   i =i+1;
end;

%
% Resort eigenvalues and eigenvectors
% in descending order
%

rho = rho(DescInd);
sqrho = sqrt(rho);
rho = diag(rho,0);
sqrho = diag(sqrho,0);
W = W(:,DescInd);


sqSb_22 = (W/sqrho)*W';

adjSb_22 = sqSb_22*(Sb_20/Sb_00)*Sb_02*sqSb_22;

[U,lambda] = eig(adjSb_22);

lambda = diag(lambda,0);
[lambda2,AscInd] = sort(abs(lambda));

nr = size(AscInd,1);
DescInd = zeros(nr,1);
i = 1;
while i<=nr;
   DescInd(i)=AscInd(nr+1-i);
   i =i+1;
end;

%
% Resort eigenvalues and eigenvectors
% in descending order
%

lambda = abs(lambda(DescInd));
U = U(:,DescInd);

%
% Renormalize U such that V'S_11V=I
%

V = sqSb_22*U;

%
% Remove zero eigenvalues for the case when
% the number of variables i Z1 is greater than
% the number of variables in Z0 and adjust
% the eigenvectors accordingly
%

if n<size(Z1,1);;
   lambda = lambda(1:n,:);
   V = V(:,1:n);
end;

beta_2 = V(:,1:r2);
alpha_2 = Sb_02*beta_2;

if isempty(Z3)==0;
   Psi = (N_03*inverse(N_33))-(alpha_2*beta_2'*N_23*inverse(N_33));
   epsilon = Zb0-(alpha_2*beta_2'*Z2)-(Psi*Z3);
else;
   Psi = [];
   epsilon = Zb0-(alpha_2*beta_2'*Z2);
end;

Omega = (1/T)*epsilon*epsilon';

if nargout>2;
   alpha1 = alpha_1;
   alpha2 = alpha_2;
   Psi_n = Psi;
   Omega_n = Omega;
   epsilon_n = epsilon;
   if isempty(beta_1)==0;
      if nr_z3>0;
         Psi_n = Psi(:,1:nr_z3);
      else;
         Psi_n = [];
      end;
      alpha1 = Psi(:,nr_z3+1:size(Psi,2));
   end;
end;

%
% end of varyrank.m
%
